import bpy

##################### Layers Schemes Operators ############################

######## Compact Scheme ########
class Operator_BlenRig_Layers_Scheme_Compact(bpy.types.Operator):    
    
    bl_idname = "blenrig5.layers_scheme_compact"   
    bl_label = "BlenRig Compact Layers Scheme"   
    bl_description = "Organize layers in a compact scheme relying on bone auto-hiding "    
    bl_options = {'REGISTER', 'UNDO', 'INTERNAL'}      

    @classmethod
    def poll(cls, context):
        if not bpy.context.object:
            return False
        else:
            return (bpy.context.object.type=='ARMATURE' and context.mode=='POSE')
        for prop in bpy.context.object.data.items():
            if prop[0] == 'rig_type' and prop[1] == 'Biped':              
                return True 

    def set_layers(self, context, N, L):

        pbones = bpy.context.active_object.pose.bones

        for b in pbones:
            layers =  L
            if b.name == N: 
                b.bone.layers = [(x in layers) for x in range(32)]

    def compact_layers(self, context):
        self.set_layers(context, 'master', [0, 25, 31])
        self.set_layers(context, 'master_pivot_point', [1, 25])
        self.set_layers(context, 'master_pivot', [26])
        self.set_layers(context, 'floor_L', [17, 25])
        self.set_layers(context, 'floor_R', [17, 25])
        self.set_layers(context, 'master_body', [17, 25])
        self.set_layers(context, 'master_body_pivot_point', [17, 25])
        self.set_layers(context, 'master_body_pivot', [26])
        self.set_layers(context, 'knee_pole_str_R', [31])
        self.set_layers(context, 'knee_str_R', [31])
        self.set_layers(context, 'knee_pole_str_L', [31])
        self.set_layers(context, 'knee_str_L', [31])
        self.set_layers(context, 'properties', [17])
        self.set_layers(context, 'properties_head', [17])
        self.set_layers(context, 'flex_neck_length', [26])
        self.set_layers(context, 'flex_neck_width', [26])
        self.set_layers(context, 'properties_torso', [17])
        self.set_layers(context, 'flex_chest_width', [26])
        self.set_layers(context, 'flex_waist_width', [26])
        self.set_layers(context, 'flex_pelvis_width', [26])
        self.set_layers(context, 'flex_ribs_width', [26])
        self.set_layers(context, 'properties_arm_L', [17])
        self.set_layers(context, 'flex_arm_width_L', [26])
        self.set_layers(context, 'flex_forearm_width_L', [26])
        self.set_layers(context, 'flex_arm_length_L', [26])
        self.set_layers(context, 'flex_forearm_length_L', [26])
        self.set_layers(context, 'flex_hand_scale_L', [26])
        self.set_layers(context, 'properties_arm_R', [17])
        self.set_layers(context, 'flex_arm_width_R', [26])
        self.set_layers(context, 'flex_forearm_width_R', [26])
        self.set_layers(context, 'flex_arm_length_R', [26])
        self.set_layers(context, 'flex_forearm_length_R', [26])
        self.set_layers(context, 'flex_hand_scale_R', [26])
        self.set_layers(context, 'properties_leg_R', [17])
        self.set_layers(context, 'flex_thigh_width_R', [26])
        self.set_layers(context, 'flex_thigh_length_R', [26])
        self.set_layers(context, 'flex_shin_length_R', [26])
        self.set_layers(context, 'flex_shin_width_R', [26])
        self.set_layers(context, 'flex_foot_scale_R', [26])
        self.set_layers(context, 'properties_leg_L', [17])
        self.set_layers(context, 'flex_thigh_width_L', [26])
        self.set_layers(context, 'flex_thigh_length_L', [26])
        self.set_layers(context, 'flex_shin_length_L', [26])
        self.set_layers(context, 'flex_shin_width_L', [26])
        self.set_layers(context, 'flex_foot_scale_L', [26])
        self.set_layers(context, 'master_torso', [0, 25, 31])
        self.set_layers(context, 'master_torso_pivot_point', [1, 25])
        self.set_layers(context, 'master_torso_pivot', [26])
        self.set_layers(context, 'spine_ctrl_4_str', [31])
        self.set_layers(context, 'shoulder_str_R', [31])
        self.set_layers(context, 'shoulder_str_L', [31])
        self.set_layers(context, 'clavi_str_R', [31])
        self.set_layers(context, 'clavi_str_L', [31])
        self.set_layers(context, 'neck_ctrl_4_str', [31])
        self.set_layers(context, 'head_str', [31])
        self.set_layers(context, 'look_str_loc', [31])
        self.set_layers(context, 'look_body', [26])
        self.set_layers(context, 'neck_ctrl_3_str', [31])
        self.set_layers(context, 'neck_ctrl_2_str', [31])
        self.set_layers(context, 'torso_ctrl_inv_str', [31])
        self.set_layers(context, 'spine_ctrl_3_str', [31])
        self.set_layers(context, 'torso_fk_ctrl', [0, 25, 30])
        self.set_layers(context, 'spine_ctrl_2_str', [31])
        self.set_layers(context, 'torso_ctrl_str', [31])
        self.set_layers(context, 'spine_ctrl_1_str', [31])
        self.set_layers(context, 'torso_ik_ctrl', [0, 25, 30])
        self.set_layers(context, 'wrist_str_R', [31])
        self.set_layers(context, 'hand_str_R', [31])
        self.set_layers(context, 'fing_lit_str_2_R', [24, 31])
        self.set_layers(context, 'fing_lit_str_3_R', [24, 31])
        self.set_layers(context, 'fing_lit_str_4_R', [24, 31])
        self.set_layers(context, 'fing_lit_str_5_R', [24, 31])
        self.set_layers(context, 'fing_ring_str_2_R', [24, 31])
        self.set_layers(context, 'fing_ring_str_3_R', [24, 31])
        self.set_layers(context, 'fing_ring_str_4_R', [24, 31])
        self.set_layers(context, 'fing_ring_str_5_R', [24, 31])
        self.set_layers(context, 'fing_ind_str_2_R', [24, 31])
        self.set_layers(context, 'fing_ind_str_3_R', [24, 31])
        self.set_layers(context, 'fing_ind_str_4_R', [24, 31])
        self.set_layers(context, 'fing_ind_str_5_R', [24, 31])
        self.set_layers(context, 'fing_mid_str_2_R', [24, 31])
        self.set_layers(context, 'fing_mid_str_3_R', [24, 31])
        self.set_layers(context, 'fing_mid_str_4_R', [24, 31])
        self.set_layers(context, 'fing_mid_str_5_R', [24, 31])
        self.set_layers(context, 'fing_thumb_str_2_R', [24, 31])
        self.set_layers(context, 'fing_thumb_str_3_R', [24, 31])
        self.set_layers(context, 'fing_thumb_str_4_R', [24, 31])
        self.set_layers(context, 'fing_thumb_str_1_R', [24, 31])
        self.set_layers(context, 'fing_lit_str_1_R', [24, 31])
        self.set_layers(context, 'fing_ring_str_1_R', [24, 31])
        self.set_layers(context, 'fing_ind_str_1_R', [24, 31])
        self.set_layers(context, 'fing_mid_str_1_R', [24, 31])
        self.set_layers(context, 'wrist_str_L', [31])
        self.set_layers(context, 'hand_str_L', [31])
        self.set_layers(context, 'fing_lit_str_2_L', [24, 31])
        self.set_layers(context, 'fing_lit_str_3_L', [24, 31])
        self.set_layers(context, 'fing_lit_str_4_L', [24, 31])
        self.set_layers(context, 'fing_lit_str_5_L', [24, 31])
        self.set_layers(context, 'fing_ring_str_2_L', [24, 31])
        self.set_layers(context, 'fing_ring_str_3_L', [24, 31])
        self.set_layers(context, 'fing_ring_str_4_L', [24, 31])
        self.set_layers(context, 'fing_ring_str_5_L', [24, 31])
        self.set_layers(context, 'fing_ind_str_2_L', [24, 31])
        self.set_layers(context, 'fing_ind_str_3_L', [24, 31])
        self.set_layers(context, 'fing_ind_str_4_L', [24, 31])
        self.set_layers(context, 'fing_ind_str_5_L', [24, 31])
        self.set_layers(context, 'fing_mid_str_2_L', [24, 31])
        self.set_layers(context, 'fing_mid_str_3_L', [24, 31])
        self.set_layers(context, 'fing_mid_str_4_L', [24, 31])
        self.set_layers(context, 'fing_mid_str_5_L', [24, 31])
        self.set_layers(context, 'fing_thumb_str_2_L', [24, 31])
        self.set_layers(context, 'fing_thumb_str_3_L', [24, 31])
        self.set_layers(context, 'fing_thumb_str_4_L', [24, 31])
        self.set_layers(context, 'fing_thumb_str_1_L', [24, 31])
        self.set_layers(context, 'fing_lit_str_1_L', [24, 31])
        self.set_layers(context, 'fing_ring_str_1_L', [24, 31])
        self.set_layers(context, 'fing_ind_str_1_L', [24, 31])
        self.set_layers(context, 'fing_mid_str_1_L', [24, 31])
        self.set_layers(context, 'elbow_pole_str_R', [31])
        self.set_layers(context, 'elbow_str_R', [31])
        self.set_layers(context, 'elbow_pole_str_L', [31])
        self.set_layers(context, 'elbow_str_L', [31])
        self.set_layers(context, 'spine_3_inv_ctrl', [0, 25, 30])
        self.set_layers(context, 'spine_3_inv', [0, 25, 30])
        self.set_layers(context, 'spine_4_ik_ctrl', [1, 25, 30])
        self.set_layers(context, 'spine_3_ik_ctrl', [0, 25, 30])
        self.set_layers(context, 'pelvis_str', [31])
        self.set_layers(context, 'pelvis_ctrl_str', [31])
        self.set_layers(context, 'torso_inv_ctrl', [0, 25, 30])
        self.set_layers(context, 'spine_2_inv', [0, 25, 30])
        self.set_layers(context, 'snap_spine_2_fk_inv', [30])
        self.set_layers(context, 'spine_1_inv', [0, 25, 30])
        self.set_layers(context, 'snap_spine_1_fk_inv', [30])
        self.set_layers(context, 'spine_2_ik_ctrl', [0, 25, 30])
        self.set_layers(context, 'pelvis_inv', [0, 25, 30])
        self.set_layers(context, 'snap_pelvis_ctrl_inv', [30])
        self.set_layers(context, 'pelvis_ctrl', [0, 25, 30])
        self.set_layers(context, 'pelvis_ik', [26])
        self.set_layers(context, 'pelvis_toon', [6, 25])
        self.set_layers(context, 'spine_1_ik_ctrl', [0, 30])
        self.set_layers(context, 'pelvis', [30, 31])
        self.set_layers(context, 'spine_1_mstr', [26])
        self.set_layers(context, 'look_torso', [26])
        self.set_layers(context, 'spine_3_inv_ik', [26, 30])
        self.set_layers(context, 'spine_2_inv_ik', [26, 30])
        self.set_layers(context, 'spine_1_ik', [26, 30])
        self.set_layers(context, 'spine_1_fk', [0, 25, 30])
        self.set_layers(context, 'snap_spine_1_inv_fk', [30])
        self.set_layers(context, 'spine_1_toon_mstr', [26])
        self.set_layers(context, 'spine_1_toon', [7, 25])
        self.set_layers(context, 'spine_1_def_bbone', [26])
        self.set_layers(context, 'spine_2_ik', [26, 30])
        self.set_layers(context, 'spine_3_ik', [26, 30])
        self.set_layers(context, 'spine_2_fk', [0, 25, 30])
        self.set_layers(context, 'snap_spine_2_inv_fk', [30])
        self.set_layers(context, 'spine_3_fk', [0, 25, 30])
        self.set_layers(context, 'snap_torso_ctrl_inv_loc', [30])
        self.set_layers(context, 'breath', [0])
        self.set_layers(context, 'snap_torso_fk_ctrl', [30])
        self.set_layers(context, 'spine_4_toon_mstr', [26])
        self.set_layers(context, 'spine_4_toon', [6, 25])
        self.set_layers(context, 'spine_3_toon_mstr', [26])
        self.set_layers(context, 'spine_3_toon', [7, 25])
        self.set_layers(context, 'spine_2_toon_mstr', [26])
        self.set_layers(context, 'spine_2_toon', [7, 25])
        self.set_layers(context, 'spine_1_def', [27, 29, 31])
        self.set_layers(context, 'spine_2_def', [27, 29, 31])
        self.set_layers(context, 'spine_3_collar', [26])
        self.set_layers(context, 'head_scale', [16, 25])
        self.set_layers(context, 'neck_base_ctrl', [26])
        self.set_layers(context, 'neck_base_hinge', [26])
        self.set_layers(context, 'neck_ik_ctrl', [1, 30])
        self.set_layers(context, 'neck_pivot_point', [1, 30])
        self.set_layers(context, 'neck_pivot', [26])
        self.set_layers(context, 'neck_fk_ctrl', [0, 25, 30])
        self.set_layers(context, 'head_ik_ctrl_mstr_hinge', [26])
        self.set_layers(context, 'head_ik_ctrl_mstr', [26])
        self.set_layers(context, 'head_ik_ctrl', [0, 25, 30])
        self.set_layers(context, 'neck_3_ik_ctrl', [0, 25, 30])
        self.set_layers(context, 'neck_2_ik_ctrl', [0, 25, 30])
        self.set_layers(context, 'neck_3_inv_ik', [26, 30])
        self.set_layers(context, 'neck_2_inv_ik', [26, 30])
        self.set_layers(context, 'neck_1_ik_ctrl', [0, 25, 30])
        self.set_layers(context, 'neck_1_ik', [26, 30])
        self.set_layers(context, 'neck_1_fk', [0, 25, 30])
        self.set_layers(context, 'neck_2_ik', [26, 30])
        self.set_layers(context, 'neck_2_fk', [0, 25, 30])
        self.set_layers(context, 'neck_2_toon_mstr', [26])
        self.set_layers(context, 'neck_2_toon', [7, 25])
        self.set_layers(context, 'neck_3_ik', [26, 30])
        self.set_layers(context, 'neck_3_fk', [0, 25, 30])
        self.set_layers(context, 'neck_3_toon_mstr', [26])
        self.set_layers(context, 'neck_3_toon', [7, 25])
        self.set_layers(context, 'neck_4_toon_mstr', [26])
        self.set_layers(context, 'neck_4_toon', [6, 25])
        self.set_layers(context, 'head_fk_mstr', [26, 30])
        self.set_layers(context, 'head_fk', [0, 25, 30])
        self.set_layers(context, 'head_stretch', [3, 25])
        self.set_layers(context, 'snap_head_ik', [30])
        self.set_layers(context, 'head_def_mstr', [27, 29, 31])
        self.set_layers(context, 'face_mstr_str', [31])
        self.set_layers(context, 'look_head', [26])
        self.set_layers(context, 'head_toon', [6, 25])
        self.set_layers(context, 'head_toon_mstr', [26])
        self.set_layers(context, 'face_toon_low', [6, 25])
        self.set_layers(context, 'head_mid_2_str', [31])
        self.set_layers(context, 'face_toon_mid', [7, 25])
        self.set_layers(context, 'face_toon_up', [6, 25])
        self.set_layers(context, 'head_mid_1_str', [31])
        self.set_layers(context, 'head_mid_ctrl', [3, 25])
        self.set_layers(context, 'head_top_ctrl', [3, 25])
        self.set_layers(context, 'head_mid_curve', [3, 25])
        self.set_layers(context, 'head_low_stretch', [5, 25])
        self.set_layers(context, 'head_mid_stretch', [5, 25])
        self.set_layers(context, 'head_def_3', [27, 31])
        self.set_layers(context, 'face_mstr_str_3', [26])
        self.set_layers(context, 'brow_mstr_str_L', [29, 31])
        self.set_layers(context, 'brow_ctrl_L', [3, 25])
        self.set_layers(context, 'toon_brow_L', [7, 25])
        self.set_layers(context, 'brow_mstr_vert_roll_L', [29])
        self.set_layers(context, 'brow_mstr_str_R', [29, 31])
        self.set_layers(context, 'brow_ctrl_R', [3, 25])
        self.set_layers(context, 'toon_brow_R', [7, 25])
        self.set_layers(context, 'brow_mstr_vert_roll_R', [29])
        self.set_layers(context, 'nose_frown_str_R', [31])
        self.set_layers(context, 'nose_frown_str_L', [31])
        self.set_layers(context, 'cheek2_lev_1_str_R', [31])
        self.set_layers(context, 'cheek2_lev_1_str_L', [31])
        self.set_layers(context, 'cheek2_lev_2_str_R', [31])
        self.set_layers(context, 'cheek2_lev_2_str_L', [31])
        self.set_layers(context, 'lev2_str_L', [31])
        self.set_layers(context, 'lev2_str_R', [31])
        self.set_layers(context, 'nose_up_str', [31])
        self.set_layers(context, 'nose_bridge_1_ctrl', [5, 25])
        self.set_layers(context, 'head_def_2', [27, 31])
        self.set_layers(context, 'ear_L', [3, 25, 27, 29, 31])
        self.set_layers(context, 'ear_up_L', [3, 25, 27, 29, 31])
        self.set_layers(context, 'ear_low_L', [3, 25, 27, 29, 31])
        self.set_layers(context, 'ear_R', [3, 25, 27, 29, 31])
        self.set_layers(context, 'ear_up_R', [3, 25, 27, 29, 31])
        self.set_layers(context, 'ear_low_R', [3, 25, 27, 29, 31])
        self.set_layers(context, 'face_mstr_str_2', [26])
        self.set_layers(context, 'nose_frown_ctrl_L', [3, 25, 31])
        self.set_layers(context, 'brow_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'brow_ctrl_1_L', [4, 25])
        self.set_layers(context, 'brow_up_1_def_L', [27, 31])
        self.set_layers(context, 'brow_low_1_bezier_out_L', [26])
        self.set_layers(context, 'brow_low_1_def_L', [27, 31])
        self.set_layers(context, 'brow_low_1_bbone_out_L', [26])
        self.set_layers(context, 'brow_up_1_bezier_out_L', [26])
        self.set_layers(context, 'brow_up_1_bbone_out_L', [26])
        self.set_layers(context, 'nose_frown_ctrl_R', [3, 25, 31])
        self.set_layers(context, 'brow_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'brow_ctrl_1_R', [4, 25])
        self.set_layers(context, 'brow_up_1_def_R', [27, 31])
        self.set_layers(context, 'brow_low_1_bezier_out_R', [26])
        self.set_layers(context, 'brow_low_1_def_R', [27, 31])
        self.set_layers(context, 'brow_low_1_bbone_out_R', [26])
        self.set_layers(context, 'brow_up_1_bezier_out_R', [26])
        self.set_layers(context, 'brow_up_1_bbone_out_R', [26])
        self.set_layers(context, 'frown_ctrl_mstr', [28, 31])
        self.set_layers(context, 'frown_ctrl', [4, 25])
        self.set_layers(context, 'mouth_str_loc_1', [31])
        self.set_layers(context, 'cheek_ctrl_2_str_R', [31])
        self.set_layers(context, 'cheek_ctrl_3_str_R', [31])
        self.set_layers(context, 'cheek_ctrl_2_str_L', [31])
        self.set_layers(context, 'cheek_ctrl_3_str_L', [31])
        self.set_layers(context, 'nose_low_str', [31])
        self.set_layers(context, 'nose_mstr', [26])
        self.set_layers(context, 'nose_bridge_2_ctrl', [4, 25, 31])
        self.set_layers(context, 'nose_bridge_1_rot', [26])
        self.set_layers(context, 'head_def_1', [27, 31])
        self.set_layers(context, 'face_mstr_str_1', [26])
        self.set_layers(context, 'uvula_1', [3])
        self.set_layers(context, 'uvula_2', [3])
        self.set_layers(context, 'maxi_str_2', [31])
        self.set_layers(context, 'maxi_str_1', [31])
        self.set_layers(context, 'maxi_str_mstr', [26, 31])
        self.set_layers(context, 'tongue_str', [31])
        self.set_layers(context, 'jaw_str_R', [31])
        self.set_layers(context, 'jaw_str_L', [31])
        self.set_layers(context, 'lev3_str_L', [31])
        self.set_layers(context, 'lev3_str_R', [31])
        self.set_layers(context, 'mouth_mstr_str', [31])
        self.set_layers(context, 'mouth_mstr_low_str', [29, 31])
        self.set_layers(context, 'mouth_mstr_up_str', [29, 31])
        self.set_layers(context, 'mouth_corner_str_L', [31])
        self.set_layers(context, 'mouth_corner_str_R', [31])
        self.set_layers(context, 'mouth_ctrl_str', [31])
        self.set_layers(context, 'mouth_str_loc_2', [31])
        self.set_layers(context, 'mouth_str_ctrl', [3, 25])
        self.set_layers(context, 'nose_ctrl', [3, 25, 27])
        self.set_layers(context, 'nostril_ctrl_L', [3, 25, 28, 31])
        self.set_layers(context, 'nostril_def_L', [27, 31])
        self.set_layers(context, 'nostril_ctrl_R', [3, 25, 28, 31])
        self.set_layers(context, 'nostril_def_R', [27, 31])
        self.set_layers(context, 'nose_bridge_2_rot', [26])
        self.set_layers(context, 'mouth_str', [26, 31])
        self.set_layers(context, 'mouth_ctrl', [3, 25])
        self.set_layers(context, 'mouth_up_ctrl', [3, 25])
        self.set_layers(context, 'mouth_low_ctrl', [3, 25])
        self.set_layers(context, 'maxi', [3, 25, 27])
        self.set_layers(context, 'mouth_frown_ctrl_L', [4, 25, 31])
        self.set_layers(context, 'mouth_frown_ctrl_R', [4, 25, 31])
        self.set_layers(context, 'chin_ctrl', [3, 28, 31])
        self.set_layers(context, 'chin_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'chin_ctrl_mid', [5, 25])
        self.set_layers(context, 'chin_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'chin_ctrl_1_R', [5, 25])
        self.set_layers(context, 'chin_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'chin_ctrl_1_L', [5, 25])
        self.set_layers(context, 'chin_1_rot_R', [26, 29])
        self.set_layers(context, 'chin_1_bbone_rot_L', [26])
        self.set_layers(context, 'chin_1_bbone_L', [26])
        self.set_layers(context, 'chin_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'chin_ctrl_2_R', [5, 25])
        self.set_layers(context, 'chin_2_rot_R', [26, 29])
        self.set_layers(context, 'chin_1_rot_L', [26, 29])
        self.set_layers(context, 'chin_1_bbone_rot_R', [26])
        self.set_layers(context, 'chin_1_def_L', [27, 31])
        self.set_layers(context, 'chin_1_bbone_R', [26])
        self.set_layers(context, 'chin_1_def_R', [27, 31])
        self.set_layers(context, 'chin_2_def_R', [27, 31])
        self.set_layers(context, 'chin_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'chin_ctrl_2_L', [5, 25])
        self.set_layers(context, 'chin_2_rot_L', [26, 29])
        self.set_layers(context, 'chin_2_def_L', [27, 31])
        self.set_layers(context, 'tongue_mstr', [3])
        self.set_layers(context, 'tongue_1_str', [31])
        self.set_layers(context, 'tongue_2_str', [31])
        self.set_layers(context, 'tongue_1_fk', [3])
        self.set_layers(context, 'tongue_3_str', [31])
        self.set_layers(context, 'tongue_2_fk', [3])
        self.set_layers(context, 'tongue_3_ik', [3])
        self.set_layers(context, 'tongue_2_ik', [3])
        self.set_layers(context, 'tongue_rot_3', [26])
        self.set_layers(context, 'tongue_rot_4', [26])
        self.set_layers(context, 'tongue_1_ik', [3])
        self.set_layers(context, 'tongue_rot_2', [26])
        self.set_layers(context, 'tongue_rot_1', [26])
        self.set_layers(context, 'tongue_1_def', [27, 31])
        self.set_layers(context, 'tongue_2_def', [27, 31])
        self.set_layers(context, 'tongue_3_def', [27, 31])
        self.set_layers(context, 'tongue_4', [26])
        self.set_layers(context, 'mouth_mstr_low_rot', [26])
        self.set_layers(context, 'mouth_mstr_maxi_loc', [26])
        self.set_layers(context, 'maxi_drv', [26])
        self.set_layers(context, 'mouth_mstr_copy_maxi', [26])
        self.set_layers(context, 'mouth_mstr_loc', [26])
        self.set_layers(context, 'mouth_mstr_ctrl', [3, 25])
        self.set_layers(context, 'mouth_mstr_ik', [26, 31])
        self.set_layers(context, 'mouth_mstr_ik_loc', [26])
        self.set_layers(context, 'mouth_mstr_ctrl_ik_loc', [26])
        self.set_layers(context, 'mouth_mstr_ctrl_loc', [26])
        self.set_layers(context, 'mouth_mstr_pivot_point', [26])
        self.set_layers(context, 'mouth_mstr_pivot', [26])
        self.set_layers(context, 'mouth_mstr', [26])
        self.set_layers(context, 'mouth_mstr_low_loc', [26])
        self.set_layers(context, 'mouth_mstr_low_copy_transforms', [26])
        self.set_layers(context, 'mouth_mstr_low_copy_transforms_positive', [26])
        self.set_layers(context, 'mouth_mstr_low_orient', [26])
        self.set_layers(context, 'mouth_mstr_low_orient_rot', [26])
        self.set_layers(context, 'mouth_mstr_up', [5, 25, 28])
        self.set_layers(context, 'lip_up_ctrl_collision', [3, 25, 28, 31])
        self.set_layers(context, 'lip_up_ctrl', [4, 25])
        self.set_layers(context, 'lip_up_loc', [26, 31])
        self.set_layers(context, 'lip_up2_ctrl', [4, 25, 28, 31])
        self.set_layers(context, 'lip_up3_ctrl', [4, 25, 28, 31])
        self.set_layers(context, 'lip_up3_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_mid', [5, 25])
        self.set_layers(context, 'mouth_corners_mstr_str_1', [26])
        self.set_layers(context, 'mouth_mstr_low_final', [26])
        self.set_layers(context, 'mouth_mstr_low_copy_transforms_final', [26])
        self.set_layers(context, 'mouth_mstr_low', [5, 25, 28])
        self.set_layers(context, 'lip_low_ctrl_collision', [3, 25, 28, 31])
        self.set_layers(context, 'lip_low_ctrl', [4, 25])
        self.set_layers(context, 'lip_low_loc', [26, 31])
        self.set_layers(context, 'lip_low2_ctrl', [4, 25, 28, 31])
        self.set_layers(context, 'lip_low3_ctrl', [4, 25, 31])
        self.set_layers(context, 'lip_low_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_mid', [4, 25])
        self.set_layers(context, 'lip_low_mstr_mid', [26, 29])
        self.set_layers(context, 'lip_low2_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_mid', [5, 25])
        self.set_layers(context, 'lip_low3_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_mid', [5, 25])
        self.set_layers(context, 'oris_low_3_def_mid', [27, 31])
        self.set_layers(context, 'oris_low_2_def_mid', [27, 31])
        self.set_layers(context, 'oris_low_1_def_mid', [27, 31])
        self.set_layers(context, 'lip_up_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_mid', [4, 25])
        self.set_layers(context, 'mouth_corner_maxi_loc_L', [26])
        self.set_layers(context, 'mouth_corner_copy_maxi_L', [26])
        self.set_layers(context, 'mouth_corner_maxi_loc_R', [26])
        self.set_layers(context, 'mouth_corner_copy_maxi_R', [26])
        self.set_layers(context, 'mouth_corners_mstr_str_2', [26])
        self.set_layers(context, 'mouth_corners_mstr_L', [26])
        self.set_layers(context, 'mouth_corner_loc_L', [26])
        self.set_layers(context, 'mouth_corner_L', [3, 25, 31])
        self.set_layers(context, 'mouth_corner_shp_at_L', [26])
        self.set_layers(context, 'cheek_ctrl_L', [3, 25, 29, 31])
        self.set_layers(context, 'brow_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'brow_ctrl_3_L', [4, 25])
        self.set_layers(context, 'brow_up_3_def_L', [27, 31])
        self.set_layers(context, 'brow_low_3_bezier_out_L', [26])
        self.set_layers(context, 'brow_low_3_def_L', [27, 31])
        self.set_layers(context, 'brow_low_3_bbone_out_L', [26])
        self.set_layers(context, 'brow_up_3_bezier_out_L', [26])
        self.set_layers(context, 'brow_up_3_bbone_out_L', [26])
        self.set_layers(context, 'brow_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'brow_ctrl_2_L', [4, 25])
        self.set_layers(context, 'brow_1_bezier_in_mstr_L', [26])
        self.set_layers(context, 'brow_1_bezier_in_L', [26])
        self.set_layers(context, 'brow_1_bbone_in_L', [26])
        self.set_layers(context, 'brow_3_bezier_out_mstr_L', [26])
        self.set_layers(context, 'brow_3_bezier_out_L', [26])
        self.set_layers(context, 'brow_2_bezier_out_mstr_L', [26])
        self.set_layers(context, 'brow_2_bezier_out_L', [26])
        self.set_layers(context, 'brow_up_2_def_L', [27, 31])
        self.set_layers(context, 'brow_low_2_bezier_out_L', [26])
        self.set_layers(context, 'brow_low_2_def_L', [27, 31])
        self.set_layers(context, 'brow_low_2_bbone_out_L', [26])
        self.set_layers(context, 'brow_up_2_bezier_out_L', [26])
        self.set_layers(context, 'brow_up_2_bbone_out_L', [26])
        self.set_layers(context, 'brow_2_rot_L', [26, 29])
        self.set_layers(context, 'brow_1_rot_L', [26, 29])
        self.set_layers(context, 'brow_1_def_L', [27, 31])
        self.set_layers(context, 'brow_1_bbone_out_L', [26, 31])
        self.set_layers(context, 'brow_ctrl_4_mstr_L', [28, 31])
        self.set_layers(context, 'brow_ctrl_4_L', [4, 25])
        self.set_layers(context, 'brow_2_bezier_in_mstr_L', [26])
        self.set_layers(context, 'brow_2_bezier_in_L', [26])
        self.set_layers(context, 'brow_2_bbone_in_L', [26])
        self.set_layers(context, 'brow_2_def_L', [27, 31])
        self.set_layers(context, 'brow_2_bbone_out_L', [26])
        self.set_layers(context, 'brow_up_4_def_L', [27, 31])
        self.set_layers(context, 'brow_low_4_bezier_out_L', [26])
        self.set_layers(context, 'brow_low_4_def_L', [27, 31])
        self.set_layers(context, 'brow_low_4_bbone_out_L', [26])
        self.set_layers(context, 'brow_up_4_bezier_out_L', [26])
        self.set_layers(context, 'brow_up_4_bbone_out_L', [26])
        self.set_layers(context, 'brow_3_rot_L', [26, 29])
        self.set_layers(context, 'brow_3_bezier_in_mstr_L', [26, 31])
        self.set_layers(context, 'brow_3_bezier_in_L', [26])
        self.set_layers(context, 'brow_3_bbone_in_L', [26])
        self.set_layers(context, 'brow_3_def_L', [27, 31])
        self.set_layers(context, 'brow_3_bbone_out_L', [26])
        self.set_layers(context, 'cheek_ctrl_drv_L', [26])
        self.set_layers(context, 'cheek_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'cheek_ctrl_1_L', [5, 25])
        self.set_layers(context, 'nose_frown_3_def_L', [27, 31])
        self.set_layers(context, 'cheek_ctrl_2_mstr_L', [28])
        self.set_layers(context, 'cheek_ctrl_2_L', [5, 25])
        self.set_layers(context, 'lev1_3_def_L', [27, 31])
        self.set_layers(context, 'cheek_1_rot_L', [26, 29])
        self.set_layers(context, 'cheek_1_def_L', [27, 31])
        self.set_layers(context, 'cheek_ctrl_3_mstr_L', [28])
        self.set_layers(context, 'cheek_ctrl_3_L', [5, 25])
        self.set_layers(context, 'lev2_3_def_L', [27, 31])
        self.set_layers(context, 'lev2_4_def_L', [27, 31])
        self.set_layers(context, 'cheek_2_rot_L', [26, 29])
        self.set_layers(context, 'cheek_2_def_L', [27, 31])
        self.set_layers(context, 'mouth_corner_distance_drv_L', [26])
        self.set_layers(context, 'mouth_corner_drv_L', [26])
        self.set_layers(context, 'chin_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'chin_ctrl_3_L', [5, 25])
        self.set_layers(context, 'chin_3_rot_L', [26, 29])
        self.set_layers(context, 'chin_3_def_L', [27, 31])
        self.set_layers(context, 'cheek_ctrl_5_mstr_L', [28, 31])
        self.set_layers(context, 'cheek_ctrl_5_L', [5, 25])
        self.set_layers(context, 'jaw_1_rot_L', [26, 29])
        self.set_layers(context, 'jaw_1_def_L', [27, 31])
        self.set_layers(context, 'jaw_2_rot_L', [26, 29])
        self.set_layers(context, 'jaw_2_def_L', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_1_L', [4, 25])
        self.set_layers(context, 'lip_low_1_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_low_1_bezier_in_R', [26])
        self.set_layers(context, 'lip_low_1_bbone_in_R', [26])
        self.set_layers(context, 'lip_low_2_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_low_2_bezier_in_L', [26])
        self.set_layers(context, 'lip_low_2_bbone_in_L', [26])
        self.set_layers(context, 'lip_low_mstr_1_L', [26, 29])
        self.set_layers(context, 'lip_low_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_low_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_low2_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_1_L', [5, 25])
        self.set_layers(context, 'lip_low2_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_low2_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_low2_1_bbone_R', [26])
        self.set_layers(context, 'lip_low3_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_1_L', [5, 25])
        self.set_layers(context, 'lip_low3_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_low3_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_low3_1_bbone_R', [26])
        self.set_layers(context, 'oris_low1_3_def_L', [27, 31])
        self.set_layers(context, 'oris_low1_2_def_L', [27, 31])
        self.set_layers(context, 'oris_low1_1_def_L', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_2_L', [4, 25])
        self.set_layers(context, 'lip_low_1_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_low_1_bezier_out_L', [26])
        self.set_layers(context, 'lip_low_3_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_low_3_bezier_in_L', [26])
        self.set_layers(context, 'lip_low_3_bbone_in_L', [26])
        self.set_layers(context, 'lip_low_mstr_2_L', [26, 29])
        self.set_layers(context, 'lip_low_2_rot_L', [26, 29])
        self.set_layers(context, 'lip_low_2_def_L', [27, 31])
        self.set_layers(context, 'lip_low2_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_2_L', [5, 25])
        self.set_layers(context, 'lip_low2_2_rot_L', [26, 29])
        self.set_layers(context, 'lip_low3_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_2_L', [5, 25])
        self.set_layers(context, 'lip_low3_2_rot_L', [26, 29])
        self.set_layers(context, 'oris_low2_3_def_L', [27, 31])
        self.set_layers(context, 'oris_low2_2_def_L', [27, 31])
        self.set_layers(context, 'oris_low2_1_def_L', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_3_L', [4, 25])
        self.set_layers(context, 'lip_low_2_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_low_2_bezier_out_L', [26])
        self.set_layers(context, 'lip_low_4_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_low_4_bezier_in_L', [26])
        self.set_layers(context, 'lip_low_4_bbone_in_L', [26])
        self.set_layers(context, 'lip_low_3_rot_L', [26, 29])
        self.set_layers(context, 'lip_low_2_bbone_out_L', [26])
        self.set_layers(context, 'lip_low_3_def_L', [27, 31])
        self.set_layers(context, 'lip_low_mstr_3_L', [26, 29])
        self.set_layers(context, 'lip_low2_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_3_L', [5, 25])
        self.set_layers(context, 'lip_low2_3_rot_L', [26, 29])
        self.set_layers(context, 'lip_low3_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_3_L', [5, 25])
        self.set_layers(context, 'lip_low3_3_rot_L', [26, 29])
        self.set_layers(context, 'oris_low3_3_def_L', [27, 31])
        self.set_layers(context, 'oris_low3_2_def_L', [27, 31])
        self.set_layers(context, 'oris_low3_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_1_L', [4, 25])
        self.set_layers(context, 'lip_up_1_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_up_1_bezier_in_R', [26])
        self.set_layers(context, 'lip_up_1_bbone_in_R', [26])
        self.set_layers(context, 'lip_up_2_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_up_2_bezier_in_L', [26])
        self.set_layers(context, 'lip_up_2_bbone_in_L', [26])
        self.set_layers(context, 'lip_up_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_2_L', [4, 25])
        self.set_layers(context, 'lip_up_1_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_up_3_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_up_3_bezier_in_L', [26])
        self.set_layers(context, 'lip_up_3_bbone_in_L', [26])
        self.set_layers(context, 'lip_up_1_bezier_out_L', [26])
        self.set_layers(context, 'lip_up_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_3_L', [4, 25])
        self.set_layers(context, 'lip_up_4_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_up_4_bezier_in_L', [26])
        self.set_layers(context, 'lip_up_4_bbone_in_L', [26])
        self.set_layers(context, 'lip_up_2_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_up_2_bezier_out_L', [26])
        self.set_layers(context, 'lip_up3_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_1_L', [5, 25])
        self.set_layers(context, 'cheek_mstr_1_L', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_1_L', [5, 25])
        self.set_layers(context, 'nose_frown_2_def_L', [27, 31])
        self.set_layers(context, 'nose_frown_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_ctrl_4_mstr_L', [28])
        self.set_layers(context, 'lip_up_ctrl_4_L', [4, 25])
        self.set_layers(context, 'lip_low_3_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_low_3_bezier_out_L', [26])
        self.set_layers(context, 'lip_low_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_low_3_bbone_out_L', [26])
        self.set_layers(context, 'lip_low_4_def_L', [27, 31])
        self.set_layers(context, 'lip_low_4_bbone_out_L', [26])
        self.set_layers(context, 'lip_up_3_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_up_3_bezier_out_L', [26])
        self.set_layers(context, 'lip_up_ctrl_4_loc_L', [26])
        self.set_layers(context, 'lip_up_mstr_4_str_L', [31])
        self.set_layers(context, 'mouth_corner_loc_R', [26])
        self.set_layers(context, 'mouth_corner_R', [3, 25, 31])
        self.set_layers(context, 'mouth_corner_shp_at_R', [26])
        self.set_layers(context, 'cheek_ctrl_R', [3, 25, 29, 31])
        self.set_layers(context, 'brow_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'brow_ctrl_3_R', [4, 25])
        self.set_layers(context, 'brow_up_3_def_R', [27, 31])
        self.set_layers(context, 'brow_low_3_bezier_out_R', [26])
        self.set_layers(context, 'brow_low_3_def_R', [27, 31])
        self.set_layers(context, 'brow_low_3_bbone_out_R', [26])
        self.set_layers(context, 'brow_up_3_bezier_out_R', [26])
        self.set_layers(context, 'brow_up_3_bbone_out_R', [26])
        self.set_layers(context, 'brow_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'brow_ctrl_2_R', [4, 25])
        self.set_layers(context, 'brow_1_bezier_in_mstr_R', [26])
        self.set_layers(context, 'brow_1_bezier_in_R', [26])
        self.set_layers(context, 'brow_1_bbone_in_R', [26])
        self.set_layers(context, 'brow_3_bezier_out_mstr_R', [26])
        self.set_layers(context, 'brow_3_bezier_out_R', [26])
        self.set_layers(context, 'brow_2_bezier_out_mstr_R', [26])
        self.set_layers(context, 'brow_2_bezier_out_R', [26])
        self.set_layers(context, 'brow_up_2_def_R', [27, 31])
        self.set_layers(context, 'brow_low_2_bezier_out_R', [26])
        self.set_layers(context, 'brow_low_2_def_R', [27, 31])
        self.set_layers(context, 'brow_low_2_bbone_out_R', [26])
        self.set_layers(context, 'brow_up_2_bezier_out_R', [26])
        self.set_layers(context, 'brow_up_2_bbone_out_R', [26])
        self.set_layers(context, 'brow_2_rot_R', [26, 29])
        self.set_layers(context, 'brow_1_rot_R', [26, 29])
        self.set_layers(context, 'brow_1_def_R', [27, 31])
        self.set_layers(context, 'brow_1_bbone_out_R', [26, 31])
        self.set_layers(context, 'brow_ctrl_4_mstr_R', [28, 31])
        self.set_layers(context, 'brow_ctrl_4_R', [4, 25])
        self.set_layers(context, 'brow_2_bezier_in_mstr_R', [26])
        self.set_layers(context, 'brow_2_bezier_in_R', [26])
        self.set_layers(context, 'brow_2_bbone_in_R', [26])
        self.set_layers(context, 'brow_2_def_R', [27, 31])
        self.set_layers(context, 'brow_2_bbone_out_R', [26])
        self.set_layers(context, 'brow_up_4_def_R', [27, 31])
        self.set_layers(context, 'brow_low_4_bezier_out_R', [26])
        self.set_layers(context, 'brow_low_4_def_R', [27, 31])
        self.set_layers(context, 'brow_low_4_bbone_out_R', [26])
        self.set_layers(context, 'brow_up_4_bezier_out_R', [26])
        self.set_layers(context, 'brow_up_4_bbone_out_R', [26])
        self.set_layers(context, 'brow_3_rot_R', [26, 29])
        self.set_layers(context, 'brow_3_bezier_in_mstr_R', [26, 31])
        self.set_layers(context, 'brow_3_bezier_in_R', [26])
        self.set_layers(context, 'brow_3_bbone_in_R', [26])
        self.set_layers(context, 'brow_3_def_R', [27, 31])
        self.set_layers(context, 'brow_3_bbone_out_R', [26])
        self.set_layers(context, 'cheek_ctrl_drv_R', [26])
        self.set_layers(context, 'cheek_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'cheek_ctrl_1_R', [5, 25])
        self.set_layers(context, 'nose_frown_3_def_R', [27, 31])
        self.set_layers(context, 'cheek_ctrl_2_mstr_R', [28])
        self.set_layers(context, 'cheek_ctrl_2_R', [5, 25])
        self.set_layers(context, 'lev1_3_def_R', [27, 31])
        self.set_layers(context, 'cheek_1_rot_R', [26, 29])
        self.set_layers(context, 'cheek_1_def_R', [27, 31])
        self.set_layers(context, 'cheek_ctrl_3_mstr_R', [28])
        self.set_layers(context, 'cheek_ctrl_3_R', [5, 25])
        self.set_layers(context, 'lev2_3_def_R', [27, 31])
        self.set_layers(context, 'lev2_4_def_R', [27, 31])
        self.set_layers(context, 'cheek_2_rot_R', [26, 29])
        self.set_layers(context, 'cheek_2_def_R', [27, 31])
        self.set_layers(context, 'mouth_corner_distance_drv_R', [26])
        self.set_layers(context, 'mouth_corner_drv_R', [26])
        self.set_layers(context, 'chin_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'chin_ctrl_3_R', [5, 25])
        self.set_layers(context, 'chin_3_rot_R', [26, 29])
        self.set_layers(context, 'chin_3_def_R', [27, 31])
        self.set_layers(context, 'cheek_ctrl_5_mstr_R', [28, 31])
        self.set_layers(context, 'cheek_ctrl_5_R', [5, 25])
        self.set_layers(context, 'jaw_1_rot_R', [26, 29])
        self.set_layers(context, 'jaw_1_def_R', [27, 31])
        self.set_layers(context, 'jaw_2_rot_R', [26, 29])
        self.set_layers(context, 'jaw_2_def_R', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_2_R', [4, 25])
        self.set_layers(context, 'lip_low_mstr_2_R', [26, 29])
        self.set_layers(context, 'lip_low2_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_2_R', [5, 25])
        self.set_layers(context, 'lip_low3_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_2_R', [5, 25])
        self.set_layers(context, 'oris_low2_3_def_R', [27, 31])
        self.set_layers(context, 'oris_low2_2_def_R', [27, 31])
        self.set_layers(context, 'oris_low2_1_def_R', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_1_R', [4, 25])
        self.set_layers(context, 'lip_low_1_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_low_2_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_low_2_bezier_in_R', [26])
        self.set_layers(context, 'lip_low_2_bbone_in_R', [26])
        self.set_layers(context, 'lip_low_1_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_low_1_bezier_out_R', [26])
        self.set_layers(context, 'lip_low_3_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_low_3_bezier_in_R', [26])
        self.set_layers(context, 'lip_low_3_bbone_in_R', [26])
        self.set_layers(context, 'lip_low_1_bezier_in_L', [26])
        self.set_layers(context, 'lip_low_1_bbone_in_L', [26])
        self.set_layers(context, 'lip_low_1_def_L', [27, 31])
        self.set_layers(context, 'lip_low_1_bbone_out_L', [26])
        self.set_layers(context, 'lip_low_mstr_1_R', [26, 29])
        self.set_layers(context, 'lip_low_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_low_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_low_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_low_2_def_R', [27, 31])
        self.set_layers(context, 'lip_low_1_def_R', [27, 31])
        self.set_layers(context, 'lip_low_1_bbone_out_R', [26])
        self.set_layers(context, 'lip_low2_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_1_R', [5, 25])
        self.set_layers(context, 'lip_low2_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_low2_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_low2_1_bbone_L', [26])
        self.set_layers(context, 'lip_low2_1_def_L', [27, 31])
        self.set_layers(context, 'lip_low2_2_def_L', [27, 31])
        self.set_layers(context, 'lip_low2_3_def_L', [27, 31])
        self.set_layers(context, 'lip_low2_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_low2_1_def_R', [27, 31])
        self.set_layers(context, 'lip_low2_2_def_R', [27, 31])
        self.set_layers(context, 'lip_low3_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_1_R', [5, 25])
        self.set_layers(context, 'lip_low3_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_low3_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_low3_1_bbone_L', [26])
        self.set_layers(context, 'lip_low3_1_def_L', [27, 31])
        self.set_layers(context, 'lip_low3_2_def_L', [27, 31])
        self.set_layers(context, 'lip_low3_3_def_L', [27, 31])
        self.set_layers(context, 'lip_low3_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_low3_1_def_R', [27, 31])
        self.set_layers(context, 'lip_low3_2_def_R', [27, 31])
        self.set_layers(context, 'oris_low1_3_def_R', [27, 31])
        self.set_layers(context, 'oris_low1_2_def_R', [27, 31])
        self.set_layers(context, 'oris_low1_1_def_R', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_3_R', [4, 25])
        self.set_layers(context, 'lip_low_2_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_low_2_bezier_out_R', [26])
        self.set_layers(context, 'lip_low_4_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_low_4_bezier_in_R', [26])
        self.set_layers(context, 'lip_low_4_bbone_in_R', [26])
        self.set_layers(context, 'lip_low_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_low_2_bbone_out_R', [26])
        self.set_layers(context, 'lip_low_3_def_R', [27, 31])
        self.set_layers(context, 'lip_low_mstr_3_R', [26, 29])
        self.set_layers(context, 'lip_low2_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_3_R', [5, 25])
        self.set_layers(context, 'lip_low2_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_low2_3_def_R', [27, 31])
        self.set_layers(context, 'lip_low3_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_3_R', [5, 25])
        self.set_layers(context, 'lip_low3_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_low3_3_def_R', [27, 31])
        self.set_layers(context, 'oris_low3_3_def_R', [27, 31])
        self.set_layers(context, 'oris_low3_2_def_R', [27, 31])
        self.set_layers(context, 'oris_low3_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_2_R', [4, 25])
        self.set_layers(context, 'lip_up_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_1_R', [4, 25])
        self.set_layers(context, 'lip_up_1_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_up_1_bezier_out_R', [26])
        self.set_layers(context, 'lip_up_3_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_up_3_bezier_in_R', [26])
        self.set_layers(context, 'lip_up_3_bbone_in_R', [26])
        self.set_layers(context, 'lip_up_2_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_up_2_bezier_in_R', [26])
        self.set_layers(context, 'lip_up_2_bbone_in_R', [26])
        self.set_layers(context, 'lip_up_1_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_up_1_bezier_in_L', [26])
        self.set_layers(context, 'lip_up_1_bbone_in_L', [26])
        self.set_layers(context, 'lip_up_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_3_R', [4, 25])
        self.set_layers(context, 'lip_up_2_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_up_2_bezier_out_R', [26])
        self.set_layers(context, 'lip_up_4_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_up_4_bezier_in_R', [26])
        self.set_layers(context, 'lip_up_4_bbone_in_R', [26])
        self.set_layers(context, 'lip_up3_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_1_R', [5, 25])
        self.set_layers(context, 'cheek_mstr_1_R', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_1_R', [5, 25])
        self.set_layers(context, 'nose_frown_2_def_R', [27, 31])
        self.set_layers(context, 'nose_frown_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up_ctrl_4_mstr_R', [28])
        self.set_layers(context, 'lip_up_ctrl_4_R', [4, 25])
        self.set_layers(context, 'lip_low_3_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_low_3_bezier_out_R', [26])
        self.set_layers(context, 'lip_low_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_low_3_bbone_out_R', [26])
        self.set_layers(context, 'lip_low_4_def_R', [27, 31])
        self.set_layers(context, 'lip_low_4_bbone_out_R', [26])
        self.set_layers(context, 'lip_up_3_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_up_3_bezier_out_R', [26])
        self.set_layers(context, 'lip_up_ctrl_4_loc_R', [26])
        self.set_layers(context, 'lip_up_mstr_4_str_R', [31])
        self.set_layers(context, 'lip_up_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_up_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_2_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_2_def_L', [27, 31])
        self.set_layers(context, 'lip_up_1_bbone_out_L', [26])
        self.set_layers(context, 'lip_up_3_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_2_bbone_out_L', [26])
        self.set_layers(context, 'lip_up_3_def_L', [27, 31])
        self.set_layers(context, 'lip_up3_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_up3_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_up3_1_bbone_R', [26])
        self.set_layers(context, 'lip_up_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_3_bbone_out_L', [26])
        self.set_layers(context, 'lip_up_4_def_L', [27, 31])
        self.set_layers(context, 'lip_up_4_bbone_out_L', [26])
        self.set_layers(context, 'lip_up_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_up_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_1_bbone_out_R', [26])
        self.set_layers(context, 'lip_up_2_def_R', [27, 31])
        self.set_layers(context, 'lip_up_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_2_bbone_out_R', [26])
        self.set_layers(context, 'lip_up_3_def_R', [27, 31])
        self.set_layers(context, 'lip_up3_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_up3_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_up3_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up3_1_bbone_L', [26])
        self.set_layers(context, 'lip_up3_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_3_bbone_out_R', [26])
        self.set_layers(context, 'lip_up_4_def_R', [27, 31])
        self.set_layers(context, 'lip_up_4_bbone_out_R', [26])
        self.set_layers(context, 'mouth_corners_mstr_R', [26])
        self.set_layers(context, 'lattice_mouth', [26])
        self.set_layers(context, 'teeth_up_str', [31])
        self.set_layers(context, 'teeth_up', [4, 25])
        self.set_layers(context, 'teeth_up_ctrl_str_R', [31])
        self.set_layers(context, 'teeth_up_ctrl_str_L', [31])
        self.set_layers(context, 'teeth_up_L', [26])
        self.set_layers(context, 'teeth_up_ctrl_L', [4, 25])
        self.set_layers(context, 'teeth_up_ctrl_mid_loc_L', [26])
        self.set_layers(context, 'teeth_up_R', [26])
        self.set_layers(context, 'teeth_up_ctrl_R', [4, 25])
        self.set_layers(context, 'teeth_up_ctrl_mid_loc_R', [26])
        self.set_layers(context, 'teeth_up_ctrl_mid_str', [25, 31])
        self.set_layers(context, 'teeth_up_ctrl_mid', [4, 25])
        self.set_layers(context, 'teeth_up_ctrl_mid_str_L', [25, 31])
        self.set_layers(context, 'teeth_up_ctrl_mid_L', [4, 25])
        self.set_layers(context, 'teeth_up_bbone_R', [26])
        self.set_layers(context, 'teeth_up_rot_1_L', [26])
        self.set_layers(context, 'teeth_up_rot_2_L', [26])
        self.set_layers(context, 'teeth_up_ctrl_mid_str_R', [25, 31])
        self.set_layers(context, 'teeth_up_ctrl_mid_R', [4, 25])
        self.set_layers(context, 'teeth_up_rot_2_R', [26])
        self.set_layers(context, 'teeth_up_bbone_L', [26])
        self.set_layers(context, 'teeth_up_1_def_L', [27, 31])
        self.set_layers(context, 'teeth_up_2_def_L', [27, 31])
        self.set_layers(context, 'teeth_up_rot_1_R', [26])
        self.set_layers(context, 'teeth_up_1_def_R', [27, 31])
        self.set_layers(context, 'teeth_up_2_def_R', [27, 31])
        self.set_layers(context, 'teeth_low_str', [31])
        self.set_layers(context, 'teeth_low', [4, 25])
        self.set_layers(context, 'teeth_low_ctrl_str_R', [31])
        self.set_layers(context, 'teeth_low_ctrl_str_L', [31])
        self.set_layers(context, 'teeth_low_L', [26])
        self.set_layers(context, 'teeth_low_ctrl_L', [4, 25])
        self.set_layers(context, 'teeth_low_ctrl_mid_loc_L', [26])
        self.set_layers(context, 'teeth_low_R', [26])
        self.set_layers(context, 'teeth_low_ctrl_R', [4, 25])
        self.set_layers(context, 'teeth_low_ctrl_mid_loc_R', [26])
        self.set_layers(context, 'teeth_low_ctrl_mid_str', [25, 31])
        self.set_layers(context, 'teeth_low_ctrl_mid', [4, 25])
        self.set_layers(context, 'teeth_low_ctrl_mid_str_L', [25, 31])
        self.set_layers(context, 'teeth_low_ctrl_mid_L', [4, 25])
        self.set_layers(context, 'teeth_low_bbone_R', [26])
        self.set_layers(context, 'teeth_low_rot_1_L', [26])
        self.set_layers(context, 'teeth_low_rot_2_L', [26])
        self.set_layers(context, 'teeth_low_ctrl_mid_str_R', [25, 31])
        self.set_layers(context, 'teeth_low_ctrl_mid_R', [4, 25])
        self.set_layers(context, 'teeth_low_rot_2_R', [26])
        self.set_layers(context, 'teeth_low_bbone_L', [26])
        self.set_layers(context, 'teeth_low_1_def_L', [27, 31])
        self.set_layers(context, 'teeth_low_2_def_L', [27, 31])
        self.set_layers(context, 'teeth_low_rot_1_R', [26])
        self.set_layers(context, 'teeth_low_1_def_R', [27, 31])
        self.set_layers(context, 'teeth_low_2_def_R', [27, 31])
        self.set_layers(context, 'lip_up_mstr_mid', [26, 29])
        self.set_layers(context, 'lip_up2_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_mid', [5, 25])
        self.set_layers(context, 'oris_up_2_def_mid', [27, 31])
        self.set_layers(context, 'oris_up_1_def_mid', [27, 31])
        self.set_layers(context, 'cheek_mstr_4_L', [26, 29])
        self.set_layers(context, 'cheek_ctrl_4_mstr_L', [28, 29, 31])
        self.set_layers(context, 'cheek_ctrl_4_L', [5, 25])
        self.set_layers(context, 'cheek_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_mstr_4_L', [26, 29])
        self.set_layers(context, 'lip_up2_ctrl_4_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_4_L', [5, 25])
        self.set_layers(context, 'lip_low2_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_low2_4_def_L', [27, 31])
        self.set_layers(context, 'lip_up3_ctrl_4_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_4_L', [5, 25])
        self.set_layers(context, 'lip_low3_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_low3_4_def_L', [27, 31])
        self.set_layers(context, 'cheek2_ctrl_4_mstr_L', [28, 29, 31])
        self.set_layers(context, 'cheek2_ctrl_4_L', [5, 25])
        self.set_layers(context, 'cheek2_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_mstr_2_L', [26, 29])
        self.set_layers(context, 'lip_up3_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_2_L', [5, 25])
        self.set_layers(context, 'lip_up3_2_rot_L', [26, 29])
        self.set_layers(context, 'cheek_mstr_2_L', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_2_L', [5, 25])
        self.set_layers(context, 'cheek2_1_rot_L', [26, 29])
        self.set_layers(context, 'cheek2_1_def_L', [27, 31])
        self.set_layers(context, 'lev1_2_def_L', [27, 31])
        self.set_layers(context, 'lev1_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up3_2_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_2_L', [5, 25])
        self.set_layers(context, 'oris_up2_2_def_L', [27, 31])
        self.set_layers(context, 'oris_up2_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_mstr_3_L', [26, 29])
        self.set_layers(context, 'lip_up3_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_3_L', [5, 25])
        self.set_layers(context, 'lip_up3_3_rot_L', [26, 29])
        self.set_layers(context, 'cheek_mstr_3_L', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_3_L', [5, 25])
        self.set_layers(context, 'cheek2_2_rot_L', [26, 29])
        self.set_layers(context, 'cheek2_2_def_L', [27, 31])
        self.set_layers(context, 'lip_up3_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_up3_3_def_L', [27, 31])
        self.set_layers(context, 'lip_up3_4_def_L', [27, 31])
        self.set_layers(context, 'lev2_2_def_L', [27, 31])
        self.set_layers(context, 'lev2_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_3_L', [5, 25])
        self.set_layers(context, 'lip_up2_3_rot_L', [26, 29])
        self.set_layers(context, 'lip_up2_4_rot_L', [26, 29])
        self.set_layers(context, 'oris_up3_2_def_L', [27, 31])
        self.set_layers(context, 'oris_up3_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_mstr_1_L', [26, 29])
        self.set_layers(context, 'lip_up2_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_1_L', [5, 25])
        self.set_layers(context, 'lip_up2_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_up2_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_up2_1_bbone_R', [26])
        self.set_layers(context, 'lip_up2_2_rot_L', [26, 29])
        self.set_layers(context, 'oris_up1_2_def_L', [27, 31])
        self.set_layers(context, 'oris_up1_1_def_L', [27, 31])
        self.set_layers(context, 'lev3_3_def_L', [27, 31])
        self.set_layers(context, 'lev3_2_def_L', [27, 31])
        self.set_layers(context, 'lev3_1_def_L', [27, 31])
        self.set_layers(context, 'oris_up4_2_def_L', [27, 31])
        self.set_layers(context, 'oris_up4_1_def_L', [27, 31])
        self.set_layers(context, 'cheek_3_rot_L', [26, 29])
        self.set_layers(context, 'cheek_3_def_L', [27, 31])
        self.set_layers(context, 'cheek_4_def_L', [27, 31])
        self.set_layers(context, 'cheek2_3_rot_L', [26, 29])
        self.set_layers(context, 'cheek2_3_def_L', [27, 31])
        self.set_layers(context, 'cheek2_4_def_L', [27, 31])
        self.set_layers(context, 'cheek_mstr_4_R', [26, 29])
        self.set_layers(context, 'cheek_ctrl_4_mstr_R', [28, 29, 31])
        self.set_layers(context, 'cheek_ctrl_4_R', [5, 25])
        self.set_layers(context, 'cheek_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_mstr_4_R', [26, 29])
        self.set_layers(context, 'lip_up2_ctrl_4_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_4_R', [5, 25])
        self.set_layers(context, 'lip_low2_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_low2_4_def_R', [27, 31])
        self.set_layers(context, 'lip_up3_ctrl_4_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_4_R', [5, 25])
        self.set_layers(context, 'lip_low3_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_low3_4_def_R', [27, 31])
        self.set_layers(context, 'cheek2_ctrl_4_mstr_R', [28, 29, 31])
        self.set_layers(context, 'cheek2_ctrl_4_R', [5, 25])
        self.set_layers(context, 'cheek2_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_mstr_2_R', [26, 29])
        self.set_layers(context, 'lip_up3_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_2_R', [5, 25])
        self.set_layers(context, 'lip_up3_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_up3_2_def_R', [27, 31])
        self.set_layers(context, 'cheek_mstr_2_R', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_2_R', [5, 25])
        self.set_layers(context, 'cheek2_1_rot_R', [26, 29])
        self.set_layers(context, 'cheek2_1_def_R', [27, 31])
        self.set_layers(context, 'lev1_2_def_R', [27, 31])
        self.set_layers(context, 'lev1_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up2_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_2_R', [5, 25])
        self.set_layers(context, 'oris_up2_2_def_R', [27, 31])
        self.set_layers(context, 'oris_up2_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up_mstr_3_R', [26, 29])
        self.set_layers(context, 'lip_up3_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_3_R', [5, 25])
        self.set_layers(context, 'lip_up3_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_up3_3_def_R', [27, 31])
        self.set_layers(context, 'cheek_mstr_3_R', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_3_R', [5, 25])
        self.set_layers(context, 'cheek2_2_rot_R', [26, 29])
        self.set_layers(context, 'cheek2_2_def_R', [27, 31])
        self.set_layers(context, 'lev2_2_def_R', [27, 31])
        self.set_layers(context, 'lev2_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up3_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_up3_4_def_R', [27, 31])
        self.set_layers(context, 'lip_up2_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_3_R', [5, 25])
        self.set_layers(context, 'lip_up2_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_up2_4_rot_R', [26, 29])
        self.set_layers(context, 'oris_up3_2_def_R', [27, 31])
        self.set_layers(context, 'oris_up3_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up_mstr_1_R', [26, 29])
        self.set_layers(context, 'lip_up2_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_1_R', [5, 25])
        self.set_layers(context, 'lip_up2_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_up2_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_up2_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_up2_1_bbone_L', [26])
        self.set_layers(context, 'lip_up2_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_2_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_3_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_4_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up2_2_def_R', [27, 31])
        self.set_layers(context, 'lip_up2_3_def_R', [27, 31])
        self.set_layers(context, 'lip_up2_4_def_R', [27, 31])
        self.set_layers(context, 'oris_up1_2_def_R', [27, 31])
        self.set_layers(context, 'oris_up1_1_def_R', [27, 31])
        self.set_layers(context, 'lev3_3_def_R', [27, 31])
        self.set_layers(context, 'lev3_2_def_R', [27, 31])
        self.set_layers(context, 'lev3_1_def_R', [27, 31])
        self.set_layers(context, 'oris_up4_2_def_R', [27, 31])
        self.set_layers(context, 'oris_up4_1_def_R', [27, 31])
        self.set_layers(context, 'cheek_3_rot_R', [26, 29])
        self.set_layers(context, 'cheek_3_def_R', [27, 31])
        self.set_layers(context, 'cheek_4_def_R', [27, 31])
        self.set_layers(context, 'cheek2_3_rot_R', [26, 29])
        self.set_layers(context, 'cheek2_3_def_R', [27, 31])
        self.set_layers(context, 'cheek2_4_def_R', [27, 31])
        self.set_layers(context, 'nose_bridge_1_def', [27, 31])
        self.set_layers(context, 'nose_bridge_2_def', [27, 31])
        self.set_layers(context, 'snap_neck_fk_pivot', [30])
        self.set_layers(context, 'arm_scale_R', [16, 25])
        self.set_layers(context, 'elbow_pole_torso_R', [26])
        self.set_layers(context, 'elbow_pole_hinge_R', [26])
        self.set_layers(context, 'hand_ik_mstr_R', [26])
        self.set_layers(context, 'hand_ik_hinge_R', [26])
        self.set_layers(context, 'hand_ik_ctrl_R', [0, 25, 30])
        self.set_layers(context, 'hand_ik_pivot_point_R', [0, 25, 30])
        self.set_layers(context, 'hand_ik_pivot_R', [26])
        self.set_layers(context, 'palm_bend_ik_ctrl_R', [0, 25, 30])
        self.set_layers(context, 'palm_bend_mstr_R', [26])
        self.set_layers(context, 'palm_bend_ik_R', [26])
        self.set_layers(context, 'fing_thumb_ik_mstr_2_R', [26])
        self.set_layers(context, 'fing_mid_ik_mstr_2_R', [26])
        self.set_layers(context, 'fing_ring_ik_mstr_2_R', [26])
        self.set_layers(context, 'fing_lit_ik_mstr_2_R', [26])
        self.set_layers(context, 'fing_ind_ik_mstr_2_R', [26])
        self.set_layers(context, 'hand_ik_shoulder_R', [26])
        self.set_layers(context, 'elbow_pole_R', [0, 25, 30])
        self.set_layers(context, 'shoulder_R', [0, 25, 30])
        self.set_layers(context, 'shoulder_rot_R', [17, 25, 30])
        self.set_layers(context, 'clavi_R', [26])
        self.set_layers(context, 'clavi_toon_R', [6, 25])
        self.set_layers(context, 'shoulder_shp_at_R', [26])
        self.set_layers(context, 'clavi_def_R', [27, 29, 31])
        self.set_layers(context, 'arm_mstr_R', [26])
        self.set_layers(context, 'arm_hinge_R', [26])
        self.set_layers(context, 'arm_ik_R', [1, 25, 30])
        self.set_layers(context, 'arm_fk_R', [0, 25, 29, 30])
        self.set_layers(context, 'snap_elbow_pole_fk_R', [30])
        self.set_layers(context, 'arm_scale_L', [16, 25])
        self.set_layers(context, 'elbow_pole_torso_L', [26])
        self.set_layers(context, 'elbow_pole_hinge_L', [26])
        self.set_layers(context, 'hand_ik_mstr_L', [26])
        self.set_layers(context, 'hand_ik_hinge_L', [26])
        self.set_layers(context, 'hand_ik_ctrl_L', [0, 25, 30])
        self.set_layers(context, 'hand_ik_pivot_point_L', [0, 25, 30])
        self.set_layers(context, 'hand_ik_pivot_L', [26])
        self.set_layers(context, 'palm_bend_ik_ctrl_L', [0, 25, 30])
        self.set_layers(context, 'palm_bend_mstr_L', [26])
        self.set_layers(context, 'palm_bend_ik_L', [26])
        self.set_layers(context, 'fing_thumb_ik_mstr_2_L', [26])
        self.set_layers(context, 'fing_mid_ik_mstr_2_L', [26])
        self.set_layers(context, 'fing_ring_ik_mstr_2_L', [26])
        self.set_layers(context, 'fing_lit_ik_mstr_2_L', [26])
        self.set_layers(context, 'fing_ind_ik_mstr_2_L', [26])
        self.set_layers(context, 'hand_ik_shoulder_L', [26])
        self.set_layers(context, 'elbow_pole_L', [0, 25, 30])
        self.set_layers(context, 'shoulder_L', [0, 25, 30])
        self.set_layers(context, 'shoulder_rot_L', [17, 25, 30])
        self.set_layers(context, 'clavi_L', [26])
        self.set_layers(context, 'clavi_toon_L', [6, 25])
        self.set_layers(context, 'shoulder_shp_at_L', [26])
        self.set_layers(context, 'clavi_def_L', [27, 29, 31])
        self.set_layers(context, 'arm_mstr_L', [26])
        self.set_layers(context, 'arm_hinge_L', [26])
        self.set_layers(context, 'arm_ik_L', [1, 25, 30])
        self.set_layers(context, 'arm_fk_L', [0, 25, 29, 30])
        self.set_layers(context, 'snap_elbow_pole_fk_L', [30])
        self.set_layers(context, 'spine_3_def', [27, 29, 31])
        self.set_layers(context, 'neck_1_def', [27, 29, 31])
        self.set_layers(context, 'neck_2_def', [27, 29, 31])
        self.set_layers(context, 'neck_3_def', [27, 29, 31])
        self.set_layers(context, 'pelvis_def_bbone', [26])
        self.set_layers(context, 'pelvis_def', [27, 29, 31])
        self.set_layers(context, 'pelvis_str_L', [31])
        self.set_layers(context, 'pelvis_str_R', [31])
        self.set_layers(context, 'leg_scale_R', [16, 25])
        self.set_layers(context, 'pelvis_toon_R', [7, 25])
        self.set_layers(context, 'knee_pole_limit_R', [26])
        self.set_layers(context, 'pelvis_def_R', [27, 29, 31])
        self.set_layers(context, 'thigh_mstr_R', [26])
        self.set_layers(context, 'thigh_hinge_R', [26])
        self.set_layers(context, 'leg_scale_L', [16, 25])
        self.set_layers(context, 'pelvis_toon_L', [7, 25])
        self.set_layers(context, 'knee_pole_limit_L', [26])
        self.set_layers(context, 'pelvis_def_L', [27, 29, 31])
        self.set_layers(context, 'thigh_mstr_L', [26])
        self.set_layers(context, 'thigh_hinge_L', [26])
        self.set_layers(context, 'lattice_body', [26])
        self.set_layers(context, 'belly_ctrl', [1])
        self.set_layers(context, 'butt_ctrl', [1])
        self.set_layers(context, 'master_body_str', [31])
        self.set_layers(context, 'sole_ctrl_R', [0, 25, 30, 31])
        self.set_layers(context, 'sole_pivot_point_R', [1, 25, 30])
        self.set_layers(context, 'sole_pivot_R', [26])
        self.set_layers(context, 'foot_roll_ctrl_R', [0, 25, 30, 31])
        self.set_layers(context, 'sole_R', [26])
        self.set_layers(context, 'foot_str_R', [31])
        self.set_layers(context, 'toe_big_str_2_R', [24])
        self.set_layers(context, 'toe_mid_str_2_R', [24])
        self.set_layers(context, 'toe_fourth_str_2_R', [24])
        self.set_layers(context, 'toe_lit_str_2_R', [24])
        self.set_layers(context, 'toe_ind_str_2_R', [24])
        self.set_layers(context, 'ankle_str_R', [31])
        self.set_layers(context, 'toe_str_1_R', [31])
        self.set_layers(context, 'toe_big_str_3_R', [24])
        self.set_layers(context, 'toe_mid_str_3_R', [24])
        self.set_layers(context, 'toe_mid_str_4_R', [24])
        self.set_layers(context, 'toe_fourth_str_3_R', [24])
        self.set_layers(context, 'toe_fourth_str_4_R', [24])
        self.set_layers(context, 'toe_lit_str_3_R', [24])
        self.set_layers(context, 'toe_ind_str_3_R', [24])
        self.set_layers(context, 'toe_ind_str_4_R', [24])
        self.set_layers(context, 'toe_str_2_R', [31])
        self.set_layers(context, 'toe_lit_str_4_R', [24])
        self.set_layers(context, 'toe_big_str_4_R', [24])
        self.set_layers(context, 'toe_fourth_str_5_R', [24])
        self.set_layers(context, 'toe_mid_str_5_R', [24])
        self.set_layers(context, 'toe_ind_str_5_R', [24])
        self.set_layers(context, 'toe_roll_2_R', [1, 25, 30])
        self.set_layers(context, 'toes_ik_ctrl_R', [0, 25, 29, 30])
        self.set_layers(context, 'toe_lit_ik_R', [24])
        self.set_layers(context, 'toe_big_ik_R', [24])
        self.set_layers(context, 'toe_fourth_ik_R', [24])
        self.set_layers(context, 'toe_mid_ik_R', [24])
        self.set_layers(context, 'toe_ind_ik_R', [24])
        self.set_layers(context, 'toes_ik_ctrl_mid_R', [0, 24])
        self.set_layers(context, 'toe_roll_1_R', [1, 25, 30])
        self.set_layers(context, 'foot_roll_mstr_R', [26])
        self.set_layers(context, 'foot_ik_ctrl_R', [0, 25, 30])
        self.set_layers(context, 'foot_roll_R', [26])
        self.set_layers(context, 'foot_ik_R', [26, 30])
        self.set_layers(context, 'toe_mstr_ik_R', [30])
        self.set_layers(context, 'toe_1_ik_R', [25, 30])
        self.set_layers(context, 'toe_2_ik_R', [25, 30])
        self.set_layers(context, 'knee_pole_mstr_R', [26])
        self.set_layers(context, 'knee_pole_R', [0, 25, 30])
        self.set_layers(context, 'thigh_ik_R', [1, 25, 30])
        self.set_layers(context, 'thigh_fk_R', [0, 25, 29, 30])
        self.set_layers(context, 'snap_knee_fk_R', [30])
        self.set_layers(context, 'hip_rot_R', [24])
        self.set_layers(context, 'sole_ctrl_L', [0, 25, 30, 31])
        self.set_layers(context, 'sole_pivot_point_L', [1, 25, 30])
        self.set_layers(context, 'sole_pivot_L', [26])
        self.set_layers(context, 'foot_roll_ctrl_L', [0, 25, 30, 31])
        self.set_layers(context, 'sole_L', [26])
        self.set_layers(context, 'foot_str_L', [31])
        self.set_layers(context, 'toe_big_str_2_L', [24])
        self.set_layers(context, 'toe_mid_str_2_L', [24])
        self.set_layers(context, 'toe_fourth_str_2_L', [24])
        self.set_layers(context, 'toe_lit_str_2_L', [24])
        self.set_layers(context, 'toe_ind_str_2_L', [24])
        self.set_layers(context, 'ankle_str_L', [31])
        self.set_layers(context, 'toe_str_1_L', [31])
        self.set_layers(context, 'toe_big_str_3_L', [24])
        self.set_layers(context, 'toe_mid_str_3_L', [24])
        self.set_layers(context, 'toe_mid_str_4_L', [24])
        self.set_layers(context, 'toe_fourth_str_3_L', [24])
        self.set_layers(context, 'toe_fourth_str_4_L', [24])
        self.set_layers(context, 'toe_lit_str_3_L', [24])
        self.set_layers(context, 'toe_ind_str_3_L', [24])
        self.set_layers(context, 'toe_ind_str_4_L', [24])
        self.set_layers(context, 'toe_str_2_L', [31])
        self.set_layers(context, 'toe_lit_str_4_L', [24])
        self.set_layers(context, 'toe_fourth_str_5_L', [24])
        self.set_layers(context, 'toe_mid_str_5_L', [24])
        self.set_layers(context, 'toe_big_str_4_L', [24])
        self.set_layers(context, 'toe_ind_str_5_L', [24])
        self.set_layers(context, 'toe_roll_2_L', [1, 25, 30])
        self.set_layers(context, 'toes_ik_ctrl_L', [0, 25, 29, 30])
        self.set_layers(context, 'toe_lit_ik_L', [24])
        self.set_layers(context, 'toe_fourth_ik_L', [24])
        self.set_layers(context, 'toe_mid_ik_L', [24])
        self.set_layers(context, 'toe_big_ik_L', [24])
        self.set_layers(context, 'toe_ind_ik_L', [24])
        self.set_layers(context, 'toes_ik_ctrl_mid_L', [0, 24])
        self.set_layers(context, 'toe_roll_1_L', [1, 25, 30])
        self.set_layers(context, 'foot_roll_mstr_L', [26])
        self.set_layers(context, 'foot_ik_ctrl_L', [0, 25, 30])
        self.set_layers(context, 'foot_roll_L', [26])
        self.set_layers(context, 'foot_ik_L', [26, 30])
        self.set_layers(context, 'toe_mstr_ik_L', [30])
        self.set_layers(context, 'toe_1_ik_L', [25, 30])
        self.set_layers(context, 'toe_2_ik_L', [25, 30])
        self.set_layers(context, 'knee_pole_mstr_L', [26])
        self.set_layers(context, 'knee_pole_L', [0, 25, 30])
        self.set_layers(context, 'thigh_ik_L', [1, 25, 30])
        self.set_layers(context, 'thigh_fk_L', [0, 25, 29, 30])
        self.set_layers(context, 'snap_knee_fk_L', [30])
        self.set_layers(context, 'hip_rot_L', [24])
        self.set_layers(context, 'forearm_ik_R', [1, 25, 30])
        self.set_layers(context, 'forearm_fk_R', [0, 25, 29, 30])
        self.set_layers(context, 'hand_toon_R', [7, 25])
        self.set_layers(context, 'forearm_twist_bezier_out_mstr_R', [26])
        self.set_layers(context, 'hand_fk_R', [0, 25, 30])
        self.set_layers(context, 'fing_ind_ik_mstr_R', [26])
        self.set_layers(context, 'fing_ind_hinge_R', [26])
        self.set_layers(context, 'fing_ind_ik_R', [0, 24, 25])
        self.set_layers(context, 'fing_thumb_ik_mstr_R', [26])
        self.set_layers(context, 'fing_thumb_hinge_R', [26])
        self.set_layers(context, 'fing_thumb_ik_R', [0, 24, 25])
        self.set_layers(context, 'fing_mid_ik_mstr_R', [26])
        self.set_layers(context, 'fing_mid_hinge_R', [26])
        self.set_layers(context, 'fing_mid_ik_R', [0, 24, 25])
        self.set_layers(context, 'fing_ring_ik_mstr_R', [26])
        self.set_layers(context, 'fing_ring_hinge_R', [26])
        self.set_layers(context, 'fing_ring_ik_R', [0, 24, 25])
        self.set_layers(context, 'fing_lit_ik_mstr_R', [26])
        self.set_layers(context, 'fing_lit_hinge_R', [26])
        self.set_layers(context, 'fing_lit_ik_R', [0, 24, 25])
        self.set_layers(context, 'hand_close_R', [0, 24, 25])
        self.set_layers(context, 'palm_bend_fk_ctrl_R', [0, 25, 30])
        self.set_layers(context, 'palm_bend_fk_R', [30])
        self.set_layers(context, 'hand_mstr_R', [26])
        self.set_layers(context, 'arm_def_mstr_R', [26])
        self.set_layers(context, 'elbow_toon_R', [6, 25])
        self.set_layers(context, 'elbow_toon_loc_R', [26])
        self.set_layers(context, 'arm_toon_mstr_R', [26])
        self.set_layers(context, 'forearm_mstr_R', [26])
        self.set_layers(context, 'forearm_def_mstr_R', [26])
        self.set_layers(context, 'forearm_toon_2_mstr_R', [26])
        self.set_layers(context, 'forearm_toon_2_R', [26])
        self.set_layers(context, 'forearm_toon_mstr_R', [26])
        self.set_layers(context, 'hand_drv_R', [26])
        self.set_layers(context, 'forearm_toon_def_R', [26])
        self.set_layers(context, 'hand_def_mstr_R', [26])
        self.set_layers(context, 'hand_def_R', [27, 29, 31])
        self.set_layers(context, 'fing_lit_ctrl_bend_loc_R', [26])
        self.set_layers(context, 'fing_lit_ctrl_loc_R', [26])
        self.set_layers(context, 'fing_ring_ctrl_bend_loc_R', [26])
        self.set_layers(context, 'fing_ring_ctrl_loc_R', [26])
        self.set_layers(context, 'fing_ind_ctrl_bend_loc_R', [26])
        self.set_layers(context, 'fing_ind_ctrl_loc_R', [26])
        self.set_layers(context, 'fing_mid_ctrl_bend_loc_R', [26])
        self.set_layers(context, 'fing_mid_ctrl_loc_R', [26])
        self.set_layers(context, 'fing_thumb_1_R', [2, 24, 25])
        self.set_layers(context, 'fing_thumb_ctrl_mstr_R', [28])
        self.set_layers(context, 'fing_thumb_ctrl_R', [0, 24, 25, 29])
        self.set_layers(context, 'fing_thumb_1_ik_R', [26])
        self.set_layers(context, 'fing_thumb_ctrl_shp_at_R', [26])
        self.set_layers(context, 'fing_thumb_2_R', [2, 24, 25])
        self.set_layers(context, 'fing_thumb_3_R', [2, 24, 25])
        self.set_layers(context, 'fing_thumb_shp_at_R', [26])
        self.set_layers(context, 'fing_thumb_2_rot_R', [26])
        self.set_layers(context, 'fing_thumb_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_thumb_fix_up_1_R', [24, 27])
        self.set_layers(context, 'fing_thumb_fix_low_1_R', [24, 27])
        self.set_layers(context, 'fing_thumb_2_def_mstr_R', [26])
        self.set_layers(context, 'fing_thumb_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_thumb_fix_low_2_R', [24, 27])
        self.set_layers(context, 'fing_thumb_fix_up_2_R', [24, 27])
        self.set_layers(context, 'fing_thumb_3_def_mstr_R', [26])
        self.set_layers(context, 'fing_thumb_3_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_spread_R', [0, 24, 25])
        self.set_layers(context, 'fing_lit_ctrl_mstr_R', [28])
        self.set_layers(context, 'fing_lit_ctrl_R', [0, 24, 25, 29])
        self.set_layers(context, 'fing_ring_ctrl_mstr_R', [28])
        self.set_layers(context, 'fing_ring_ctrl_R', [0, 24, 25, 29])
        self.set_layers(context, 'fing_ind_ctrl_mstr_R', [28])
        self.set_layers(context, 'fing_ind_ctrl_R', [0, 24, 25, 29])
        self.set_layers(context, 'fing_mid_ctrl_mstr_R', [28])
        self.set_layers(context, 'fing_mid_ctrl_R', [0, 24, 25, 29])
        self.set_layers(context, 'fing_lit_1_R', [26])
        self.set_layers(context, 'fing_lit_ctrl_shp_at_R', [26])
        self.set_layers(context, 'fing_lit_2_R', [2, 24, 25])
        self.set_layers(context, 'fing_lit_3_R', [2, 24, 25])
        self.set_layers(context, 'fing_lit_4_R', [2, 24, 25])
        self.set_layers(context, 'fing_lit_shp_at_R', [26])
        self.set_layers(context, 'fing_lit_2_rot_R', [26])
        self.set_layers(context, 'fing_lit_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_up_1_R', [24, 27])
        self.set_layers(context, 'fing_lit_fix_low_1_R', [24, 27])
        self.set_layers(context, 'fing_lit_2_def_mstr_R', [26])
        self.set_layers(context, 'fing_lit_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_low_2_R', [24, 27])
        self.set_layers(context, 'fing_lit_fix_up_2_R', [24, 27])
        self.set_layers(context, 'fing_lit_3_def_mstr_R', [26])
        self.set_layers(context, 'fing_lit_3_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_low_3_R', [24, 27])
        self.set_layers(context, 'fing_lit_fix_up_3_R', [24, 27])
        self.set_layers(context, 'fing_lit_4_def_mstr_R', [26])
        self.set_layers(context, 'fing_lit_4_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_1_R', [26])
        self.set_layers(context, 'fing_ring_ctrl_shp_at_R', [26])
        self.set_layers(context, 'fing_ring_2_R', [2, 24, 25])
        self.set_layers(context, 'fing_ring_3_rot_R', [26])
        self.set_layers(context, 'fing_ring_4_rot_R', [26])
        self.set_layers(context, 'fing_ring_3_R', [2, 24, 25])
        self.set_layers(context, 'fing_ring_4_R', [2, 24, 25])
        self.set_layers(context, 'fing_ring_shp_at_R', [26])
        self.set_layers(context, 'fing_ring_2_rot_R', [26])
        self.set_layers(context, 'fing_ring_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_up_1_R', [24, 27])
        self.set_layers(context, 'fing_ring_fix_low_1_R', [24, 27])
        self.set_layers(context, 'fing_ring_2_def_mstr_R', [26])
        self.set_layers(context, 'fing_ring_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_low_2_R', [24, 27])
        self.set_layers(context, 'fing_ring_fix_up_2_R', [24, 27])
        self.set_layers(context, 'fing_ring_3_def_mstr_R', [26])
        self.set_layers(context, 'fing_ring_3_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_low_3_R', [24, 27])
        self.set_layers(context, 'fing_ring_fix_up_3_R', [24, 27])
        self.set_layers(context, 'fing_ring_4_def_mstr_R', [26])
        self.set_layers(context, 'fing_ring_4_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_1_R', [26])
        self.set_layers(context, 'fing_ind_ctrl_shp_at_R', [26])
        self.set_layers(context, 'fing_ind_2_R', [2, 24, 25])
        self.set_layers(context, 'fing_ind_3_R', [2, 24, 25])
        self.set_layers(context, 'fing_ind_4_R', [2, 24, 25])
        self.set_layers(context, 'fing_ind_shp_at_R', [26])
        self.set_layers(context, 'fing_ind_2_rot_R', [26])
        self.set_layers(context, 'fing_ind_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_1_R', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_1_R', [24, 27])
        self.set_layers(context, 'fing_ind_2_def_mstr_R', [26])
        self.set_layers(context, 'fing_ind_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_2_R', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_2_R', [24, 27])
        self.set_layers(context, 'fing_ind_3_def_mstr_R', [26])
        self.set_layers(context, 'fing_ind_3_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_3_R', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_3_R', [24, 27])
        self.set_layers(context, 'fing_ind_4_def_mstr_R', [26])
        self.set_layers(context, 'fing_ind_4_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_1_R', [26])
        self.set_layers(context, 'fing_mid_ctrl_shp_at_R', [26])
        self.set_layers(context, 'fing_mid_2_R', [2, 24, 25])
        self.set_layers(context, 'fing_mid_3_R', [2, 24, 25])
        self.set_layers(context, 'fing_mid_4_R', [2, 24, 25])
        self.set_layers(context, 'fing_mid_shp_at_R', [26])
        self.set_layers(context, 'fing_mid_2_rot_R', [26])
        self.set_layers(context, 'fing_mid_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_up_1_R', [24, 27])
        self.set_layers(context, 'fing_mid_fix_low_1_R', [24, 27])
        self.set_layers(context, 'fing_mid_2_def_mstr_R', [26])
        self.set_layers(context, 'fing_mid_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_low_2_R', [24, 27])
        self.set_layers(context, 'fing_mid_fix_up_2_R', [24, 27])
        self.set_layers(context, 'fing_mid_3_def_mstr_R', [26])
        self.set_layers(context, 'fing_mid_3_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_low_3_R', [24, 27])
        self.set_layers(context, 'fing_mid_fix_up_3_R', [24, 27])
        self.set_layers(context, 'fing_mid_4_def_mstr_R', [26])
        self.set_layers(context, 'fing_mid_4_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'hand_accessory_R', [17, 25])
        self.set_layers(context, 'forearm_rot_R', [26])
        self.set_layers(context, 'forearm_twist_R', [26])
        self.set_layers(context, 'forearm_twist_bezier_out_R', [26])
        self.set_layers(context, 'elbow_fix_R', [27])
        self.set_layers(context, 'arm_fix_R', [27])
        self.set_layers(context, 'elbow_line_R', [0, 25, 31])
        self.set_layers(context, 'arm_drv_R', [26])
        self.set_layers(context, 'back_fix_R', [27])
        self.set_layers(context, 'chest_fix_R', [27])
        self.set_layers(context, 'arm_twist_R', [26])
        self.set_layers(context, 'arm_def_bezier_in_R', [26])
        self.set_layers(context, 'arm_def_bbone_in_R', [26])
        self.set_layers(context, 'forearm_toon_R', [7, 25])
        self.set_layers(context, 'forearm_def_bezier_out_mstr_R', [26])
        self.set_layers(context, 'forearm_def_bezier_out_R', [26])
        self.set_layers(context, 'forearm_def_bezier_out_rot_R', [26])
        self.set_layers(context, 'forearm_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'forearm_twist_bezier_in_R', [26])
        self.set_layers(context, 'forearm_twist_bbone_in_R', [26])
        self.set_layers(context, 'forearm_twist_def_R', [27, 31])
        self.set_layers(context, 'hand_twist_drv_R', [26])
        self.set_layers(context, 'wrist_fix_low_R', [27])
        self.set_layers(context, 'wrist_fix_up_R', [27])
        self.set_layers(context, 'forearm_twist_bbone_out_R', [26])
        self.set_layers(context, 'forearm_no_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'forearm_no_twist_bezier_in_R', [26])
        self.set_layers(context, 'forearm_no_twist_bbone_in_R', [26])
        self.set_layers(context, 'forearm_no_twist_def_R', [27])
        self.set_layers(context, 'arm_toon_R', [7, 25])
        self.set_layers(context, 'arm_def_bezier_out_mstr_R', [26])
        self.set_layers(context, 'arm_def_bezier_out_R', [26])
        self.set_layers(context, 'arm_def_R', [27, 31])
        self.set_layers(context, 'arm_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'arm_twist_bezier_in_R', [26])
        self.set_layers(context, 'arm_def_bbone_out_R', [26])
        self.set_layers(context, 'arm_twist_bbone_in_R', [26])
        self.set_layers(context, 'arm_twist_def_R', [27, 31])
        self.set_layers(context, 'arm_twist_curved_R', [26])
        self.set_layers(context, 'arm_twist_bezier_out_mstr_R', [26])
        self.set_layers(context, 'arm_twist_bezier_out_R', [26])
        self.set_layers(context, 'arm_twist_bbone_out_rot_R', [26])
        self.set_layers(context, 'arm_twist_bbone_out_R', [26])
        self.set_layers(context, 'forearm_def_curved_R', [26])
        self.set_layers(context, 'forearm_def_bezier_in_mstr_R', [26])
        self.set_layers(context, 'forearm_def_bezier_in_R', [26])
        self.set_layers(context, 'forearm_def_bbone_in_R', [26])
        self.set_layers(context, 'forearm_def_R', [27, 31])
        self.set_layers(context, 'forearm_def_bbone_out_R', [26])
        self.set_layers(context, 'forearm_def_no_twist_bbone_in_R', [26])
        self.set_layers(context, 'forearm_def_no_twist_R', [27])
        self.set_layers(context, 'forearm_def_bbone_out_no_twist_R', [26])
        self.set_layers(context, 'forearm_ik_L', [1, 25, 30])
        self.set_layers(context, 'forearm_fk_L', [0, 25, 29, 30])
        self.set_layers(context, 'hand_toon_L', [7, 25])
        self.set_layers(context, 'forearm_twist_bezier_out_mstr_L', [26])
        self.set_layers(context, 'hand_fk_L', [0, 25, 30])
        self.set_layers(context, 'fing_ind_ik_mstr_L', [26])
        self.set_layers(context, 'fing_ind_hinge_L', [26])
        self.set_layers(context, 'fing_ind_ik_L', [0, 24, 25])
        self.set_layers(context, 'fing_thumb_ik_mstr_L', [26])
        self.set_layers(context, 'fing_thumb_hinge_L', [26])
        self.set_layers(context, 'fing_thumb_ik_L', [0, 24, 25])
        self.set_layers(context, 'fing_mid_ik_mstr_L', [26])
        self.set_layers(context, 'fing_mid_hinge_L', [26])
        self.set_layers(context, 'fing_mid_ik_L', [0, 24, 25])
        self.set_layers(context, 'fing_ring_ik_mstr_L', [26])
        self.set_layers(context, 'fing_ring_hinge_L', [26])
        self.set_layers(context, 'fing_ring_ik_L', [0, 24, 25])
        self.set_layers(context, 'fing_lit_ik_mstr_L', [26])
        self.set_layers(context, 'fing_lit_hinge_L', [26])
        self.set_layers(context, 'fing_lit_ik_L', [0, 24, 25])
        self.set_layers(context, 'hand_close_L', [0, 24, 25])
        self.set_layers(context, 'palm_bend_fk_ctrl_L', [0, 25, 30])
        self.set_layers(context, 'palm_bend_fk_L', [30])
        self.set_layers(context, 'hand_mstr_L', [26])
        self.set_layers(context, 'arm_def_mstr_L', [26])
        self.set_layers(context, 'elbow_toon_L', [6, 25])
        self.set_layers(context, 'elbow_toon_loc_L', [26])
        self.set_layers(context, 'arm_toon_mstr_L', [26])
        self.set_layers(context, 'forearm_mstr_L', [26])
        self.set_layers(context, 'forearm_def_mstr_L', [26])
        self.set_layers(context, 'forearm_toon_2_mstr_L', [26])
        self.set_layers(context, 'forearm_toon_2_L', [26])
        self.set_layers(context, 'forearm_toon_mstr_L', [26])
        self.set_layers(context, 'hand_drv_L', [26])
        self.set_layers(context, 'forearm_toon_def_L', [26])
        self.set_layers(context, 'hand_def_mstr_L', [26])
        self.set_layers(context, 'hand_def_L', [27, 29, 31])
        self.set_layers(context, 'fing_lit_ctrl_bend_loc_L', [26])
        self.set_layers(context, 'fing_lit_ctrl_loc_L', [26])
        self.set_layers(context, 'fing_ring_ctrl_bend_loc_L', [26])
        self.set_layers(context, 'fing_ring_ctrl_loc_L', [26])
        self.set_layers(context, 'fing_ind_ctrl_bend_loc_L', [26])
        self.set_layers(context, 'fing_ind_ctrl_loc_L', [26])
        self.set_layers(context, 'fing_mid_ctrl_bend_loc_L', [26])
        self.set_layers(context, 'fing_mid_ctrl_loc_L', [26])
        self.set_layers(context, 'fing_thumb_1_L', [2, 24, 25])
        self.set_layers(context, 'fing_thumb_ctrl_mstr_L', [28])
        self.set_layers(context, 'fing_thumb_ctrl_L', [0, 24, 25, 29])
        self.set_layers(context, 'fing_thumb_1_ik_L', [26])
        self.set_layers(context, 'fing_thumb_ctrl_shp_at_L', [26])
        self.set_layers(context, 'fing_thumb_2_L', [2, 24, 25])
        self.set_layers(context, 'fing_thumb_3_L', [2, 24, 25])
        self.set_layers(context, 'fing_thumb_shp_at_L', [26])
        self.set_layers(context, 'fing_thumb_2_rot_L', [26])
        self.set_layers(context, 'fing_thumb_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_thumb_fix_low_1_L', [24, 27])
        self.set_layers(context, 'fing_thumb_fix_up_1_L', [24, 27])
        self.set_layers(context, 'fing_thumb_2_def_mstr_L', [26])
        self.set_layers(context, 'fing_thumb_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_thumb_fix_low_2_L', [24, 27])
        self.set_layers(context, 'fing_thumb_fix_up_2_L', [24, 27])
        self.set_layers(context, 'fing_thumb_3_def_mstr_L', [26])
        self.set_layers(context, 'fing_thumb_3_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_spread_L', [0, 24, 25])
        self.set_layers(context, 'fing_lit_ctrl_mstr_L', [28])
        self.set_layers(context, 'fing_lit_ctrl_L', [0, 24, 25, 29])
        self.set_layers(context, 'fing_ring_ctrl_mstr_L', [28])
        self.set_layers(context, 'fing_ring_ctrl_L', [0, 24, 25, 29])
        self.set_layers(context, 'fing_ind_ctrl_mstr_L', [28])
        self.set_layers(context, 'fing_ind_ctrl_L', [0, 24, 25, 29])
        self.set_layers(context, 'fing_mid_ctrl_mstr_L', [28])
        self.set_layers(context, 'fing_mid_ctrl_L', [0, 24, 25, 29])
        self.set_layers(context, 'fing_lit_1_L', [26])
        self.set_layers(context, 'fing_lit_ctrl_shp_at_L', [26])
        self.set_layers(context, 'fing_lit_2_L', [2, 24, 25])
        self.set_layers(context, 'fing_lit_3_L', [2, 24, 25])
        self.set_layers(context, 'fing_lit_4_L', [2, 24, 25])
        self.set_layers(context, 'fing_lit_shp_at_L', [26])
        self.set_layers(context, 'fing_lit_2_rot_L', [26])
        self.set_layers(context, 'fing_lit_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_low_1_L', [24, 27])
        self.set_layers(context, 'fing_lit_fix_up_1_L', [24, 27])
        self.set_layers(context, 'fing_lit_2_def_mstr_L', [26])
        self.set_layers(context, 'fing_lit_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_low_2_L', [24, 27])
        self.set_layers(context, 'fing_lit_fix_up_2_L', [24, 27])
        self.set_layers(context, 'fing_lit_3_def_mstr_L', [26])
        self.set_layers(context, 'fing_lit_3_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_low_3_L', [24, 27])
        self.set_layers(context, 'fing_lit_fix_up_3_L', [24, 27])
        self.set_layers(context, 'fing_lit_4_def_mstr_L', [26])
        self.set_layers(context, 'fing_lit_4_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_1_L', [26])
        self.set_layers(context, 'fing_ring_ctrl_shp_at_L', [26])
        self.set_layers(context, 'fing_ring_2_L', [2, 24, 25])
        self.set_layers(context, 'fing_ring_3_rot_L', [26])
        self.set_layers(context, 'fing_ring_4_rot_L', [26])
        self.set_layers(context, 'fing_ring_3_L', [2, 24, 25])
        self.set_layers(context, 'fing_ring_4_L', [2, 24, 25])
        self.set_layers(context, 'fing_ring_shp_at_L', [26])
        self.set_layers(context, 'fing_ring_2_rot_L', [26])
        self.set_layers(context, 'fing_ring_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_low_1_L', [24, 27])
        self.set_layers(context, 'fing_ring_fix_up_1_L', [24, 27])
        self.set_layers(context, 'fing_ring_2_def_mstr_L', [26])
        self.set_layers(context, 'fing_ring_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_low_2_L', [24, 27])
        self.set_layers(context, 'fing_ring_fix_up_2_L', [24, 27])
        self.set_layers(context, 'fing_ring_3_def_mstr_L', [26])
        self.set_layers(context, 'fing_ring_3_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_low_3_L', [24, 27])
        self.set_layers(context, 'fing_ring_fix_up_3_L', [24, 27])
        self.set_layers(context, 'fing_ring_4_def_mstr_L', [26])
        self.set_layers(context, 'fing_ring_4_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_1_L', [26])
        self.set_layers(context, 'fing_ind_ctrl_shp_at_L', [26])
        self.set_layers(context, 'fing_ind_2_L', [2, 24, 25])
        self.set_layers(context, 'fing_ind_3_L', [2, 24, 25])
        self.set_layers(context, 'fing_ind_4_L', [2, 24, 25])
        self.set_layers(context, 'fing_ind_shp_at_L', [26])
        self.set_layers(context, 'fing_ind_2_rot_L', [26])
        self.set_layers(context, 'fing_ind_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_1_L', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_1_L', [24, 27])
        self.set_layers(context, 'fing_ind_2_def_mstr_L', [26])
        self.set_layers(context, 'fing_ind_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_2_L', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_2_L', [24, 27])
        self.set_layers(context, 'fing_ind_3_def_mstr_L', [26])
        self.set_layers(context, 'fing_ind_3_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_3_L', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_3_L', [24, 27])
        self.set_layers(context, 'fing_ind_4_def_mstr_L', [26])
        self.set_layers(context, 'fing_ind_4_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_1_L', [26])
        self.set_layers(context, 'fing_mid_ctrl_shp_at_L', [26])
        self.set_layers(context, 'fing_mid_2_L', [2, 24, 25])
        self.set_layers(context, 'fing_mid_3_L', [2, 24, 25])
        self.set_layers(context, 'fing_mid_4_L', [2, 24, 25])
        self.set_layers(context, 'fing_mid_shp_at_L', [26])
        self.set_layers(context, 'fing_mid_2_rot_L', [26])
        self.set_layers(context, 'fing_mid_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_low_1_L', [24, 27])
        self.set_layers(context, 'fing_mid_fix_up_1_L', [24, 27])
        self.set_layers(context, 'fing_mid_2_def_mstr_L', [26])
        self.set_layers(context, 'fing_mid_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_low_2_L', [24, 27])
        self.set_layers(context, 'fing_mid_fix_up_2_L', [24, 27])
        self.set_layers(context, 'fing_mid_3_def_mstr_L', [26])
        self.set_layers(context, 'fing_mid_3_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_low_3_L', [24, 27])
        self.set_layers(context, 'fing_mid_fix_up_3_L', [24, 27])
        self.set_layers(context, 'fing_mid_4_def_mstr_L', [26])
        self.set_layers(context, 'fing_mid_4_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'hand_accessory_L', [17, 25])
        self.set_layers(context, 'forearm_rot_L', [26])
        self.set_layers(context, 'forearm_twist_L', [26])
        self.set_layers(context, 'forearm_twist_bezier_out_L', [26])
        self.set_layers(context, 'elbow_fix_L', [27])
        self.set_layers(context, 'arm_fix_L', [27])
        self.set_layers(context, 'elbow_line_L', [0, 25, 31])
        self.set_layers(context, 'arm_toon_L', [7, 25])
        self.set_layers(context, 'arm_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'arm_twist_bezier_in_L', [26])
        self.set_layers(context, 'arm_def_bezier_out_mstr_L', [26])
        self.set_layers(context, 'arm_def_bezier_out_L', [26])
        self.set_layers(context, 'forearm_def_curved_L', [26])
        self.set_layers(context, 'forearm_def_bezier_in_mstr_L', [26])
        self.set_layers(context, 'forearm_def_bezier_in_L', [26])
        self.set_layers(context, 'forearm_def_bbone_in_L', [26])
        self.set_layers(context, 'forearm_def_no_twist_bbone_in_L', [26])
        self.set_layers(context, 'forearm_toon_L', [7, 25])
        self.set_layers(context, 'forearm_def_bezier_out_mstr_L', [26])
        self.set_layers(context, 'forearm_def_bezier_out_L', [26])
        self.set_layers(context, 'forearm_def_bezier_out_rot_L', [26])
        self.set_layers(context, 'forearm_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'forearm_twist_bezier_in_L', [26])
        self.set_layers(context, 'forearm_twist_bbone_in_L', [26])
        self.set_layers(context, 'forearm_twist_def_L', [27, 31])
        self.set_layers(context, 'forearm_twist_bbone_out_L', [26])
        self.set_layers(context, 'hand_twist_drv_L', [26])
        self.set_layers(context, 'wrist_fix_low_L', [27])
        self.set_layers(context, 'wrist_fix_up_L', [27])
        self.set_layers(context, 'forearm_no_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'forearm_no_twist_bezier_in_L', [26])
        self.set_layers(context, 'forearm_no_twist_bbone_in_L', [26])
        self.set_layers(context, 'forearm_no_twist_def_L', [27])
        self.set_layers(context, 'forearm_def_L', [27, 31])
        self.set_layers(context, 'forearm_def_bbone_out_L', [26])
        self.set_layers(context, 'forearm_def_no_twist_L', [27])
        self.set_layers(context, 'forearm_def_bbone_out_no_twist_L', [26])
        self.set_layers(context, 'arm_drv_L', [26])
        self.set_layers(context, 'back_fix_L', [27])
        self.set_layers(context, 'chest_fix_L', [27])
        self.set_layers(context, 'arm_twist_L', [26])
        self.set_layers(context, 'arm_twist_bbone_in_L', [26])
        self.set_layers(context, 'arm_twist_def_L', [27, 31])
        self.set_layers(context, 'arm_twist_curved_L', [26])
        self.set_layers(context, 'arm_twist_bezier_out_mstr_L', [26])
        self.set_layers(context, 'arm_twist_bezier_out_L', [26])
        self.set_layers(context, 'arm_twist_bbone_out_rot_L', [26])
        self.set_layers(context, 'arm_twist_bbone_out_L', [26])
        self.set_layers(context, 'arm_def_bezier_in_L', [26])
        self.set_layers(context, 'arm_def_bbone_in_L', [26])
        self.set_layers(context, 'arm_def_L', [27, 31])
        self.set_layers(context, 'arm_def_bbone_out_L', [26])
        self.set_layers(context, 'shin_ik_R', [1, 25, 30])
        self.set_layers(context, 'shin_fk_R', [0, 25, 29, 30])
        self.set_layers(context, 'foot_toon_R', [7, 25])
        self.set_layers(context, 'shin_twist_bezier_out_mstr_R', [26])
        self.set_layers(context, 'foot_scale_R', [26])
        self.set_layers(context, 'foot_fk_R', [0, 25, 30])
        self.set_layers(context, 'toes_spread_R', [24])
        self.set_layers(context, 'toe_lit_str_1_R', [24])
        self.set_layers(context, 'toe_fourth_str_1_R', [24])
        self.set_layers(context, 'toe_mid_str_1_R', [24])
        self.set_layers(context, 'toe_big_str_1_R', [24])
        self.set_layers(context, 'toe_ind_str_1_R', [24])
        self.set_layers(context, 'snap_sole_ctrl_fk_R', [30])
        self.set_layers(context, 'toe_1_fk_R', [0, 24, 25, 30])
        self.set_layers(context, 'snap_toes_ctrl_mid_fk_R', [30])
        self.set_layers(context, 'toe_2_fk_R', [0, 24, 25, 30])
        self.set_layers(context, 'snap_toes_ctrl_fk_R', [30])
        self.set_layers(context, 'toes_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'toe_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'toe_fix_up_1_R', [24, 27])
        self.set_layers(context, 'toe_fix_low_1_R', [24, 27])
        self.set_layers(context, 'toe_big_ctrl_mstr_R', [26])
        self.set_layers(context, 'toe_big_ctrl_R', [24, 29])
        self.set_layers(context, 'toe_big_1_R', [26])
        self.set_layers(context, 'toe_big_2_R', [24])
        self.set_layers(context, 'toe_big_3_R', [24])
        self.set_layers(context, 'toe_big_shp_at_R', [26])
        self.set_layers(context, 'toe_big_2_rot_R', [26])
        self.set_layers(context, 'toe_big_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_big_1_def_R', [24, 29])
        self.set_layers(context, 'toe_big_fix_up_1_R', [24])
        self.set_layers(context, 'toe_big_fix_low_1_R', [24])
        self.set_layers(context, 'toe_big_2_def_mstr_R', [26])
        self.set_layers(context, 'toe_big_2_def_R', [24, 29])
        self.set_layers(context, 'toe_big_fix_up_2_R', [24])
        self.set_layers(context, 'toe_big_fix_low_2_R', [24])
        self.set_layers(context, 'toe_big_3_def_mstr_R', [26])
        self.set_layers(context, 'toe_big_3_def_R', [24, 29])
        self.set_layers(context, 'toe_lit_ctrl_mstr_R', [26])
        self.set_layers(context, 'toe_lit_ctrl_R', [24, 29])
        self.set_layers(context, 'toe_lit_1_R', [26])
        self.set_layers(context, 'toe_lit_2_R', [24])
        self.set_layers(context, 'toe_lit_3_R', [24])
        self.set_layers(context, 'toe_lit_shp_at_R', [26])
        self.set_layers(context, 'toe_lit_2_rot_R', [26])
        self.set_layers(context, 'toe_lit_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_lit_1_def_R', [24, 29])
        self.set_layers(context, 'toe_lit_fix_up_1_R', [24])
        self.set_layers(context, 'toe_lit_fix_low_1_R', [24])
        self.set_layers(context, 'toe_lit_2_def_mstr_R', [26])
        self.set_layers(context, 'toe_lit_2_def_R', [24, 29])
        self.set_layers(context, 'toe_lit_fix_up_2_R', [24])
        self.set_layers(context, 'toe_lit_fix_low_2_R', [24])
        self.set_layers(context, 'toe_lit_3_def_mstr_R', [26])
        self.set_layers(context, 'toe_lit_3_def_R', [24, 29])
        self.set_layers(context, 'toe_fourth_ctrl_mstr_R', [26])
        self.set_layers(context, 'toe_fourth_ctrl_R', [24, 29])
        self.set_layers(context, 'toe_fourth_1_R', [26])
        self.set_layers(context, 'toe_fourth_2_R', [24])
        self.set_layers(context, 'toe_fourth_3_R', [24])
        self.set_layers(context, 'toe_fourth_4_R', [24])
        self.set_layers(context, 'toe_fourth_shp_at_R', [26])
        self.set_layers(context, 'toe_fourth_2_rot_R', [26])
        self.set_layers(context, 'toe_fourth_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_fourth_1_def_R', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_1_R', [24])
        self.set_layers(context, 'toe_fourth_fix_low_1_R', [24])
        self.set_layers(context, 'toe_fourth_2_def_mstr_R', [26])
        self.set_layers(context, 'toe_fourth_2_def_R', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_2_R', [24])
        self.set_layers(context, 'toe_fourth_fix_low_2_R', [24])
        self.set_layers(context, 'toe_fourth_3_def_mstr_R', [26])
        self.set_layers(context, 'toe_fourth_3_def_R', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_3_R', [24])
        self.set_layers(context, 'toe_fourth_fix_low_3_R', [24])
        self.set_layers(context, 'toe_fourth_4_def_mstr_R', [26])
        self.set_layers(context, 'toe_fourth_4_def_R', [24, 29])
        self.set_layers(context, 'toe_mid_ctrl_mstr_R', [26])
        self.set_layers(context, 'toe_mid_ctrl_R', [24, 29])
        self.set_layers(context, 'toe_mid_1_R', [26])
        self.set_layers(context, 'toe_mid_2_R', [24])
        self.set_layers(context, 'toe_mid_3_R', [24])
        self.set_layers(context, 'toe_mid_4_R', [24])
        self.set_layers(context, 'toe_mid_shp_at_R', [26])
        self.set_layers(context, 'toe_mid_2_rot_R', [26])
        self.set_layers(context, 'toe_mid_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_mid_1_def_R', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_1_R', [24])
        self.set_layers(context, 'toe_mid_fix_low_1_R', [24])
        self.set_layers(context, 'toe_mid_2_def_mstr_R', [26])
        self.set_layers(context, 'toe_mid_2_def_R', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_2_R', [24])
        self.set_layers(context, 'toe_mid_fix_low_2_R', [24])
        self.set_layers(context, 'toe_mid_3_def_mstr_R', [26])
        self.set_layers(context, 'toe_mid_3_def_R', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_3_R', [24])
        self.set_layers(context, 'toe_mid_fix_low_3_R', [24])
        self.set_layers(context, 'toe_mid_4_def_mstr_R', [26])
        self.set_layers(context, 'toe_mid_4_def_R', [24, 29])
        self.set_layers(context, 'toe_ind_ctrl_mstr_R', [26])
        self.set_layers(context, 'toe_ind_ctrl_R', [24, 29])
        self.set_layers(context, 'toe_ind_1_R', [26])
        self.set_layers(context, 'toe_ind_2_R', [24])
        self.set_layers(context, 'toe_ind_3_R', [24])
        self.set_layers(context, 'toe_ind_4_R', [24])
        self.set_layers(context, 'toe_ind_shp_at_R', [26])
        self.set_layers(context, 'toe_ind_2_rot_R', [26])
        self.set_layers(context, 'toe_ind_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_ind_1_def_R', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_1_R', [24])
        self.set_layers(context, 'toe_ind_fix_low_1_R', [24])
        self.set_layers(context, 'toe_ind_2_def_mstr_R', [26])
        self.set_layers(context, 'toe_ind_2_def_R', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_2_R', [24])
        self.set_layers(context, 'toe_ind_fix_low_2_R', [24])
        self.set_layers(context, 'toe_ind_3_def_mstr_R', [26])
        self.set_layers(context, 'toe_ind_3_def_R', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_3_R', [24])
        self.set_layers(context, 'toe_ind_fix_low_3_R', [24])
        self.set_layers(context, 'toe_ind_4_def_mstr_R', [26])
        self.set_layers(context, 'toe_ind_4_def_R', [24, 29])
        self.set_layers(context, 'shin_rot_R', [26])
        self.set_layers(context, 'foot_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'foot_drv_R', [26])
        self.set_layers(context, 'thigh_def_mstr_R', [26])
        self.set_layers(context, 'knee_toon_R', [6, 25])
        self.set_layers(context, 'knee_toon_loc_R', [26])
        self.set_layers(context, 'thigh_toon_mstr_R', [26])
        self.set_layers(context, 'thigh_drv_R', [26])
        self.set_layers(context, 'hip_fix_R', [27])
        self.set_layers(context, 'thigh_fix_R', [27])
        self.set_layers(context, 'buttock_ctrl_R', [1, 26, 31])
        self.set_layers(context, 'buttock_def_R', [27, 31])
        self.set_layers(context, 'thigh_twist_R', [26])
        self.set_layers(context, 'thigh_def_bezier_in_R', [26])
        self.set_layers(context, 'thigh_def_bbone_in_R', [26])
        self.set_layers(context, 'knee_fix_R', [27])
        self.set_layers(context, 'shin_fix_R', [27])
        self.set_layers(context, 'thigh_toon_R', [7, 25])
        self.set_layers(context, 'thigh_def_bezier_out_mstr_R', [26])
        self.set_layers(context, 'thigh_def_bezier_out_R', [26])
        self.set_layers(context, 'thigh_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'thigh_twist_bezier_in_R', [26])
        self.set_layers(context, 'thigh_twist_bbone_in_R', [26])
        self.set_layers(context, 'thigh_twist_def_R', [27, 31])
        self.set_layers(context, 'thigh_def_R', [27, 31])
        self.set_layers(context, 'thigh_def_bbone_out_R', [26])
        self.set_layers(context, 'shin_mstr_R', [26])
        self.set_layers(context, 'shin_def_mstr_R', [26])
        self.set_layers(context, 'shin_toon_2_mstr_R', [26])
        self.set_layers(context, 'shin_toon_2_R', [26])
        self.set_layers(context, 'shin_toon_mstr_R', [26])
        self.set_layers(context, 'shin_toon_def_R', [26])
        self.set_layers(context, 'shin_twist_R', [26])
        self.set_layers(context, 'shin_twist_bezier_out_R', [26])
        self.set_layers(context, 'shin_def_curved_R', [26])
        self.set_layers(context, 'shin_def_bezier_in_mstr_R', [26])
        self.set_layers(context, 'shin_def_bezier_in_R', [26])
        self.set_layers(context, 'shin_def_bbone_in_R', [26])
        self.set_layers(context, 'shin_def_no_twist_bbone_in_R', [26])
        self.set_layers(context, 'shin_toon_R', [7, 25])
        self.set_layers(context, 'shin_def_bezier_out_mstr_R', [26])
        self.set_layers(context, 'shin_def_bezier_out_R', [26])
        self.set_layers(context, 'shin_def_bezier_out_rot_R', [26])
        self.set_layers(context, 'shin_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'shin_twist_bezier_in_R', [26])
        self.set_layers(context, 'shin_twist_bbone_in_R', [26])
        self.set_layers(context, 'shin_twist_def_R', [27, 31])
        self.set_layers(context, 'shin_twist_bbone_out_R', [26])
        self.set_layers(context, 'shin_no_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'shin_no_twist_bezier_in_R', [26])
        self.set_layers(context, 'shin_no_twist_bbone_in_R', [26])
        self.set_layers(context, 'shin_no_twist_def_R', [27])
        self.set_layers(context, 'shin_def_R', [27, 31])
        self.set_layers(context, 'shin_def_bbone_out_R', [26])
        self.set_layers(context, 'shin_def_no_twist_R', [27])
        self.set_layers(context, 'shin_def_no_twist_bbone_out_R', [26])
        self.set_layers(context, 'thigh_twist_curved_R', [26])
        self.set_layers(context, 'thigh_twist_bezier_out_mstr_R', [26])
        self.set_layers(context, 'thigh_twist_bezier_out_R', [26])
        self.set_layers(context, 'thigh_twist_bbone_out_rot_R', [26])
        self.set_layers(context, 'thigh_twist_bbone_out_R', [26])
        self.set_layers(context, 'knee_line_R', [0, 25, 31])
        self.set_layers(context, 'shin_ik_L', [1, 25, 30])
        self.set_layers(context, 'shin_fk_L', [0, 25, 29, 30])
        self.set_layers(context, 'foot_toon_L', [7, 25])
        self.set_layers(context, 'shin_twist_bezier_out_mstr_L', [26])
        self.set_layers(context, 'foot_scale_L', [26])
        self.set_layers(context, 'foot_fk_L', [0, 25, 30])
        self.set_layers(context, 'toes_spread_L', [24])
        self.set_layers(context, 'toe_lit_str_1_L', [24])
        self.set_layers(context, 'toe_fourth_str_1_L', [24])
        self.set_layers(context, 'toe_mid_str_1_L', [24])
        self.set_layers(context, 'toe_big_str_1_L', [24])
        self.set_layers(context, 'toe_ind_str_1_L', [24])
        self.set_layers(context, 'toe_1_fk_L', [0, 24, 25, 30])
        self.set_layers(context, 'snap_toes_ctrl_mid_fk_L', [30])
        self.set_layers(context, 'toe_2_fk_L', [0, 24, 25, 30])
        self.set_layers(context, 'snap_toes_ctrl_fk_L', [30])
        self.set_layers(context, 'toes_ctrl_shp_at_L', [26])
        self.set_layers(context, 'toe_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'toe_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'toe_fix_up_1_L', [24, 27])
        self.set_layers(context, 'toe_fix_low_1_L', [24, 27])
        self.set_layers(context, 'snap_sole_ctrl_fk_L', [30])
        self.set_layers(context, 'toe_lit_ctrl_mstr_L', [26])
        self.set_layers(context, 'toe_lit_ctrl_L', [24, 29])
        self.set_layers(context, 'toe_lit_1_L', [26])
        self.set_layers(context, 'toe_lit_2_L', [24])
        self.set_layers(context, 'toe_lit_3_L', [24])
        self.set_layers(context, 'toe_lit_shp_at_L', [26])
        self.set_layers(context, 'toe_lit_2_rot_L', [26])
        self.set_layers(context, 'toe_lit_1_def_L', [24, 29])
        self.set_layers(context, 'toe_lit_fix_up_1_L', [24])
        self.set_layers(context, 'toe_lit_fix_low_1_L', [24])
        self.set_layers(context, 'toe_lit_2_def_mstr_L', [26])
        self.set_layers(context, 'toe_lit_2_def_L', [24, 29])
        self.set_layers(context, 'toe_lit_fix_up_2_L', [24])
        self.set_layers(context, 'toe_lit_fix_low_2_L', [24])
        self.set_layers(context, 'toe_lit_3_def_mstr_L', [26])
        self.set_layers(context, 'toe_lit_3_def_L', [24, 29])
        self.set_layers(context, 'toe_lit_ctrl_shp_at_L', [26])
        self.set_layers(context, 'toe_big_ctrl_mstr_L', [26])
        self.set_layers(context, 'toe_big_ctrl_L', [24, 29])
        self.set_layers(context, 'toe_big_1_L', [26])
        self.set_layers(context, 'toe_big_2_L', [24])
        self.set_layers(context, 'toe_big_3_L', [24])
        self.set_layers(context, 'toe_big_shp_at_L', [26])
        self.set_layers(context, 'toe_big_ctrl_shp_at_L', [26])
        self.set_layers(context, 'toe_big_2_rot_L', [26])
        self.set_layers(context, 'toe_big_1_def_L', [24, 29])
        self.set_layers(context, 'toe_big_fix_up_1_L', [24])
        self.set_layers(context, 'toe_big_fix_low_1_L', [24])
        self.set_layers(context, 'toe_big_2_def_mstr_L', [26])
        self.set_layers(context, 'toe_big_2_def_L', [24, 29])
        self.set_layers(context, 'toe_big_fix_up_2_L', [24])
        self.set_layers(context, 'toe_big_fix_low_2_L', [24])
        self.set_layers(context, 'toe_big_3_def_mstr_L', [26])
        self.set_layers(context, 'toe_big_3_def_L', [24, 29])
        self.set_layers(context, 'toe_fourth_ctrl_mstr_L', [26])
        self.set_layers(context, 'toe_fourth_ctrl_L', [24, 29])
        self.set_layers(context, 'toe_fourth_1_L', [26])
        self.set_layers(context, 'toe_fourth_2_L', [24])
        self.set_layers(context, 'toe_fourth_3_L', [24])
        self.set_layers(context, 'toe_fourth_4_L', [24])
        self.set_layers(context, 'toe_fourth_shp_at_L', [26])
        self.set_layers(context, 'toe_fourth_2_rot_L', [26])
        self.set_layers(context, 'toe_fourth_1_def_L', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_1_L', [24])
        self.set_layers(context, 'toe_fourth_fix_low_1_L', [24])
        self.set_layers(context, 'toe_fourth_2_def_mstr_L', [26])
        self.set_layers(context, 'toe_fourth_2_def_L', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_2_L', [24])
        self.set_layers(context, 'toe_fourth_fix_low_2_L', [24])
        self.set_layers(context, 'toe_fourth_3_def_mstr_L', [26])
        self.set_layers(context, 'toe_fourth_3_def_L', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_3_L', [24])
        self.set_layers(context, 'toe_fourth_fix_low_3_L', [24])
        self.set_layers(context, 'toe_fourth_4_def_mstr_L', [26])
        self.set_layers(context, 'toe_fourth_4_def_L', [24, 29])
        self.set_layers(context, 'toe_fourth_ctrl_shp_at_L', [26])
        self.set_layers(context, 'toe_mid_ctrl_mstr_L', [26])
        self.set_layers(context, 'toe_mid_ctrl_L', [24, 29])
        self.set_layers(context, 'toe_mid_1_L', [26])
        self.set_layers(context, 'toe_mid_2_L', [24])
        self.set_layers(context, 'toe_mid_3_L', [24])
        self.set_layers(context, 'toe_mid_4_L', [24])
        self.set_layers(context, 'toe_mid_shp_at_L', [26])
        self.set_layers(context, 'toe_mid_2_rot_L', [26])
        self.set_layers(context, 'toe_mid_ctrl_shp_at_L', [26])
        self.set_layers(context, 'toe_mid_1_def_L', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_1_L', [24])
        self.set_layers(context, 'toe_mid_fix_low_1_L', [24])
        self.set_layers(context, 'toe_mid_2_def_mstr_L', [26])
        self.set_layers(context, 'toe_mid_2_def_L', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_2_L', [24])
        self.set_layers(context, 'toe_mid_fix_low_2_L', [24])
        self.set_layers(context, 'toe_mid_3_def_mstr_L', [26])
        self.set_layers(context, 'toe_mid_3_def_L', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_3_L', [24])
        self.set_layers(context, 'toe_mid_fix_low_3_L', [24])
        self.set_layers(context, 'toe_mid_4_def_mstr_L', [26])
        self.set_layers(context, 'toe_mid_4_def_L', [24, 29])
        self.set_layers(context, 'toe_ind_ctrl_mstr_L', [26])
        self.set_layers(context, 'toe_ind_ctrl_L', [24, 29])
        self.set_layers(context, 'toe_ind_1_L', [26])
        self.set_layers(context, 'toe_ind_2_L', [24])
        self.set_layers(context, 'toe_ind_3_L', [24])
        self.set_layers(context, 'toe_ind_4_L', [24])
        self.set_layers(context, 'toe_ind_shp_at_L', [26])
        self.set_layers(context, 'toe_ind_2_rot_L', [26])
        self.set_layers(context, 'toe_ind_1_def_L', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_1_L', [24])
        self.set_layers(context, 'toe_ind_fix_low_1_L', [24])
        self.set_layers(context, 'toe_ind_2_def_mstr_L', [26])
        self.set_layers(context, 'toe_ind_2_def_L', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_2_L', [24])
        self.set_layers(context, 'toe_ind_fix_low_2_L', [24])
        self.set_layers(context, 'toe_ind_3_def_mstr_L', [26])
        self.set_layers(context, 'toe_ind_3_def_L', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_3_L', [24])
        self.set_layers(context, 'toe_ind_fix_low_3_L', [24])
        self.set_layers(context, 'toe_ind_4_def_mstr_L', [26])
        self.set_layers(context, 'toe_ind_4_def_L', [24, 29])
        self.set_layers(context, 'toe_ind_ctrl_shp_at_L', [26])
        self.set_layers(context, 'shin_rot_L', [26])
        self.set_layers(context, 'foot_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'foot_drv_L', [26])
        self.set_layers(context, 'thigh_def_mstr_L', [26])
        self.set_layers(context, 'knee_toon_L', [6, 25])
        self.set_layers(context, 'knee_toon_loc_L', [26])
        self.set_layers(context, 'thigh_toon_mstr_L', [26])
        self.set_layers(context, 'thigh_drv_L', [26])
        self.set_layers(context, 'buttock_ctrl_L', [1, 26, 31])
        self.set_layers(context, 'buttock_def_L', [27, 31])
        self.set_layers(context, 'hip_fix_L', [27])
        self.set_layers(context, 'thigh_fix_L', [27])
        self.set_layers(context, 'thigh_twist_L', [26])
        self.set_layers(context, 'thigh_def_bezier_in_L', [26])
        self.set_layers(context, 'thigh_def_bbone_in_L', [26])
        self.set_layers(context, 'knee_fix_L', [27])
        self.set_layers(context, 'shin_fix_L', [27])
        self.set_layers(context, 'thigh_toon_L', [7, 25])
        self.set_layers(context, 'thigh_def_bezier_out_mstr_L', [26])
        self.set_layers(context, 'thigh_def_bezier_out_L', [26])
        self.set_layers(context, 'thigh_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'thigh_twist_bezier_in_L', [26])
        self.set_layers(context, 'thigh_twist_bbone_in_L', [26])
        self.set_layers(context, 'thigh_twist_def_L', [27, 31])
        self.set_layers(context, 'thigh_def_L', [27, 31])
        self.set_layers(context, 'thigh_def_bbone_out_L', [26])
        self.set_layers(context, 'shin_mstr_L', [26])
        self.set_layers(context, 'shin_def_mstr_L', [26])
        self.set_layers(context, 'shin_toon_2_mstr_L', [26])
        self.set_layers(context, 'shin_toon_2_L', [26])
        self.set_layers(context, 'shin_toon_mstr_L', [26])
        self.set_layers(context, 'shin_toon_def_L', [26])
        self.set_layers(context, 'shin_twist_L', [26])
        self.set_layers(context, 'shin_twist_bezier_out_L', [26])
        self.set_layers(context, 'shin_def_curved_L', [26])
        self.set_layers(context, 'shin_def_bezier_in_mstr_L', [26])
        self.set_layers(context, 'shin_def_bezier_in_L', [26])
        self.set_layers(context, 'shin_def_bbone_in_L', [26])
        self.set_layers(context, 'shin_def_no_twist_bbone_in_L', [26])
        self.set_layers(context, 'shin_toon_L', [7, 25])
        self.set_layers(context, 'shin_def_bezier_out_mstr_L', [26])
        self.set_layers(context, 'shin_def_bezier_out_L', [26])
        self.set_layers(context, 'shin_def_bezier_out_rot_L', [26])
        self.set_layers(context, 'shin_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'shin_twist_bezier_in_L', [26])
        self.set_layers(context, 'shin_twist_bbone_in_L', [26])
        self.set_layers(context, 'shin_twist_def_L', [27, 31])
        self.set_layers(context, 'shin_twist_bbone_out_L', [26])
        self.set_layers(context, 'shin_no_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'shin_no_twist_bezier_in_L', [26])
        self.set_layers(context, 'shin_no_twist_bbone_in_L', [26])
        self.set_layers(context, 'shin_no_twist_def_L', [27])
        self.set_layers(context, 'shin_def_L', [27, 31])
        self.set_layers(context, 'shin_def_bbone_out_L', [26])
        self.set_layers(context, 'shin_def_no_twist_L', [27])
        self.set_layers(context, 'shin_def_no_twist_bbone_out_L', [26])
        self.set_layers(context, 'thigh_twist_curved_L', [26])
        self.set_layers(context, 'thigh_twist_bezier_out_mstr_L', [26])
        self.set_layers(context, 'thigh_twist_bezier_out_L', [26])
        self.set_layers(context, 'thigh_twist_bbone_out_rot_L', [26])
        self.set_layers(context, 'thigh_twist_bbone_out_L', [26])
        self.set_layers(context, 'knee_line_L', [0, 25, 31])
        self.set_layers(context, 'look_mstr', [26])
        self.set_layers(context, 'look', [0, 25])
        self.set_layers(context, 'look_L', [0, 25, 31])
        self.set_layers(context, 'eye_ctrl_L', [0, 25])
        self.set_layers(context, 'pupil_ctrl_L', [0, 25])
        self.set_layers(context, 'iris_ctrl_L', [0, 25])
        self.set_layers(context, 'eye_mstr_str_L', [29, 31])
        self.set_layers(context, 'toon_eye_up_L', [7, 25])
        self.set_layers(context, 'toon_eye_out_L', [7, 25])
        self.set_layers(context, 'toon_eye_in_L', [7, 25])
        self.set_layers(context, 'toon_eye_low_L', [7, 25])
        self.set_layers(context, 'eye_mstr_vert_roll_L', [29])
        self.set_layers(context, 'eye_def_L', [27])
        self.set_layers(context, 'lattice_eye_L', [7, 25, 26])
        self.set_layers(context, 'eyelid_up_ctrl_L', [3, 25, 28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_2_L', [4, 25])
        self.set_layers(context, 'eyelid_up_vert_def_2_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_up_vert_def_2_L', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_2_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_1_L', [4, 25])
        self.set_layers(context, 'eyelid_up_vert_def_1_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_up_3_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_3_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_up_3_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_up_1_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_1_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_up_vert_def_1_L', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_1_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_2_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_up_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_3_L', [4, 25])
        self.set_layers(context, 'eyelid_up_vert_def_3_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_up_2_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_up_4_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_4_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_up_4_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_up_vert_def_3_L', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_3_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_3_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_up_3_def_L', [27, 31])
        self.set_layers(context, 'eyelid_low_ctrl_L', [3, 25, 28, 31])
        self.set_layers(context, 'eyelid_ctrl_out_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_ctrl_out_L', [4, 25])
        self.set_layers(context, 'eyelid_out_vert_def_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_out_vert_def_L', [27, 31])
        self.set_layers(context, 'eyelid_out_vert_def_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_3_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_3_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_up_4_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_up_3_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_4_def_L', [27, 31])
        self.set_layers(context, 'eyelid_up_4_L_bbone_out_L', [26, 31])
        self.set_layers(context, 'eyelid_low_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_2_L', [4, 25])
        self.set_layers(context, 'eyelid_low_vert_def_2_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_low_vert_def_2_L', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_2_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_1_L', [4, 25])
        self.set_layers(context, 'eyelid_low_vert_def_1_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_low_1_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_1_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_low_3_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_low_2_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_low_vert_def_1_L', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_1_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_3_L', [4, 25])
        self.set_layers(context, 'eyelid_low_vert_def_3_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_low_4_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_4_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_low_4_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_low_vert_def_3_L', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_3_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_3_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_low_3_def_L', [27, 31])
        self.set_layers(context, 'eyelid_low_3_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_low_4_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_low_3_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_4_def_L', [27, 31])
        self.set_layers(context, 'eyelid_low_4_L_bbone_out_L', [26, 31])
        self.set_layers(context, 'eyelid_ctrl_in_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_ctrl_in_L', [4, 25])
        self.set_layers(context, 'eyelid_in_vert_def_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_in_vert_def_L', [27, 31])
        self.set_layers(context, 'eyelid_in_vert_def_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_low_2_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_low_2_def_L', [27, 31])
        self.set_layers(context, 'eyelid_low_2_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_1_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_low_1_L_bezier_in_mstr_L', [26, 31])
        self.set_layers(context, 'eyelid_low_1_L_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_low_1_L_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_low_1_def_L', [27, 31])
        self.set_layers(context, 'eyelid_low_1_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_up_2_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_up_2_def_L', [27, 31])
        self.set_layers(context, 'eyelid_up_2_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_1_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_up_1_L_bezier_in_mstr_L', [26, 31])
        self.set_layers(context, 'eyelid_up_1_L_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_up_1_L_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_up_1_def_L', [27, 31])
        self.set_layers(context, 'eyelid_up_1_bbone_out_L', [26])
        self.set_layers(context, 'look_R', [0, 25, 31])
        self.set_layers(context, 'eye_ctrl_R', [0, 25])
        self.set_layers(context, 'pupil_ctrl_R', [0, 25])
        self.set_layers(context, 'iris_ctrl_R', [0, 25])
        self.set_layers(context, 'eye_mstr_str_R', [29, 31])
        self.set_layers(context, 'toon_eye_up_R', [7, 25])
        self.set_layers(context, 'toon_eye_out_R', [7, 25])
        self.set_layers(context, 'toon_eye_in_R', [7, 25])
        self.set_layers(context, 'toon_eye_low_R', [7, 25])
        self.set_layers(context, 'eye_mstr_vert_roll_R', [29])
        self.set_layers(context, 'eye_def_R', [27])
        self.set_layers(context, 'lattice_eye_R', [7, 25, 26])
        self.set_layers(context, 'eyelid_low_ctrl_R', [3, 25, 28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_R', [3, 25, 28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_2_R', [4, 25])
        self.set_layers(context, 'eyelid_low_vert_def_2_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_low_vert_def_2_R', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_2_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_ctrl_2_R', [4, 25])
        self.set_layers(context, 'eyelid_up_vert_def_2_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_up_vert_def_2_R', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_2_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_1_R', [4, 25])
        self.set_layers(context, 'eyelid_low_vert_def_1_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_low_1_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_1_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_low_3_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_low_vert_def_1_R', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_1_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_2_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_up_ctrl_1_R', [4, 25])
        self.set_layers(context, 'eyelid_up_vert_def_1_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_up_3_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_3_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_up_3_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_up_1_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_1_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_up_vert_def_1_R', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_1_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_2_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_up_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_3_R', [4, 25])
        self.set_layers(context, 'eyelid_low_vert_def_3_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_low_4_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_4_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_low_4_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_low_vert_def_3_R', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_3_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_3_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_low_3_def_R', [27, 31])
        self.set_layers(context, 'eyelid_up_ctrl_3_R', [4, 25])
        self.set_layers(context, 'eyelid_up_vert_def_3_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_up_2_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_up_4_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_4_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_up_4_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_up_vert_def_3_R', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_3_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_3_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_up_3_def_R', [27, 31])
        self.set_layers(context, 'eyelid_ctrl_in_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_ctrl_in_R', [4, 25])
        self.set_layers(context, 'eyelid_in_vert_def_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_in_vert_def_R', [27, 31])
        self.set_layers(context, 'eyelid_in_vert_def_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_low_2_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_low_2_def_R', [27, 31])
        self.set_layers(context, 'eyelid_low_2_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_1_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_low_1_L_bezier_in_mstr_R', [26, 31])
        self.set_layers(context, 'eyelid_low_1_L_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_low_1_L_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_low_1_def_R', [27, 31])
        self.set_layers(context, 'eyelid_low_1_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_up_2_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_up_2_def_R', [27, 31])
        self.set_layers(context, 'eyelid_up_2_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_1_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_up_1_L_bezier_in_mstr_R', [26, 31])
        self.set_layers(context, 'eyelid_up_1_L_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_up_1_L_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_up_1_def_R', [27, 31])
        self.set_layers(context, 'eyelid_up_1_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_ctrl_out_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_ctrl_out_R', [4, 25])
        self.set_layers(context, 'eyelid_out_vert_def_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_out_vert_def_R', [27, 31])
        self.set_layers(context, 'eyelid_out_vert_def_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_low_4_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_low_3_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_4_def_R', [27, 31])
        self.set_layers(context, 'eyelid_low_4_L_bbone_out_R', [26, 31])
        self.set_layers(context, 'eyelid_up_3_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_3_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_up_4_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_up_3_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_4_def_R', [27, 31])
        self.set_layers(context, 'eyelid_up_4_L_bbone_out_R', [26, 31])
        self.set_layers(context, 'hat_free', [17, 25, 31])
        self.set_layers(context, 'hat_head', [26])
        self.set_layers(context, 'hat_mstr', [17, 25])
        self.set_layers(context, 'eyeglasses_free', [17, 25, 31])
        self.set_layers(context, 'eyeglasses_head', [26])
        self.set_layers(context, 'eyeglasses_mstr', [17, 25])
        self.set_layers(context, 'accessory_mstr', [26])
        self.set_layers(context, 'accessory', [17, 25])
        
    def set_data_props(self, context):
        scene = bpy.context.view_layer      
        bpy.context.active_object.data['bone_auto_hide'] = 1.0
        bpy.context.active_object.data['custom_layers'] = 0.0
        bpy.context.active_object.data['layers_count'] = 10
        bpy.context.active_object.data['layer_list'] = 'BODY, BODY 2, FINGERS - TOES, FACIAL 1, FACIAL 2, FACIAL 3, TOON 1, TOON 2, SCALE, EXTRAS, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, OPTIONALS, PROTECTED, MECH, DEFORMATION, ACTIONS, BONE-ROLLS, SNAPPING, REPROPORTION'
        scene.update()         

    def update_scene(self, context):
        current = bpy.context.scene.frame_current
        bpy.context.scene.frame_set(2, subframe=0)
        bpy.context.scene.frame_set(current, subframe=0)

    def execute(self, context):
        self.compact_layers(context)   
        self.set_data_props(context) 
        self.update_scene(context) 

        return {'FINISHED'}

######## Expanded Scheme ########    
class Operator_BlenRig_Layers_Scheme_Expanded(bpy.types.Operator):    
    
    bl_idname = "blenrig5.layers_scheme_expanded"   
    bl_label = "BlenRig Expanded Layers Scheme"   
    bl_description = "Organize layers by body parts and FK / IK "    
    bl_options = {'REGISTER', 'UNDO', 'INTERNAL'}      

    @classmethod
    def poll(cls, context):
        if not bpy.context.object:
            return False
        else:
            return (bpy.context.object.type=='ARMATURE' and context.mode=='POSE')
        for prop in bpy.context.object.data.items():
            if prop[0] == 'rig_type' and prop[1] == 'Biped':              
                return True 

    def set_layers(self, context, N, L):

        pbones = bpy.context.active_object.pose.bones

        for b in pbones:
            layers =  L
            if b.name == N: 
                b.bone.layers = [(x in layers) for x in range(32)]

    def expanded_layers(self, context):
        self.set_layers(context, 'master', [8, 18, 21, 25, 31])
        self.set_layers(context, 'master_pivot_point', [8, 18, 21, 25])
        self.set_layers(context, 'master_pivot', [26])
        self.set_layers(context, 'floor_L', [11, 25])
        self.set_layers(context, 'floor_R', [11, 25])
        self.set_layers(context, 'master_body', [11, 25])
        self.set_layers(context, 'master_body_pivot_point', [11, 25])
        self.set_layers(context, 'master_body_pivot', [26])
        self.set_layers(context, 'knee_pole_str_R', [31])
        self.set_layers(context, 'knee_str_R', [31])
        self.set_layers(context, 'knee_pole_str_L', [31])
        self.set_layers(context, 'knee_str_L', [31])
        self.set_layers(context, 'properties', [11, 25])
        self.set_layers(context, 'properties_head', [11, 25])
        self.set_layers(context, 'flex_neck_length', [26])
        self.set_layers(context, 'flex_neck_width', [26])
        self.set_layers(context, 'properties_torso', [11, 25])
        self.set_layers(context, 'flex_chest_width', [26])
        self.set_layers(context, 'flex_waist_width', [26])
        self.set_layers(context, 'flex_pelvis_width', [26])
        self.set_layers(context, 'flex_ribs_width', [26])
        self.set_layers(context, 'properties_arm_L', [11, 25])
        self.set_layers(context, 'flex_arm_width_L', [26])
        self.set_layers(context, 'flex_forearm_width_L', [26])
        self.set_layers(context, 'flex_arm_length_L', [26])
        self.set_layers(context, 'flex_forearm_length_L', [26])
        self.set_layers(context, 'flex_hand_scale_L', [26])
        self.set_layers(context, 'properties_arm_R', [11, 25])
        self.set_layers(context, 'flex_arm_width_R', [26])
        self.set_layers(context, 'flex_forearm_width_R', [26])
        self.set_layers(context, 'flex_arm_length_R', [26])
        self.set_layers(context, 'flex_forearm_length_R', [26])
        self.set_layers(context, 'flex_hand_scale_R', [26])
        self.set_layers(context, 'properties_leg_R', [11, 25])
        self.set_layers(context, 'flex_thigh_width_R', [26])
        self.set_layers(context, 'flex_thigh_length_R', [26])
        self.set_layers(context, 'flex_shin_length_R', [26])
        self.set_layers(context, 'flex_shin_width_R', [26])
        self.set_layers(context, 'flex_foot_scale_R', [26])
        self.set_layers(context, 'properties_leg_L', [11, 25])
        self.set_layers(context, 'flex_thigh_width_L', [26])
        self.set_layers(context, 'flex_thigh_length_L', [26])
        self.set_layers(context, 'flex_shin_length_L', [26])
        self.set_layers(context, 'flex_shin_width_L', [26])
        self.set_layers(context, 'flex_foot_scale_L', [26])
        self.set_layers(context, 'master_torso', [8, 18, 21, 25, 31])
        self.set_layers(context, 'master_torso_pivot_point', [8, 18, 21, 25])
        self.set_layers(context, 'master_torso_pivot', [26])
        self.set_layers(context, 'spine_ctrl_4_str', [31])
        self.set_layers(context, 'shoulder_str_R', [31])
        self.set_layers(context, 'shoulder_str_L', [31])
        self.set_layers(context, 'clavi_str_R', [31])
        self.set_layers(context, 'clavi_str_L', [31])
        self.set_layers(context, 'neck_ctrl_4_str', [31])
        self.set_layers(context, 'head_str', [31])
        self.set_layers(context, 'look_str_loc', [31])
        self.set_layers(context, 'look_body', [26])
        self.set_layers(context, 'neck_ctrl_3_str', [31])
        self.set_layers(context, 'neck_ctrl_2_str', [31])
        self.set_layers(context, 'torso_ctrl_inv_str', [31])
        self.set_layers(context, 'spine_ctrl_3_str', [31])
        self.set_layers(context, 'torso_fk_ctrl', [18, 25, 30])
        self.set_layers(context, 'spine_ctrl_2_str', [31])
        self.set_layers(context, 'torso_ctrl_str', [31])
        self.set_layers(context, 'spine_ctrl_1_str', [31])
        self.set_layers(context, 'torso_ik_ctrl', [21, 25, 30])
        self.set_layers(context, 'wrist_str_R', [31])
        self.set_layers(context, 'hand_str_R', [31])
        self.set_layers(context, 'fing_lit_str_2_R', [24, 31])
        self.set_layers(context, 'fing_lit_str_3_R', [24, 31])
        self.set_layers(context, 'fing_lit_str_4_R', [24, 31])
        self.set_layers(context, 'fing_lit_str_5_R', [24, 31])
        self.set_layers(context, 'fing_ring_str_2_R', [24, 31])
        self.set_layers(context, 'fing_ring_str_3_R', [24, 31])
        self.set_layers(context, 'fing_ring_str_4_R', [24, 31])
        self.set_layers(context, 'fing_ring_str_5_R', [24, 31])
        self.set_layers(context, 'fing_ind_str_2_R', [24, 31])
        self.set_layers(context, 'fing_ind_str_3_R', [24, 31])
        self.set_layers(context, 'fing_ind_str_4_R', [24, 31])
        self.set_layers(context, 'fing_ind_str_5_R', [24, 31])
        self.set_layers(context, 'fing_mid_str_2_R', [24, 31])
        self.set_layers(context, 'fing_mid_str_3_R', [24, 31])
        self.set_layers(context, 'fing_mid_str_4_R', [24, 31])
        self.set_layers(context, 'fing_mid_str_5_R', [24, 31])
        self.set_layers(context, 'fing_thumb_str_2_R', [24, 31])
        self.set_layers(context, 'fing_thumb_str_3_R', [24, 31])
        self.set_layers(context, 'fing_thumb_str_4_R', [24, 31])
        self.set_layers(context, 'fing_thumb_str_1_R', [24, 31])
        self.set_layers(context, 'fing_lit_str_1_R', [24, 31])
        self.set_layers(context, 'fing_ring_str_1_R', [24, 31])
        self.set_layers(context, 'fing_ind_str_1_R', [24, 31])
        self.set_layers(context, 'fing_mid_str_1_R', [24, 31])
        self.set_layers(context, 'wrist_str_L', [31])
        self.set_layers(context, 'hand_str_L', [31])
        self.set_layers(context, 'fing_lit_str_2_L', [24, 31])
        self.set_layers(context, 'fing_lit_str_3_L', [24, 31])
        self.set_layers(context, 'fing_lit_str_4_L', [24, 31])
        self.set_layers(context, 'fing_lit_str_5_L', [24, 31])
        self.set_layers(context, 'fing_ring_str_2_L', [24, 31])
        self.set_layers(context, 'fing_ring_str_3_L', [24, 31])
        self.set_layers(context, 'fing_ring_str_4_L', [24, 31])
        self.set_layers(context, 'fing_ring_str_5_L', [24, 31])
        self.set_layers(context, 'fing_ind_str_2_L', [24, 31])
        self.set_layers(context, 'fing_ind_str_3_L', [24, 31])
        self.set_layers(context, 'fing_ind_str_4_L', [24, 31])
        self.set_layers(context, 'fing_ind_str_5_L', [24, 31])
        self.set_layers(context, 'fing_mid_str_2_L', [24, 31])
        self.set_layers(context, 'fing_mid_str_3_L', [24, 31])
        self.set_layers(context, 'fing_mid_str_4_L', [24, 31])
        self.set_layers(context, 'fing_mid_str_5_L', [24, 31])
        self.set_layers(context, 'fing_thumb_str_2_L', [24, 31])
        self.set_layers(context, 'fing_thumb_str_3_L', [24, 31])
        self.set_layers(context, 'fing_thumb_str_4_L', [24, 31])
        self.set_layers(context, 'fing_thumb_str_1_L', [24, 31])
        self.set_layers(context, 'fing_lit_str_1_L', [24, 31])
        self.set_layers(context, 'fing_ring_str_1_L', [24, 31])
        self.set_layers(context, 'fing_ind_str_1_L', [24, 31])
        self.set_layers(context, 'fing_mid_str_1_L', [24, 31])
        self.set_layers(context, 'elbow_pole_str_R', [31])
        self.set_layers(context, 'elbow_str_R', [31])
        self.set_layers(context, 'elbow_pole_str_L', [31])
        self.set_layers(context, 'elbow_str_L', [31])
        self.set_layers(context, 'spine_3_inv_ctrl', [8, 25, 30])
        self.set_layers(context, 'spine_3_inv', [8, 25, 30])
        self.set_layers(context, 'spine_4_ik_ctrl', [21, 25, 30])
        self.set_layers(context, 'spine_3_ik_ctrl', [21, 25, 30])
        self.set_layers(context, 'pelvis_str', [31])
        self.set_layers(context, 'pelvis_ctrl_str', [31])
        self.set_layers(context, 'torso_inv_ctrl', [8, 25, 30])
        self.set_layers(context, 'spine_2_inv', [8, 25, 30])
        self.set_layers(context, 'snap_spine_2_fk_inv', [30])
        self.set_layers(context, 'spine_1_inv', [8, 25, 30])
        self.set_layers(context, 'snap_spine_1_fk_inv', [30])
        self.set_layers(context, 'spine_2_ik_ctrl', [21, 25, 30])
        self.set_layers(context, 'pelvis_inv', [8, 25, 30])
        self.set_layers(context, 'snap_pelvis_ctrl_inv', [30])
        self.set_layers(context, 'pelvis_ctrl', [18, 21, 25, 30])
        self.set_layers(context, 'pelvis_ik', [26])
        self.set_layers(context, 'pelvis_toon', [13, 25])
        self.set_layers(context, 'spine_1_ik_ctrl', [21, 25, 30])
        self.set_layers(context, 'pelvis', [30, 31])
        self.set_layers(context, 'spine_1_mstr', [26])
        self.set_layers(context, 'look_torso', [26])
        self.set_layers(context, 'spine_3_inv_ik', [26, 30])
        self.set_layers(context, 'spine_2_inv_ik', [26, 30])
        self.set_layers(context, 'spine_1_ik', [26, 30])
        self.set_layers(context, 'spine_1_fk', [18, 25, 30])
        self.set_layers(context, 'snap_spine_1_inv_fk', [30])
        self.set_layers(context, 'spine_1_toon_mstr', [26])
        self.set_layers(context, 'spine_1_toon', [14, 25])
        self.set_layers(context, 'spine_1_def_bbone', [26])
        self.set_layers(context, 'spine_2_ik', [26, 30])
        self.set_layers(context, 'spine_3_ik', [26, 30])
        self.set_layers(context, 'spine_2_fk', [18, 25, 30])
        self.set_layers(context, 'snap_spine_2_inv_fk', [30])
        self.set_layers(context, 'spine_3_fk', [18, 25, 30])
        self.set_layers(context, 'snap_torso_ctrl_inv_loc', [30])
        self.set_layers(context, 'breath', [8, 18, 21])
        self.set_layers(context, 'snap_torso_fk_ctrl', [30])
        self.set_layers(context, 'spine_4_toon_mstr', [26])
        self.set_layers(context, 'spine_4_toon', [13, 25])
        self.set_layers(context, 'spine_3_toon_mstr', [26])
        self.set_layers(context, 'spine_3_toon', [14, 25])
        self.set_layers(context, 'spine_2_toon_mstr', [26])
        self.set_layers(context, 'spine_2_toon', [14, 25])
        self.set_layers(context, 'spine_1_def', [27, 29, 31])
        self.set_layers(context, 'spine_2_def', [27, 29, 31])
        self.set_layers(context, 'spine_3_collar', [26])
        self.set_layers(context, 'head_scale', [15, 25])
        self.set_layers(context, 'neck_base_ctrl', [26])
        self.set_layers(context, 'neck_base_hinge', [26])
        self.set_layers(context, 'neck_ik_ctrl', [7, 30])
        self.set_layers(context, 'neck_pivot_point', [7, 30])
        self.set_layers(context, 'neck_pivot', [26])
        self.set_layers(context, 'neck_fk_ctrl', [4, 25, 30])
        self.set_layers(context, 'head_ik_ctrl_mstr_hinge', [26])
        self.set_layers(context, 'head_ik_ctrl_mstr', [26])
        self.set_layers(context, 'head_ik_ctrl', [7, 25, 30])
        self.set_layers(context, 'neck_3_ik_ctrl', [7, 25, 30])
        self.set_layers(context, 'neck_2_ik_ctrl', [7, 25, 30])
        self.set_layers(context, 'neck_3_inv_ik', [26, 30])
        self.set_layers(context, 'neck_2_inv_ik', [26, 30])
        self.set_layers(context, 'neck_1_ik_ctrl', [7, 25, 30])
        self.set_layers(context, 'neck_1_ik', [26, 30])
        self.set_layers(context, 'neck_1_fk', [4, 25, 30])
        self.set_layers(context, 'neck_2_ik', [26, 30])
        self.set_layers(context, 'neck_2_fk', [4, 25, 30])
        self.set_layers(context, 'neck_2_toon_mstr', [26])
        self.set_layers(context, 'neck_2_toon', [14, 25])
        self.set_layers(context, 'neck_3_ik', [26, 30])
        self.set_layers(context, 'neck_3_fk', [4, 25, 30])
        self.set_layers(context, 'neck_3_toon_mstr', [26])
        self.set_layers(context, 'neck_3_toon', [14, 25])
        self.set_layers(context, 'neck_4_toon_mstr', [26])
        self.set_layers(context, 'neck_4_toon', [13, 25])
        self.set_layers(context, 'head_fk_mstr', [26, 30])
        self.set_layers(context, 'head_fk', [4, 25, 30])
        self.set_layers(context, 'head_stretch', [0, 25])
        self.set_layers(context, 'snap_head_ik', [30])
        self.set_layers(context, 'head_def_mstr', [27, 29, 31])
        self.set_layers(context, 'face_mstr_str', [31])
        self.set_layers(context, 'look_head', [26])
        self.set_layers(context, 'head_toon', [13, 25])
        self.set_layers(context, 'head_toon_mstr', [26])
        self.set_layers(context, 'face_toon_low', [13, 25])
        self.set_layers(context, 'head_mid_2_str', [31])
        self.set_layers(context, 'face_toon_mid', [14, 25])
        self.set_layers(context, 'face_toon_up', [13, 25])
        self.set_layers(context, 'head_mid_1_str', [31])
        self.set_layers(context, 'head_mid_ctrl', [0, 25])
        self.set_layers(context, 'head_top_ctrl', [0, 25])
        self.set_layers(context, 'head_mid_curve', [0, 25])
        self.set_layers(context, 'head_low_stretch', [2, 25])
        self.set_layers(context, 'head_mid_stretch', [2, 25])
        self.set_layers(context, 'head_def_3', [27, 31])
        self.set_layers(context, 'face_mstr_str_3', [26])
        self.set_layers(context, 'brow_mstr_str_L', [29, 31])
        self.set_layers(context, 'brow_ctrl_L', [0, 25])
        self.set_layers(context, 'toon_brow_L', [14, 25])
        self.set_layers(context, 'brow_mstr_vert_roll_L', [29])
        self.set_layers(context, 'brow_mstr_str_R', [29, 31])
        self.set_layers(context, 'brow_ctrl_R', [0, 25])
        self.set_layers(context, 'toon_brow_R', [14, 25])
        self.set_layers(context, 'brow_mstr_vert_roll_R', [29])
        self.set_layers(context, 'nose_frown_str_R', [31])
        self.set_layers(context, 'nose_frown_str_L', [31])
        self.set_layers(context, 'cheek2_lev_1_str_R', [31])
        self.set_layers(context, 'cheek2_lev_1_str_L', [31])
        self.set_layers(context, 'cheek2_lev_2_str_R', [31])
        self.set_layers(context, 'cheek2_lev_2_str_L', [31])
        self.set_layers(context, 'lev2_str_L', [31])
        self.set_layers(context, 'lev2_str_R', [31])
        self.set_layers(context, 'nose_up_str', [31])
        self.set_layers(context, 'nose_bridge_1_ctrl', [2, 25])
        self.set_layers(context, 'head_def_2', [27, 31])
        self.set_layers(context, 'ear_L', [0, 25, 27, 29, 31])
        self.set_layers(context, 'ear_up_L', [0, 25, 27, 29, 31])
        self.set_layers(context, 'ear_low_L', [0, 25, 27, 29, 31])
        self.set_layers(context, 'ear_R', [0, 25, 27, 29, 31])
        self.set_layers(context, 'ear_up_R', [0, 25, 27, 29, 31])
        self.set_layers(context, 'ear_low_R', [0, 25, 27, 29, 31])
        self.set_layers(context, 'face_mstr_str_2', [26])
        self.set_layers(context, 'nose_frown_ctrl_L', [0, 25, 31])
        self.set_layers(context, 'brow_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'brow_ctrl_1_L', [1, 25])
        self.set_layers(context, 'brow_up_1_def_L', [27, 31])
        self.set_layers(context, 'brow_low_1_bezier_out_L', [26])
        self.set_layers(context, 'brow_low_1_def_L', [27, 31])
        self.set_layers(context, 'brow_low_1_bbone_out_L', [26])
        self.set_layers(context, 'brow_up_1_bezier_out_L', [26])
        self.set_layers(context, 'brow_up_1_bbone_out_L', [26])
        self.set_layers(context, 'nose_frown_ctrl_R', [0, 25, 31])
        self.set_layers(context, 'brow_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'brow_ctrl_1_R', [1, 25])
        self.set_layers(context, 'brow_up_1_def_R', [27, 31])
        self.set_layers(context, 'brow_low_1_bezier_out_R', [26])
        self.set_layers(context, 'brow_low_1_def_R', [27, 31])
        self.set_layers(context, 'brow_low_1_bbone_out_R', [26])
        self.set_layers(context, 'brow_up_1_bezier_out_R', [26])
        self.set_layers(context, 'brow_up_1_bbone_out_R', [26])
        self.set_layers(context, 'frown_ctrl_mstr', [28, 31])
        self.set_layers(context, 'frown_ctrl', [1, 25])
        self.set_layers(context, 'mouth_str_loc_1', [31])
        self.set_layers(context, 'cheek_ctrl_2_str_R', [31])
        self.set_layers(context, 'cheek_ctrl_3_str_R', [31])
        self.set_layers(context, 'cheek_ctrl_2_str_L', [31])
        self.set_layers(context, 'cheek_ctrl_3_str_L', [31])
        self.set_layers(context, 'nose_low_str', [31])
        self.set_layers(context, 'nose_mstr', [26])
        self.set_layers(context, 'nose_bridge_2_ctrl', [1, 25, 31])
        self.set_layers(context, 'nose_bridge_1_rot', [26])
        self.set_layers(context, 'head_def_1', [27, 31])
        self.set_layers(context, 'face_mstr_str_1', [26])
        self.set_layers(context, 'uvula_1', [0])
        self.set_layers(context, 'uvula_2', [0])
        self.set_layers(context, 'maxi_str_2', [31])
        self.set_layers(context, 'maxi_str_1', [31])
        self.set_layers(context, 'maxi_str_mstr', [26, 31])
        self.set_layers(context, 'tongue_str', [31])
        self.set_layers(context, 'jaw_str_R', [31])
        self.set_layers(context, 'jaw_str_L', [31])
        self.set_layers(context, 'lev3_str_L', [31])
        self.set_layers(context, 'lev3_str_R', [31])
        self.set_layers(context, 'mouth_mstr_str', [31])
        self.set_layers(context, 'mouth_mstr_low_str', [29, 31])
        self.set_layers(context, 'mouth_mstr_up_str', [29, 31])
        self.set_layers(context, 'mouth_corner_str_L', [31])
        self.set_layers(context, 'mouth_corner_str_R', [31])
        self.set_layers(context, 'mouth_ctrl_str', [31])
        self.set_layers(context, 'mouth_str_loc_2', [31])
        self.set_layers(context, 'mouth_str_ctrl', [0, 25])
        self.set_layers(context, 'nose_ctrl', [0, 25, 27])
        self.set_layers(context, 'nostril_ctrl_L', [0, 25, 28, 31])
        self.set_layers(context, 'nostril_def_L', [27, 31])
        self.set_layers(context, 'nostril_ctrl_R', [0, 25, 28, 31])
        self.set_layers(context, 'nostril_def_R', [27, 31])
        self.set_layers(context, 'nose_bridge_2_rot', [26])
        self.set_layers(context, 'mouth_str', [26, 31])
        self.set_layers(context, 'mouth_ctrl', [0, 25])
        self.set_layers(context, 'mouth_up_ctrl', [0, 25])
        self.set_layers(context, 'mouth_low_ctrl', [0, 25])
        self.set_layers(context, 'maxi', [0, 25, 27])
        self.set_layers(context, 'mouth_frown_ctrl_L', [1, 25, 31])
        self.set_layers(context, 'mouth_frown_ctrl_R', [1, 25, 31])
        self.set_layers(context, 'chin_ctrl', [0, 28, 31])
        self.set_layers(context, 'chin_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'chin_ctrl_mid', [2, 25])
        self.set_layers(context, 'chin_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'chin_ctrl_1_R', [2, 25])
        self.set_layers(context, 'chin_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'chin_ctrl_1_L', [2, 25])
        self.set_layers(context, 'chin_1_rot_R', [26, 29])
        self.set_layers(context, 'chin_1_bbone_rot_L', [26])
        self.set_layers(context, 'chin_1_bbone_L', [26])
        self.set_layers(context, 'chin_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'chin_ctrl_2_R', [2, 25])
        self.set_layers(context, 'chin_2_rot_R', [26, 29])
        self.set_layers(context, 'chin_1_rot_L', [26, 29])
        self.set_layers(context, 'chin_1_bbone_rot_R', [26])
        self.set_layers(context, 'chin_1_def_L', [27, 31])
        self.set_layers(context, 'chin_1_bbone_R', [26])
        self.set_layers(context, 'chin_1_def_R', [27, 31])
        self.set_layers(context, 'chin_2_def_R', [27, 31])
        self.set_layers(context, 'chin_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'chin_ctrl_2_L', [2, 25])
        self.set_layers(context, 'chin_2_rot_L', [26, 29])
        self.set_layers(context, 'chin_2_def_L', [27, 31])
        self.set_layers(context, 'tongue_mstr', [0])
        self.set_layers(context, 'tongue_1_str', [31])
        self.set_layers(context, 'tongue_2_str', [31])
        self.set_layers(context, 'tongue_1_fk', [0])
        self.set_layers(context, 'tongue_3_str', [31])
        self.set_layers(context, 'tongue_2_fk', [0])
        self.set_layers(context, 'tongue_3_ik', [0])
        self.set_layers(context, 'tongue_2_ik', [0])
        self.set_layers(context, 'tongue_rot_3', [26])
        self.set_layers(context, 'tongue_rot_4', [26])
        self.set_layers(context, 'tongue_1_ik', [0])
        self.set_layers(context, 'tongue_rot_2', [26])
        self.set_layers(context, 'tongue_rot_1', [26])
        self.set_layers(context, 'tongue_1_def', [27, 31])
        self.set_layers(context, 'tongue_2_def', [27, 31])
        self.set_layers(context, 'tongue_3_def', [27, 31])
        self.set_layers(context, 'tongue_4', [26])
        self.set_layers(context, 'mouth_mstr_low_rot', [26])
        self.set_layers(context, 'mouth_mstr_maxi_loc', [26])
        self.set_layers(context, 'maxi_drv', [26])
        self.set_layers(context, 'mouth_mstr_copy_maxi', [26])
        self.set_layers(context, 'mouth_mstr_loc', [26])
        self.set_layers(context, 'mouth_mstr_ctrl', [0, 25])
        self.set_layers(context, 'mouth_mstr_ik', [26, 31])
        self.set_layers(context, 'mouth_mstr_ik_loc', [26])
        self.set_layers(context, 'mouth_mstr_ctrl_ik_loc', [26])
        self.set_layers(context, 'mouth_mstr_ctrl_loc', [26])
        self.set_layers(context, 'mouth_mstr_pivot_point', [26])
        self.set_layers(context, 'mouth_mstr_pivot', [26])
        self.set_layers(context, 'mouth_mstr', [26])
        self.set_layers(context, 'mouth_mstr_low_loc', [26])
        self.set_layers(context, 'mouth_mstr_low_copy_transforms', [26])
        self.set_layers(context, 'mouth_mstr_low_copy_transforms_positive', [26])
        self.set_layers(context, 'mouth_mstr_low_orient', [26])
        self.set_layers(context, 'mouth_mstr_low_orient_rot', [26])
        self.set_layers(context, 'mouth_mstr_up', [2, 25, 28])
        self.set_layers(context, 'lip_up_ctrl_collision', [0, 25, 28, 31])
        self.set_layers(context, 'lip_up_ctrl', [1, 25])
        self.set_layers(context, 'lip_up_loc', [26, 31])
        self.set_layers(context, 'lip_up2_ctrl', [1, 25, 28, 31])
        self.set_layers(context, 'lip_up3_ctrl', [1, 25, 28, 31])
        self.set_layers(context, 'lip_up3_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_mid', [2, 25])
        self.set_layers(context, 'mouth_corners_mstr_str_1', [26])
        self.set_layers(context, 'mouth_mstr_low_final', [26])
        self.set_layers(context, 'mouth_mstr_low_copy_transforms_final', [26])
        self.set_layers(context, 'mouth_mstr_low', [2, 25, 28])
        self.set_layers(context, 'lip_low_ctrl_collision', [0, 25, 28, 31])
        self.set_layers(context, 'lip_low_ctrl', [1, 25])
        self.set_layers(context, 'lip_low_loc', [26, 31])
        self.set_layers(context, 'lip_low2_ctrl', [1, 25, 28, 31])
        self.set_layers(context, 'lip_low3_ctrl', [1, 25, 31])
        self.set_layers(context, 'lip_low_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_mid', [1, 25])
        self.set_layers(context, 'lip_low_mstr_mid', [26, 29])
        self.set_layers(context, 'lip_low2_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_mid', [2, 25])
        self.set_layers(context, 'lip_low3_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_mid', [2, 25])
        self.set_layers(context, 'oris_low_3_def_mid', [27, 31])
        self.set_layers(context, 'oris_low_2_def_mid', [27, 31])
        self.set_layers(context, 'oris_low_1_def_mid', [27, 31])
        self.set_layers(context, 'lip_up_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_mid', [1, 25])
        self.set_layers(context, 'mouth_corner_maxi_loc_L', [26])
        self.set_layers(context, 'mouth_corner_copy_maxi_L', [26])
        self.set_layers(context, 'mouth_corner_maxi_loc_R', [26])
        self.set_layers(context, 'mouth_corner_copy_maxi_R', [26])
        self.set_layers(context, 'mouth_corners_mstr_str_2', [26])
        self.set_layers(context, 'mouth_corners_mstr_L', [26])
        self.set_layers(context, 'mouth_corner_loc_L', [26])
        self.set_layers(context, 'mouth_corner_L', [0, 25, 31])
        self.set_layers(context, 'mouth_corner_shp_at_L', [26])
        self.set_layers(context, 'cheek_ctrl_L', [0, 25, 29, 31])
        self.set_layers(context, 'brow_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'brow_ctrl_3_L', [1, 25])
        self.set_layers(context, 'brow_up_3_def_L', [27, 31])
        self.set_layers(context, 'brow_low_3_bezier_out_L', [26])
        self.set_layers(context, 'brow_low_3_def_L', [27, 31])
        self.set_layers(context, 'brow_low_3_bbone_out_L', [26])
        self.set_layers(context, 'brow_up_3_bezier_out_L', [26])
        self.set_layers(context, 'brow_up_3_bbone_out_L', [26])
        self.set_layers(context, 'brow_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'brow_ctrl_2_L', [1, 25])
        self.set_layers(context, 'brow_1_bezier_in_mstr_L', [26])
        self.set_layers(context, 'brow_1_bezier_in_L', [26])
        self.set_layers(context, 'brow_1_bbone_in_L', [26])
        self.set_layers(context, 'brow_3_bezier_out_mstr_L', [26])
        self.set_layers(context, 'brow_3_bezier_out_L', [26])
        self.set_layers(context, 'brow_2_bezier_out_mstr_L', [26])
        self.set_layers(context, 'brow_2_bezier_out_L', [26])
        self.set_layers(context, 'brow_up_2_def_L', [27, 31])
        self.set_layers(context, 'brow_low_2_bezier_out_L', [26])
        self.set_layers(context, 'brow_low_2_def_L', [27, 31])
        self.set_layers(context, 'brow_low_2_bbone_out_L', [26])
        self.set_layers(context, 'brow_up_2_bezier_out_L', [26])
        self.set_layers(context, 'brow_up_2_bbone_out_L', [26])
        self.set_layers(context, 'brow_2_rot_L', [26, 29])
        self.set_layers(context, 'brow_1_rot_L', [26, 29])
        self.set_layers(context, 'brow_1_def_L', [27, 31])
        self.set_layers(context, 'brow_1_bbone_out_L', [26, 31])
        self.set_layers(context, 'brow_ctrl_4_mstr_L', [28, 31])
        self.set_layers(context, 'brow_ctrl_4_L', [1, 25])
        self.set_layers(context, 'brow_2_bezier_in_mstr_L', [26])
        self.set_layers(context, 'brow_2_bezier_in_L', [26])
        self.set_layers(context, 'brow_2_bbone_in_L', [26])
        self.set_layers(context, 'brow_2_def_L', [27, 31])
        self.set_layers(context, 'brow_2_bbone_out_L', [26])
        self.set_layers(context, 'brow_up_4_def_L', [27, 31])
        self.set_layers(context, 'brow_low_4_bezier_out_L', [26])
        self.set_layers(context, 'brow_low_4_def_L', [27, 31])
        self.set_layers(context, 'brow_low_4_bbone_out_L', [26])
        self.set_layers(context, 'brow_up_4_bezier_out_L', [26])
        self.set_layers(context, 'brow_up_4_bbone_out_L', [26])
        self.set_layers(context, 'brow_3_rot_L', [26, 29])
        self.set_layers(context, 'brow_3_bezier_in_mstr_L', [26, 31])
        self.set_layers(context, 'brow_3_bezier_in_L', [26])
        self.set_layers(context, 'brow_3_bbone_in_L', [26])
        self.set_layers(context, 'brow_3_def_L', [27, 31])
        self.set_layers(context, 'brow_3_bbone_out_L', [26])
        self.set_layers(context, 'cheek_ctrl_drv_L', [26])
        self.set_layers(context, 'cheek_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'cheek_ctrl_1_L', [2, 25])
        self.set_layers(context, 'nose_frown_3_def_L', [27, 31])
        self.set_layers(context, 'cheek_ctrl_2_mstr_L', [28])
        self.set_layers(context, 'cheek_ctrl_2_L', [2, 25])
        self.set_layers(context, 'lev1_3_def_L', [27, 31])
        self.set_layers(context, 'cheek_1_rot_L', [26, 29])
        self.set_layers(context, 'cheek_1_def_L', [27, 31])
        self.set_layers(context, 'cheek_ctrl_3_mstr_L', [28])
        self.set_layers(context, 'cheek_ctrl_3_L', [2, 25])
        self.set_layers(context, 'lev2_3_def_L', [27, 31])
        self.set_layers(context, 'lev2_4_def_L', [27, 31])
        self.set_layers(context, 'cheek_2_rot_L', [26, 29])
        self.set_layers(context, 'cheek_2_def_L', [27, 31])
        self.set_layers(context, 'mouth_corner_distance_drv_L', [26])
        self.set_layers(context, 'mouth_corner_drv_L', [26])
        self.set_layers(context, 'chin_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'chin_ctrl_3_L', [2, 25])
        self.set_layers(context, 'chin_3_rot_L', [26, 29])
        self.set_layers(context, 'chin_3_def_L', [27, 31])
        self.set_layers(context, 'cheek_ctrl_5_mstr_L', [28, 31])
        self.set_layers(context, 'cheek_ctrl_5_L', [2, 25])
        self.set_layers(context, 'jaw_1_rot_L', [26, 29])
        self.set_layers(context, 'jaw_1_def_L', [27, 31])
        self.set_layers(context, 'jaw_2_rot_L', [26, 29])
        self.set_layers(context, 'jaw_2_def_L', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_1_L', [1, 25])
        self.set_layers(context, 'lip_low_1_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_low_1_bezier_in_R', [26])
        self.set_layers(context, 'lip_low_1_bbone_in_R', [26])
        self.set_layers(context, 'lip_low_2_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_low_2_bezier_in_L', [26])
        self.set_layers(context, 'lip_low_2_bbone_in_L', [26])
        self.set_layers(context, 'lip_low_mstr_1_L', [26, 29])
        self.set_layers(context, 'lip_low_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_low_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_low2_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_1_L', [2, 25])
        self.set_layers(context, 'lip_low2_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_low2_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_low2_1_bbone_R', [26])
        self.set_layers(context, 'lip_low3_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_1_L', [2, 25])
        self.set_layers(context, 'lip_low3_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_low3_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_low3_1_bbone_R', [26])
        self.set_layers(context, 'oris_low1_3_def_L', [27, 31])
        self.set_layers(context, 'oris_low1_2_def_L', [27, 31])
        self.set_layers(context, 'oris_low1_1_def_L', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_2_L', [1, 25])
        self.set_layers(context, 'lip_low_1_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_low_1_bezier_out_L', [26])
        self.set_layers(context, 'lip_low_3_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_low_3_bezier_in_L', [26])
        self.set_layers(context, 'lip_low_3_bbone_in_L', [26])
        self.set_layers(context, 'lip_low_mstr_2_L', [26, 29])
        self.set_layers(context, 'lip_low_2_rot_L', [26, 29])
        self.set_layers(context, 'lip_low_2_def_L', [27, 31])
        self.set_layers(context, 'lip_low2_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_2_L', [2, 25])
        self.set_layers(context, 'lip_low2_2_rot_L', [26, 29])
        self.set_layers(context, 'lip_low3_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_2_L', [2, 25])
        self.set_layers(context, 'lip_low3_2_rot_L', [26, 29])
        self.set_layers(context, 'oris_low2_3_def_L', [27, 31])
        self.set_layers(context, 'oris_low2_2_def_L', [27, 31])
        self.set_layers(context, 'oris_low2_1_def_L', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_3_L', [1, 25])
        self.set_layers(context, 'lip_low_2_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_low_2_bezier_out_L', [26])
        self.set_layers(context, 'lip_low_4_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_low_4_bezier_in_L', [26])
        self.set_layers(context, 'lip_low_4_bbone_in_L', [26])
        self.set_layers(context, 'lip_low_3_rot_L', [26, 29])
        self.set_layers(context, 'lip_low_2_bbone_out_L', [26])
        self.set_layers(context, 'lip_low_3_def_L', [27, 31])
        self.set_layers(context, 'lip_low_mstr_3_L', [26, 29])
        self.set_layers(context, 'lip_low2_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_3_L', [2, 25])
        self.set_layers(context, 'lip_low2_3_rot_L', [26, 29])
        self.set_layers(context, 'lip_low3_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_3_L', [2, 25])
        self.set_layers(context, 'lip_low3_3_rot_L', [26, 29])
        self.set_layers(context, 'oris_low3_3_def_L', [27, 31])
        self.set_layers(context, 'oris_low3_2_def_L', [27, 31])
        self.set_layers(context, 'oris_low3_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_1_L', [1, 25])
        self.set_layers(context, 'lip_up_1_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_up_1_bezier_in_R', [26])
        self.set_layers(context, 'lip_up_1_bbone_in_R', [26])
        self.set_layers(context, 'lip_up_2_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_up_2_bezier_in_L', [26])
        self.set_layers(context, 'lip_up_2_bbone_in_L', [26])
        self.set_layers(context, 'lip_up_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_2_L', [1, 25])
        self.set_layers(context, 'lip_up_1_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_up_3_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_up_3_bezier_in_L', [26])
        self.set_layers(context, 'lip_up_3_bbone_in_L', [26])
        self.set_layers(context, 'lip_up_1_bezier_out_L', [26])
        self.set_layers(context, 'lip_up_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_3_L', [1, 25])
        self.set_layers(context, 'lip_up_4_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_up_4_bezier_in_L', [26])
        self.set_layers(context, 'lip_up_4_bbone_in_L', [26])
        self.set_layers(context, 'lip_up_2_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_up_2_bezier_out_L', [26])
        self.set_layers(context, 'lip_up3_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_1_L', [2, 25])
        self.set_layers(context, 'cheek_mstr_1_L', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_1_L', [2, 25])
        self.set_layers(context, 'nose_frown_2_def_L', [27, 31])
        self.set_layers(context, 'nose_frown_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_ctrl_4_mstr_L', [28])
        self.set_layers(context, 'lip_up_ctrl_4_L', [1, 25])
        self.set_layers(context, 'lip_low_3_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_low_3_bezier_out_L', [26])
        self.set_layers(context, 'lip_low_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_low_3_bbone_out_L', [26])
        self.set_layers(context, 'lip_low_4_def_L', [27, 31])
        self.set_layers(context, 'lip_low_4_bbone_out_L', [26])
        self.set_layers(context, 'lip_up_3_bezier_out_mstr_L', [26])
        self.set_layers(context, 'lip_up_3_bezier_out_L', [26])
        self.set_layers(context, 'lip_up_ctrl_4_loc_L', [26])
        self.set_layers(context, 'lip_up_mstr_4_str_L', [31])
        self.set_layers(context, 'mouth_corner_loc_R', [26])
        self.set_layers(context, 'mouth_corner_R', [0, 25, 31])
        self.set_layers(context, 'mouth_corner_shp_at_R', [26])
        self.set_layers(context, 'cheek_ctrl_R', [0, 25, 29, 31])
        self.set_layers(context, 'brow_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'brow_ctrl_3_R', [1, 25])
        self.set_layers(context, 'brow_up_3_def_R', [27, 31])
        self.set_layers(context, 'brow_low_3_bezier_out_R', [26])
        self.set_layers(context, 'brow_low_3_def_R', [27, 31])
        self.set_layers(context, 'brow_low_3_bbone_out_R', [26])
        self.set_layers(context, 'brow_up_3_bezier_out_R', [26])
        self.set_layers(context, 'brow_up_3_bbone_out_R', [26])
        self.set_layers(context, 'brow_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'brow_ctrl_2_R', [1, 25])
        self.set_layers(context, 'brow_1_bezier_in_mstr_R', [26])
        self.set_layers(context, 'brow_1_bezier_in_R', [26])
        self.set_layers(context, 'brow_1_bbone_in_R', [26])
        self.set_layers(context, 'brow_3_bezier_out_mstr_R', [26])
        self.set_layers(context, 'brow_3_bezier_out_R', [26])
        self.set_layers(context, 'brow_2_bezier_out_mstr_R', [26])
        self.set_layers(context, 'brow_2_bezier_out_R', [26])
        self.set_layers(context, 'brow_up_2_def_R', [27, 31])
        self.set_layers(context, 'brow_low_2_bezier_out_R', [26])
        self.set_layers(context, 'brow_low_2_def_R', [27, 31])
        self.set_layers(context, 'brow_low_2_bbone_out_R', [26])
        self.set_layers(context, 'brow_up_2_bezier_out_R', [26])
        self.set_layers(context, 'brow_up_2_bbone_out_R', [26])
        self.set_layers(context, 'brow_2_rot_R', [26, 29])
        self.set_layers(context, 'brow_1_rot_R', [26, 29])
        self.set_layers(context, 'brow_1_def_R', [27, 31])
        self.set_layers(context, 'brow_1_bbone_out_R', [26, 31])
        self.set_layers(context, 'brow_ctrl_4_mstr_R', [28, 31])
        self.set_layers(context, 'brow_ctrl_4_R', [1, 25])
        self.set_layers(context, 'brow_2_bezier_in_mstr_R', [26])
        self.set_layers(context, 'brow_2_bezier_in_R', [26])
        self.set_layers(context, 'brow_2_bbone_in_R', [26])
        self.set_layers(context, 'brow_2_def_R', [27, 31])
        self.set_layers(context, 'brow_2_bbone_out_R', [26])
        self.set_layers(context, 'brow_up_4_def_R', [27, 31])
        self.set_layers(context, 'brow_low_4_bezier_out_R', [26])
        self.set_layers(context, 'brow_low_4_def_R', [27, 31])
        self.set_layers(context, 'brow_low_4_bbone_out_R', [26])
        self.set_layers(context, 'brow_up_4_bezier_out_R', [26])
        self.set_layers(context, 'brow_up_4_bbone_out_R', [26])
        self.set_layers(context, 'brow_3_rot_R', [26, 29])
        self.set_layers(context, 'brow_3_bezier_in_mstr_R', [26, 31])
        self.set_layers(context, 'brow_3_bezier_in_R', [26])
        self.set_layers(context, 'brow_3_bbone_in_R', [26])
        self.set_layers(context, 'brow_3_def_R', [27, 31])
        self.set_layers(context, 'brow_3_bbone_out_R', [26])
        self.set_layers(context, 'cheek_ctrl_drv_R', [26])
        self.set_layers(context, 'cheek_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'cheek_ctrl_1_R', [2, 25])
        self.set_layers(context, 'nose_frown_3_def_R', [27, 31])
        self.set_layers(context, 'cheek_ctrl_2_mstr_R', [28])
        self.set_layers(context, 'cheek_ctrl_2_R', [2, 25])
        self.set_layers(context, 'lev1_3_def_R', [27, 31])
        self.set_layers(context, 'cheek_1_rot_R', [26, 29])
        self.set_layers(context, 'cheek_1_def_R', [27, 31])
        self.set_layers(context, 'cheek_ctrl_3_mstr_R', [28])
        self.set_layers(context, 'cheek_ctrl_3_R', [2, 25])
        self.set_layers(context, 'lev2_3_def_R', [27, 31])
        self.set_layers(context, 'lev2_4_def_R', [27, 31])
        self.set_layers(context, 'cheek_2_rot_R', [26, 29])
        self.set_layers(context, 'cheek_2_def_R', [27, 31])
        self.set_layers(context, 'mouth_corner_distance_drv_R', [26])
        self.set_layers(context, 'mouth_corner_drv_R', [26])
        self.set_layers(context, 'chin_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'chin_ctrl_3_R', [2, 25])
        self.set_layers(context, 'chin_3_rot_R', [26, 29])
        self.set_layers(context, 'chin_3_def_R', [27, 31])
        self.set_layers(context, 'cheek_ctrl_5_mstr_R', [28, 31])
        self.set_layers(context, 'cheek_ctrl_5_R', [2, 25])
        self.set_layers(context, 'jaw_1_rot_R', [26, 29])
        self.set_layers(context, 'jaw_1_def_R', [27, 31])
        self.set_layers(context, 'jaw_2_rot_R', [26, 29])
        self.set_layers(context, 'jaw_2_def_R', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_2_R', [1, 25])
        self.set_layers(context, 'lip_low_mstr_2_R', [26, 29])
        self.set_layers(context, 'lip_low2_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_2_R', [2, 25])
        self.set_layers(context, 'lip_low3_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_2_R', [2, 25])
        self.set_layers(context, 'oris_low2_3_def_R', [27, 31])
        self.set_layers(context, 'oris_low2_2_def_R', [27, 31])
        self.set_layers(context, 'oris_low2_1_def_R', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_1_R', [1, 25])
        self.set_layers(context, 'lip_low_1_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_low_2_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_low_2_bezier_in_R', [26])
        self.set_layers(context, 'lip_low_2_bbone_in_R', [26])
        self.set_layers(context, 'lip_low_1_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_low_1_bezier_out_R', [26])
        self.set_layers(context, 'lip_low_3_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_low_3_bezier_in_R', [26])
        self.set_layers(context, 'lip_low_3_bbone_in_R', [26])
        self.set_layers(context, 'lip_low_1_bezier_in_L', [26])
        self.set_layers(context, 'lip_low_1_bbone_in_L', [26])
        self.set_layers(context, 'lip_low_1_def_L', [27, 31])
        self.set_layers(context, 'lip_low_1_bbone_out_L', [26])
        self.set_layers(context, 'lip_low_mstr_1_R', [26, 29])
        self.set_layers(context, 'lip_low_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_low_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_low_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_low_2_def_R', [27, 31])
        self.set_layers(context, 'lip_low_1_def_R', [27, 31])
        self.set_layers(context, 'lip_low_1_bbone_out_R', [26])
        self.set_layers(context, 'lip_low2_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_1_R', [2, 25])
        self.set_layers(context, 'lip_low2_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_low2_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_low2_1_bbone_L', [26])
        self.set_layers(context, 'lip_low2_1_def_L', [27, 31])
        self.set_layers(context, 'lip_low2_2_def_L', [27, 31])
        self.set_layers(context, 'lip_low2_3_def_L', [27, 31])
        self.set_layers(context, 'lip_low2_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_low2_1_def_R', [27, 31])
        self.set_layers(context, 'lip_low2_2_def_R', [27, 31])
        self.set_layers(context, 'lip_low3_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_1_R', [2, 25])
        self.set_layers(context, 'lip_low3_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_low3_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_low3_1_bbone_L', [26])
        self.set_layers(context, 'lip_low3_1_def_L', [27, 31])
        self.set_layers(context, 'lip_low3_2_def_L', [27, 31])
        self.set_layers(context, 'lip_low3_3_def_L', [27, 31])
        self.set_layers(context, 'lip_low3_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_low3_1_def_R', [27, 31])
        self.set_layers(context, 'lip_low3_2_def_R', [27, 31])
        self.set_layers(context, 'oris_low1_3_def_R', [27, 31])
        self.set_layers(context, 'oris_low1_2_def_R', [27, 31])
        self.set_layers(context, 'oris_low1_1_def_R', [27, 31])
        self.set_layers(context, 'lip_low_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low_ctrl_3_R', [1, 25])
        self.set_layers(context, 'lip_low_2_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_low_2_bezier_out_R', [26])
        self.set_layers(context, 'lip_low_4_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_low_4_bezier_in_R', [26])
        self.set_layers(context, 'lip_low_4_bbone_in_R', [26])
        self.set_layers(context, 'lip_low_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_low_2_bbone_out_R', [26])
        self.set_layers(context, 'lip_low_3_def_R', [27, 31])
        self.set_layers(context, 'lip_low_mstr_3_R', [26, 29])
        self.set_layers(context, 'lip_low2_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low2_ctrl_3_R', [2, 25])
        self.set_layers(context, 'lip_low2_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_low2_3_def_R', [27, 31])
        self.set_layers(context, 'lip_low3_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_low3_ctrl_3_R', [2, 25])
        self.set_layers(context, 'lip_low3_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_low3_3_def_R', [27, 31])
        self.set_layers(context, 'oris_low3_3_def_R', [27, 31])
        self.set_layers(context, 'oris_low3_2_def_R', [27, 31])
        self.set_layers(context, 'oris_low3_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_2_R', [1, 25])
        self.set_layers(context, 'lip_up_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_1_R', [1, 25])
        self.set_layers(context, 'lip_up_1_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_up_1_bezier_out_R', [26])
        self.set_layers(context, 'lip_up_3_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_up_3_bezier_in_R', [26])
        self.set_layers(context, 'lip_up_3_bbone_in_R', [26])
        self.set_layers(context, 'lip_up_2_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_up_2_bezier_in_R', [26])
        self.set_layers(context, 'lip_up_2_bbone_in_R', [26])
        self.set_layers(context, 'lip_up_1_bezier_in_mstr_L', [26])
        self.set_layers(context, 'lip_up_1_bezier_in_L', [26])
        self.set_layers(context, 'lip_up_1_bbone_in_L', [26])
        self.set_layers(context, 'lip_up_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up_ctrl_3_R', [1, 25])
        self.set_layers(context, 'lip_up_2_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_up_2_bezier_out_R', [26])
        self.set_layers(context, 'lip_up_4_bezier_in_mstr_R', [26])
        self.set_layers(context, 'lip_up_4_bezier_in_R', [26])
        self.set_layers(context, 'lip_up_4_bbone_in_R', [26])
        self.set_layers(context, 'lip_up3_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_1_R', [2, 25])
        self.set_layers(context, 'cheek_mstr_1_R', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_1_R', [2, 25])
        self.set_layers(context, 'nose_frown_2_def_R', [27, 31])
        self.set_layers(context, 'nose_frown_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up_ctrl_4_mstr_R', [28])
        self.set_layers(context, 'lip_up_ctrl_4_R', [1, 25])
        self.set_layers(context, 'lip_low_3_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_low_3_bezier_out_R', [26])
        self.set_layers(context, 'lip_low_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_low_3_bbone_out_R', [26])
        self.set_layers(context, 'lip_low_4_def_R', [27, 31])
        self.set_layers(context, 'lip_low_4_bbone_out_R', [26])
        self.set_layers(context, 'lip_up_3_bezier_out_mstr_R', [26])
        self.set_layers(context, 'lip_up_3_bezier_out_R', [26])
        self.set_layers(context, 'lip_up_ctrl_4_loc_R', [26])
        self.set_layers(context, 'lip_up_mstr_4_str_R', [31])
        self.set_layers(context, 'lip_up_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_up_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_2_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_2_def_L', [27, 31])
        self.set_layers(context, 'lip_up_1_bbone_out_L', [26])
        self.set_layers(context, 'lip_up_3_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_2_bbone_out_L', [26])
        self.set_layers(context, 'lip_up_3_def_L', [27, 31])
        self.set_layers(context, 'lip_up3_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_up3_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_up3_1_bbone_R', [26])
        self.set_layers(context, 'lip_up_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_3_bbone_out_L', [26])
        self.set_layers(context, 'lip_up_4_def_L', [27, 31])
        self.set_layers(context, 'lip_up_4_bbone_out_L', [26])
        self.set_layers(context, 'lip_up_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_up_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_1_bbone_out_R', [26])
        self.set_layers(context, 'lip_up_2_def_R', [27, 31])
        self.set_layers(context, 'lip_up_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_2_bbone_out_R', [26])
        self.set_layers(context, 'lip_up_3_def_R', [27, 31])
        self.set_layers(context, 'lip_up3_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_up3_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_up3_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up3_1_bbone_L', [26])
        self.set_layers(context, 'lip_up3_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_3_bbone_out_R', [26])
        self.set_layers(context, 'lip_up_4_def_R', [27, 31])
        self.set_layers(context, 'lip_up_4_bbone_out_R', [26])
        self.set_layers(context, 'mouth_corners_mstr_R', [26])
        self.set_layers(context, 'lattice_mouth', [26])
        self.set_layers(context, 'teeth_up_str', [31])
        self.set_layers(context, 'teeth_up', [1, 25])
        self.set_layers(context, 'teeth_up_ctrl_str_R', [31])
        self.set_layers(context, 'teeth_up_ctrl_str_L', [31])
        self.set_layers(context, 'teeth_up_L', [26])
        self.set_layers(context, 'teeth_up_ctrl_L', [1, 25])
        self.set_layers(context, 'teeth_up_ctrl_mid_loc_L', [26])
        self.set_layers(context, 'teeth_up_R', [26])
        self.set_layers(context, 'teeth_up_ctrl_R', [1, 25])
        self.set_layers(context, 'teeth_up_ctrl_mid_loc_R', [26])
        self.set_layers(context, 'teeth_up_ctrl_mid_str', [25, 31])
        self.set_layers(context, 'teeth_up_ctrl_mid', [1, 25])
        self.set_layers(context, 'teeth_up_ctrl_mid_str_L', [25, 31])
        self.set_layers(context, 'teeth_up_ctrl_mid_L', [1, 25])
        self.set_layers(context, 'teeth_up_bbone_R', [26])
        self.set_layers(context, 'teeth_up_rot_1_L', [26])
        self.set_layers(context, 'teeth_up_rot_2_L', [26])
        self.set_layers(context, 'teeth_up_ctrl_mid_str_R', [25, 31])
        self.set_layers(context, 'teeth_up_ctrl_mid_R', [1, 25])
        self.set_layers(context, 'teeth_up_rot_2_R', [26])
        self.set_layers(context, 'teeth_up_bbone_L', [26])
        self.set_layers(context, 'teeth_up_1_def_L', [27, 31])
        self.set_layers(context, 'teeth_up_2_def_L', [27, 31])
        self.set_layers(context, 'teeth_up_rot_1_R', [26])
        self.set_layers(context, 'teeth_up_1_def_R', [27, 31])
        self.set_layers(context, 'teeth_up_2_def_R', [27, 31])
        self.set_layers(context, 'teeth_low_str', [31])
        self.set_layers(context, 'teeth_low', [1, 25])
        self.set_layers(context, 'teeth_low_ctrl_str_R', [31])
        self.set_layers(context, 'teeth_low_ctrl_str_L', [31])
        self.set_layers(context, 'teeth_low_L', [26])
        self.set_layers(context, 'teeth_low_ctrl_L', [1, 25])
        self.set_layers(context, 'teeth_low_ctrl_mid_loc_L', [26])
        self.set_layers(context, 'teeth_low_R', [26])
        self.set_layers(context, 'teeth_low_ctrl_R', [1, 25])
        self.set_layers(context, 'teeth_low_ctrl_mid_loc_R', [26])
        self.set_layers(context, 'teeth_low_ctrl_mid_str', [25, 31])
        self.set_layers(context, 'teeth_low_ctrl_mid', [1, 25])
        self.set_layers(context, 'teeth_low_ctrl_mid_str_L', [25, 31])
        self.set_layers(context, 'teeth_low_ctrl_mid_L', [1, 25])
        self.set_layers(context, 'teeth_low_bbone_R', [26])
        self.set_layers(context, 'teeth_low_rot_1_L', [26])
        self.set_layers(context, 'teeth_low_rot_2_L', [26])
        self.set_layers(context, 'teeth_low_ctrl_mid_str_R', [25, 31])
        self.set_layers(context, 'teeth_low_ctrl_mid_R', [1, 25])
        self.set_layers(context, 'teeth_low_rot_2_R', [26])
        self.set_layers(context, 'teeth_low_bbone_L', [26])
        self.set_layers(context, 'teeth_low_1_def_L', [27, 31])
        self.set_layers(context, 'teeth_low_2_def_L', [27, 31])
        self.set_layers(context, 'teeth_low_rot_1_R', [26])
        self.set_layers(context, 'teeth_low_1_def_R', [27, 31])
        self.set_layers(context, 'teeth_low_2_def_R', [27, 31])
        self.set_layers(context, 'lip_up_mstr_mid', [26, 29])
        self.set_layers(context, 'lip_up2_ctrl_mstr_mid', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_mid', [2, 25])
        self.set_layers(context, 'oris_up_2_def_mid', [27, 31])
        self.set_layers(context, 'oris_up_1_def_mid', [27, 31])
        self.set_layers(context, 'cheek_mstr_4_L', [26, 29])
        self.set_layers(context, 'cheek_ctrl_4_mstr_L', [28, 29, 31])
        self.set_layers(context, 'cheek_ctrl_4_L', [2, 25])
        self.set_layers(context, 'cheek_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_mstr_4_L', [26, 29])
        self.set_layers(context, 'lip_up2_ctrl_4_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_4_L', [2, 25])
        self.set_layers(context, 'lip_low2_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_low2_4_def_L', [27, 31])
        self.set_layers(context, 'lip_up3_ctrl_4_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_4_L', [2, 25])
        self.set_layers(context, 'lip_low3_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_low3_4_def_L', [27, 31])
        self.set_layers(context, 'cheek2_ctrl_4_mstr_L', [28, 29, 31])
        self.set_layers(context, 'cheek2_ctrl_4_L', [2, 25])
        self.set_layers(context, 'cheek2_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_up_mstr_2_L', [26, 29])
        self.set_layers(context, 'lip_up3_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_2_L', [2, 25])
        self.set_layers(context, 'lip_up3_2_rot_L', [26, 29])
        self.set_layers(context, 'cheek_mstr_2_L', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_2_L', [2, 25])
        self.set_layers(context, 'cheek2_1_rot_L', [26, 29])
        self.set_layers(context, 'cheek2_1_def_L', [27, 31])
        self.set_layers(context, 'lev1_2_def_L', [27, 31])
        self.set_layers(context, 'lev1_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up3_2_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_2_L', [2, 25])
        self.set_layers(context, 'oris_up2_2_def_L', [27, 31])
        self.set_layers(context, 'oris_up2_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_mstr_3_L', [26, 29])
        self.set_layers(context, 'lip_up3_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_3_L', [2, 25])
        self.set_layers(context, 'lip_up3_3_rot_L', [26, 29])
        self.set_layers(context, 'cheek_mstr_3_L', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_3_L', [2, 25])
        self.set_layers(context, 'cheek2_2_rot_L', [26, 29])
        self.set_layers(context, 'cheek2_2_def_L', [27, 31])
        self.set_layers(context, 'lip_up3_4_rot_L', [26, 29])
        self.set_layers(context, 'lip_up3_3_def_L', [27, 31])
        self.set_layers(context, 'lip_up3_4_def_L', [27, 31])
        self.set_layers(context, 'lev2_2_def_L', [27, 31])
        self.set_layers(context, 'lev2_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_3_L', [2, 25])
        self.set_layers(context, 'lip_up2_3_rot_L', [26, 29])
        self.set_layers(context, 'lip_up2_4_rot_L', [26, 29])
        self.set_layers(context, 'oris_up3_2_def_L', [27, 31])
        self.set_layers(context, 'oris_up3_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up_mstr_1_L', [26, 29])
        self.set_layers(context, 'lip_up2_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_1_L', [2, 25])
        self.set_layers(context, 'lip_up2_1_rot_L', [26, 29])
        self.set_layers(context, 'lip_up2_1_bbone_rot_R', [26])
        self.set_layers(context, 'lip_up2_1_bbone_R', [26])
        self.set_layers(context, 'lip_up2_2_rot_L', [26, 29])
        self.set_layers(context, 'oris_up1_2_def_L', [27, 31])
        self.set_layers(context, 'oris_up1_1_def_L', [27, 31])
        self.set_layers(context, 'lev3_3_def_L', [27, 31])
        self.set_layers(context, 'lev3_2_def_L', [27, 31])
        self.set_layers(context, 'lev3_1_def_L', [27, 31])
        self.set_layers(context, 'oris_up4_2_def_L', [27, 31])
        self.set_layers(context, 'oris_up4_1_def_L', [27, 31])
        self.set_layers(context, 'cheek_3_rot_L', [26, 29])
        self.set_layers(context, 'cheek_3_def_L', [27, 31])
        self.set_layers(context, 'cheek_4_def_L', [27, 31])
        self.set_layers(context, 'cheek2_3_rot_L', [26, 29])
        self.set_layers(context, 'cheek2_3_def_L', [27, 31])
        self.set_layers(context, 'cheek2_4_def_L', [27, 31])
        self.set_layers(context, 'cheek_mstr_4_R', [26, 29])
        self.set_layers(context, 'cheek_ctrl_4_mstr_R', [28, 29, 31])
        self.set_layers(context, 'cheek_ctrl_4_R', [2, 25])
        self.set_layers(context, 'cheek_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_mstr_4_R', [26, 29])
        self.set_layers(context, 'lip_up2_ctrl_4_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_4_R', [2, 25])
        self.set_layers(context, 'lip_low2_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_low2_4_def_R', [27, 31])
        self.set_layers(context, 'lip_up3_ctrl_4_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_4_R', [2, 25])
        self.set_layers(context, 'lip_low3_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_low3_4_def_R', [27, 31])
        self.set_layers(context, 'cheek2_ctrl_4_mstr_R', [28, 29, 31])
        self.set_layers(context, 'cheek2_ctrl_4_R', [2, 25])
        self.set_layers(context, 'cheek2_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_up_mstr_2_R', [26, 29])
        self.set_layers(context, 'lip_up3_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_2_R', [2, 25])
        self.set_layers(context, 'lip_up3_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_up3_2_def_R', [27, 31])
        self.set_layers(context, 'cheek_mstr_2_R', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_2_R', [2, 25])
        self.set_layers(context, 'cheek2_1_rot_R', [26, 29])
        self.set_layers(context, 'cheek2_1_def_R', [27, 31])
        self.set_layers(context, 'lev1_2_def_R', [27, 31])
        self.set_layers(context, 'lev1_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up2_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_2_R', [2, 25])
        self.set_layers(context, 'oris_up2_2_def_R', [27, 31])
        self.set_layers(context, 'oris_up2_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up_mstr_3_R', [26, 29])
        self.set_layers(context, 'lip_up3_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up3_ctrl_3_R', [2, 25])
        self.set_layers(context, 'lip_up3_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_up3_3_def_R', [27, 31])
        self.set_layers(context, 'cheek_mstr_3_R', [26, 29])
        self.set_layers(context, 'cheek2_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'cheek2_ctrl_3_R', [2, 25])
        self.set_layers(context, 'cheek2_2_rot_R', [26, 29])
        self.set_layers(context, 'cheek2_2_def_R', [27, 31])
        self.set_layers(context, 'lev2_2_def_R', [27, 31])
        self.set_layers(context, 'lev2_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up3_4_rot_R', [26, 29])
        self.set_layers(context, 'lip_up3_4_def_R', [27, 31])
        self.set_layers(context, 'lip_up2_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_3_R', [2, 25])
        self.set_layers(context, 'lip_up2_3_rot_R', [26, 29])
        self.set_layers(context, 'lip_up2_4_rot_R', [26, 29])
        self.set_layers(context, 'oris_up3_2_def_R', [27, 31])
        self.set_layers(context, 'oris_up3_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up_mstr_1_R', [26, 29])
        self.set_layers(context, 'lip_up2_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'lip_up2_ctrl_1_R', [2, 25])
        self.set_layers(context, 'lip_up2_2_rot_R', [26, 29])
        self.set_layers(context, 'lip_up2_1_rot_R', [26, 29])
        self.set_layers(context, 'lip_up2_1_bbone_rot_L', [26])
        self.set_layers(context, 'lip_up2_1_bbone_L', [26])
        self.set_layers(context, 'lip_up2_1_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_2_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_3_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_4_def_L', [27, 31])
        self.set_layers(context, 'lip_up2_1_def_R', [27, 31])
        self.set_layers(context, 'lip_up2_2_def_R', [27, 31])
        self.set_layers(context, 'lip_up2_3_def_R', [27, 31])
        self.set_layers(context, 'lip_up2_4_def_R', [27, 31])
        self.set_layers(context, 'oris_up1_2_def_R', [27, 31])
        self.set_layers(context, 'oris_up1_1_def_R', [27, 31])
        self.set_layers(context, 'lev3_3_def_R', [27, 31])
        self.set_layers(context, 'lev3_2_def_R', [27, 31])
        self.set_layers(context, 'lev3_1_def_R', [27, 31])
        self.set_layers(context, 'oris_up4_2_def_R', [27, 31])
        self.set_layers(context, 'oris_up4_1_def_R', [27, 31])
        self.set_layers(context, 'cheek_3_rot_R', [26, 29])
        self.set_layers(context, 'cheek_3_def_R', [27, 31])
        self.set_layers(context, 'cheek_4_def_R', [27, 31])
        self.set_layers(context, 'cheek2_3_rot_R', [26, 29])
        self.set_layers(context, 'cheek2_3_def_R', [27, 31])
        self.set_layers(context, 'cheek2_4_def_R', [27, 31])
        self.set_layers(context, 'nose_bridge_1_def', [27, 31])
        self.set_layers(context, 'nose_bridge_2_def', [27, 31])
        self.set_layers(context, 'snap_neck_fk_pivot', [30])
        self.set_layers(context, 'arm_scale_R', [15, 25])
        self.set_layers(context, 'elbow_pole_torso_R', [26])
        self.set_layers(context, 'elbow_pole_hinge_R', [26])
        self.set_layers(context, 'hand_ik_mstr_R', [26])
        self.set_layers(context, 'hand_ik_hinge_R', [26])
        self.set_layers(context, 'hand_ik_ctrl_R', [6, 25, 30])
        self.set_layers(context, 'hand_ik_pivot_point_R', [6, 25, 30])
        self.set_layers(context, 'hand_ik_pivot_R', [26])
        self.set_layers(context, 'palm_bend_ik_ctrl_R', [6, 25, 30])
        self.set_layers(context, 'palm_bend_mstr_R', [26])
        self.set_layers(context, 'palm_bend_ik_R', [26])
        self.set_layers(context, 'fing_thumb_ik_mstr_2_R', [26])
        self.set_layers(context, 'fing_mid_ik_mstr_2_R', [26])
        self.set_layers(context, 'fing_ring_ik_mstr_2_R', [26])
        self.set_layers(context, 'fing_lit_ik_mstr_2_R', [26])
        self.set_layers(context, 'fing_ind_ik_mstr_2_R', [26])
        self.set_layers(context, 'hand_ik_shoulder_R', [26])
        self.set_layers(context, 'elbow_pole_R', [6, 25, 30])
        self.set_layers(context, 'shoulder_R', [3, 6, 25, 30])
        self.set_layers(context, 'shoulder_rot_R', [11, 25, 30])
        self.set_layers(context, 'clavi_R', [26])
        self.set_layers(context, 'clavi_toon_R', [13, 25])
        self.set_layers(context, 'shoulder_shp_at_R', [26])
        self.set_layers(context, 'clavi_def_R', [27, 29, 31])
        self.set_layers(context, 'arm_mstr_R', [26])
        self.set_layers(context, 'arm_hinge_R', [26])
        self.set_layers(context, 'arm_ik_R', [6, 25, 30])
        self.set_layers(context, 'arm_fk_R', [3, 25, 29, 30])
        self.set_layers(context, 'snap_elbow_pole_fk_R', [30])
        self.set_layers(context, 'arm_scale_L', [15, 25])
        self.set_layers(context, 'elbow_pole_torso_L', [26])
        self.set_layers(context, 'elbow_pole_hinge_L', [26])
        self.set_layers(context, 'hand_ik_mstr_L', [26])
        self.set_layers(context, 'hand_ik_hinge_L', [26])
        self.set_layers(context, 'hand_ik_ctrl_L', [16, 25, 30])
        self.set_layers(context, 'hand_ik_pivot_point_L', [16, 25, 30])
        self.set_layers(context, 'hand_ik_pivot_L', [26])
        self.set_layers(context, 'palm_bend_ik_ctrl_L', [16, 25, 30])
        self.set_layers(context, 'palm_bend_mstr_L', [26])
        self.set_layers(context, 'palm_bend_ik_L', [26])
        self.set_layers(context, 'fing_thumb_ik_mstr_2_L', [26])
        self.set_layers(context, 'fing_mid_ik_mstr_2_L', [26])
        self.set_layers(context, 'fing_ring_ik_mstr_2_L', [26])
        self.set_layers(context, 'fing_lit_ik_mstr_2_L', [26])
        self.set_layers(context, 'fing_ind_ik_mstr_2_L', [26])
        self.set_layers(context, 'hand_ik_shoulder_L', [26])
        self.set_layers(context, 'elbow_pole_L', [16, 25, 30])
        self.set_layers(context, 'shoulder_L', [5, 16, 25, 30])
        self.set_layers(context, 'shoulder_rot_L', [11, 25, 30])
        self.set_layers(context, 'clavi_L', [26])
        self.set_layers(context, 'clavi_toon_L', [13, 25])
        self.set_layers(context, 'shoulder_shp_at_L', [26])
        self.set_layers(context, 'clavi_def_L', [27, 29, 31])
        self.set_layers(context, 'arm_mstr_L', [26])
        self.set_layers(context, 'arm_hinge_L', [26])
        self.set_layers(context, 'arm_ik_L', [16, 25, 30])
        self.set_layers(context, 'arm_fk_L', [5, 25, 29, 30])
        self.set_layers(context, 'snap_elbow_pole_fk_L', [30])
        self.set_layers(context, 'spine_3_def', [27, 29, 31])
        self.set_layers(context, 'neck_1_def', [27, 29, 31])
        self.set_layers(context, 'neck_2_def', [27, 29, 31])
        self.set_layers(context, 'neck_3_def', [27, 29, 31])
        self.set_layers(context, 'pelvis_def_bbone', [26])
        self.set_layers(context, 'pelvis_def', [27, 29, 31])
        self.set_layers(context, 'pelvis_str_L', [31])
        self.set_layers(context, 'pelvis_str_R', [31])
        self.set_layers(context, 'leg_scale_R', [15, 25])
        self.set_layers(context, 'pelvis_toon_R', [14, 25])
        self.set_layers(context, 'knee_pole_limit_R', [26])
        self.set_layers(context, 'pelvis_def_R', [27, 29, 31])
        self.set_layers(context, 'thigh_mstr_R', [26])
        self.set_layers(context, 'thigh_hinge_R', [26])
        self.set_layers(context, 'leg_scale_L', [15, 25])
        self.set_layers(context, 'pelvis_toon_L', [14, 25])
        self.set_layers(context, 'knee_pole_limit_L', [26])
        self.set_layers(context, 'pelvis_def_L', [27, 29, 31])
        self.set_layers(context, 'thigh_mstr_L', [26])
        self.set_layers(context, 'thigh_hinge_L', [26])
        self.set_layers(context, 'lattice_body', [26])
        self.set_layers(context, 'belly_ctrl', [8, 18, 21])
        self.set_layers(context, 'butt_ctrl', [8, 18, 21])
        self.set_layers(context, 'master_body_str', [31])
        self.set_layers(context, 'sole_ctrl_R', [23, 25, 30, 31])
        self.set_layers(context, 'sole_pivot_point_R', [23, 25, 30])
        self.set_layers(context, 'sole_pivot_R', [26])
        self.set_layers(context, 'foot_roll_ctrl_R', [23, 25, 30, 31])
        self.set_layers(context, 'sole_R', [26])
        self.set_layers(context, 'foot_str_R', [31])
        self.set_layers(context, 'toe_big_str_2_R', [24])
        self.set_layers(context, 'toe_mid_str_2_R', [24])
        self.set_layers(context, 'toe_fourth_str_2_R', [24])
        self.set_layers(context, 'toe_lit_str_2_R', [24])
        self.set_layers(context, 'toe_ind_str_2_R', [24])
        self.set_layers(context, 'ankle_str_R', [31])
        self.set_layers(context, 'toe_str_1_R', [31])
        self.set_layers(context, 'toe_big_str_3_R', [24])
        self.set_layers(context, 'toe_mid_str_3_R', [24])
        self.set_layers(context, 'toe_mid_str_4_R', [24])
        self.set_layers(context, 'toe_fourth_str_3_R', [24])
        self.set_layers(context, 'toe_fourth_str_4_R', [24])
        self.set_layers(context, 'toe_lit_str_3_R', [24])
        self.set_layers(context, 'toe_ind_str_3_R', [24])
        self.set_layers(context, 'toe_ind_str_4_R', [24])
        self.set_layers(context, 'toe_str_2_R', [31])
        self.set_layers(context, 'toe_lit_str_4_R', [24])
        self.set_layers(context, 'toe_big_str_4_R', [24])
        self.set_layers(context, 'toe_fourth_str_5_R', [24])
        self.set_layers(context, 'toe_mid_str_5_R', [24])
        self.set_layers(context, 'toe_ind_str_5_R', [24])
        self.set_layers(context, 'toe_roll_2_R', [23, 25, 30])
        self.set_layers(context, 'toes_ik_ctrl_R', [23, 25, 29, 30])
        self.set_layers(context, 'toe_lit_ik_R', [10, 24, 25])
        self.set_layers(context, 'toe_big_ik_R', [10, 24, 25])
        self.set_layers(context, 'toe_fourth_ik_R', [10, 24, 25])
        self.set_layers(context, 'toe_mid_ik_R', [10, 24, 25])
        self.set_layers(context, 'toe_ind_ik_R', [10, 24, 25])
        self.set_layers(context, 'toes_ik_ctrl_mid_R', [23, 24])
        self.set_layers(context, 'toe_roll_1_R', [23, 25, 30])
        self.set_layers(context, 'foot_roll_mstr_R', [26])
        self.set_layers(context, 'foot_ik_ctrl_R', [23, 25, 30])
        self.set_layers(context, 'foot_roll_R', [26])
        self.set_layers(context, 'foot_ik_R', [26, 30])
        self.set_layers(context, 'toe_mstr_ik_R', [30])
        self.set_layers(context, 'toe_1_ik_R', [25, 30])
        self.set_layers(context, 'toe_2_ik_R', [25, 30])
        self.set_layers(context, 'knee_pole_mstr_R', [26])
        self.set_layers(context, 'knee_pole_R', [23, 25, 30])
        self.set_layers(context, 'thigh_ik_R', [23, 25, 30])
        self.set_layers(context, 'thigh_fk_R', [20, 25, 29, 30])
        self.set_layers(context, 'snap_knee_fk_R', [30])
        self.set_layers(context, 'hip_rot_R', [24])
        self.set_layers(context, 'sole_ctrl_L', [9, 25, 30, 31])
        self.set_layers(context, 'sole_pivot_point_L', [9, 25, 30])
        self.set_layers(context, 'sole_pivot_L', [26])
        self.set_layers(context, 'foot_roll_ctrl_L', [9, 25, 30, 31])
        self.set_layers(context, 'sole_L', [26])
        self.set_layers(context, 'foot_str_L', [31])
        self.set_layers(context, 'toe_big_str_2_L', [24])
        self.set_layers(context, 'toe_mid_str_2_L', [24])
        self.set_layers(context, 'toe_fourth_str_2_L', [24])
        self.set_layers(context, 'toe_lit_str_2_L', [24])
        self.set_layers(context, 'toe_ind_str_2_L', [24])
        self.set_layers(context, 'ankle_str_L', [31])
        self.set_layers(context, 'toe_str_1_L', [31])
        self.set_layers(context, 'toe_big_str_3_L', [24])
        self.set_layers(context, 'toe_mid_str_3_L', [24])
        self.set_layers(context, 'toe_mid_str_4_L', [24])
        self.set_layers(context, 'toe_fourth_str_3_L', [24])
        self.set_layers(context, 'toe_fourth_str_4_L', [24])
        self.set_layers(context, 'toe_lit_str_3_L', [24])
        self.set_layers(context, 'toe_ind_str_3_L', [24])
        self.set_layers(context, 'toe_ind_str_4_L', [24])
        self.set_layers(context, 'toe_str_2_L', [31])
        self.set_layers(context, 'toe_lit_str_4_L', [24])
        self.set_layers(context, 'toe_fourth_str_5_L', [24])
        self.set_layers(context, 'toe_mid_str_5_L', [24])
        self.set_layers(context, 'toe_big_str_4_L', [24])
        self.set_layers(context, 'toe_ind_str_5_L', [24])
        self.set_layers(context, 'toe_roll_2_L', [9, 25, 30])
        self.set_layers(context, 'toes_ik_ctrl_L', [9, 25, 29, 30])
        self.set_layers(context, 'toe_lit_ik_L', [10, 24, 25])
        self.set_layers(context, 'toe_fourth_ik_L', [10, 24, 25])
        self.set_layers(context, 'toe_mid_ik_L', [10, 24, 25])
        self.set_layers(context, 'toe_big_ik_L', [10, 24, 25])
        self.set_layers(context, 'toe_ind_ik_L', [10, 24, 25])
        self.set_layers(context, 'toes_ik_ctrl_mid_L', [9, 24])
        self.set_layers(context, 'toe_roll_1_L', [9, 25, 30])
        self.set_layers(context, 'foot_roll_mstr_L', [26])
        self.set_layers(context, 'foot_ik_ctrl_L', [9, 25, 30])
        self.set_layers(context, 'foot_roll_L', [26])
        self.set_layers(context, 'foot_ik_L', [26, 30])
        self.set_layers(context, 'toe_mstr_ik_L', [30])
        self.set_layers(context, 'toe_1_ik_L', [25, 30])
        self.set_layers(context, 'toe_2_ik_L', [25, 30])
        self.set_layers(context, 'knee_pole_mstr_L', [26])
        self.set_layers(context, 'knee_pole_L', [9, 25, 30])
        self.set_layers(context, 'thigh_ik_L', [9, 25, 30])
        self.set_layers(context, 'thigh_fk_L', [22, 25, 29, 30])
        self.set_layers(context, 'snap_knee_fk_L', [30])
        self.set_layers(context, 'hip_rot_L', [24])
        self.set_layers(context, 'forearm_ik_R', [6, 25, 30])
        self.set_layers(context, 'forearm_fk_R', [3, 25, 29, 30])
        self.set_layers(context, 'hand_toon_R', [14, 25])
        self.set_layers(context, 'forearm_twist_bezier_out_mstr_R', [26])
        self.set_layers(context, 'hand_fk_R', [3, 25, 30])
        self.set_layers(context, 'fing_ind_ik_mstr_R', [26])
        self.set_layers(context, 'fing_ind_hinge_R', [26])
        self.set_layers(context, 'fing_ind_ik_R', [19, 24, 25])
        self.set_layers(context, 'fing_thumb_ik_mstr_R', [26])
        self.set_layers(context, 'fing_thumb_hinge_R', [26])
        self.set_layers(context, 'fing_thumb_ik_R', [19, 24, 25])
        self.set_layers(context, 'fing_mid_ik_mstr_R', [26])
        self.set_layers(context, 'fing_mid_hinge_R', [26])
        self.set_layers(context, 'fing_mid_ik_R', [19, 24, 25])
        self.set_layers(context, 'fing_ring_ik_mstr_R', [26])
        self.set_layers(context, 'fing_ring_hinge_R', [26])
        self.set_layers(context, 'fing_ring_ik_R', [19, 24, 25])
        self.set_layers(context, 'fing_lit_ik_mstr_R', [26])
        self.set_layers(context, 'fing_lit_hinge_R', [26])
        self.set_layers(context, 'fing_lit_ik_R', [19, 24, 25])
        self.set_layers(context, 'hand_close_R', [3, 6, 24, 25])
        self.set_layers(context, 'palm_bend_fk_ctrl_R', [3, 25, 30])
        self.set_layers(context, 'palm_bend_fk_R', [30])
        self.set_layers(context, 'hand_mstr_R', [26])
        self.set_layers(context, 'arm_def_mstr_R', [26])
        self.set_layers(context, 'elbow_toon_R', [13, 25])
        self.set_layers(context, 'elbow_toon_loc_R', [26])
        self.set_layers(context, 'arm_toon_mstr_R', [26])
        self.set_layers(context, 'forearm_mstr_R', [26])
        self.set_layers(context, 'forearm_def_mstr_R', [26])
        self.set_layers(context, 'forearm_toon_2_mstr_R', [26])
        self.set_layers(context, 'forearm_toon_2_R', [26])
        self.set_layers(context, 'forearm_toon_mstr_R', [26])
        self.set_layers(context, 'hand_drv_R', [26])
        self.set_layers(context, 'forearm_toon_def_R', [26])
        self.set_layers(context, 'hand_def_mstr_R', [26])
        self.set_layers(context, 'hand_def_R', [27, 29, 31])
        self.set_layers(context, 'fing_lit_ctrl_bend_loc_R', [26])
        self.set_layers(context, 'fing_lit_ctrl_loc_R', [26])
        self.set_layers(context, 'fing_ring_ctrl_bend_loc_R', [26])
        self.set_layers(context, 'fing_ring_ctrl_loc_R', [26])
        self.set_layers(context, 'fing_ind_ctrl_bend_loc_R', [26])
        self.set_layers(context, 'fing_ind_ctrl_loc_R', [26])
        self.set_layers(context, 'fing_mid_ctrl_bend_loc_R', [26])
        self.set_layers(context, 'fing_mid_ctrl_loc_R', [26])
        self.set_layers(context, 'fing_thumb_1_R', [17, 24, 25])
        self.set_layers(context, 'fing_thumb_ctrl_mstr_R', [28])
        self.set_layers(context, 'fing_thumb_ctrl_R', [3, 6, 24, 25, 29])
        self.set_layers(context, 'fing_thumb_1_ik_R', [26])
        self.set_layers(context, 'fing_thumb_ctrl_shp_at_R', [26])
        self.set_layers(context, 'fing_thumb_2_R', [17, 24, 25])
        self.set_layers(context, 'fing_thumb_3_R', [17, 24, 25])
        self.set_layers(context, 'fing_thumb_shp_at_R', [26])
        self.set_layers(context, 'fing_thumb_2_rot_R', [26])
        self.set_layers(context, 'fing_thumb_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_thumb_fix_up_1_R', [24, 27])
        self.set_layers(context, 'fing_thumb_fix_low_1_R', [24, 27])
        self.set_layers(context, 'fing_thumb_2_def_mstr_R', [26])
        self.set_layers(context, 'fing_thumb_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_thumb_fix_low_2_R', [24, 27])
        self.set_layers(context, 'fing_thumb_fix_up_2_R', [24, 27])
        self.set_layers(context, 'fing_thumb_3_def_mstr_R', [26])
        self.set_layers(context, 'fing_thumb_3_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_spread_R', [3, 6, 24, 25])
        self.set_layers(context, 'fing_lit_ctrl_mstr_R', [28])
        self.set_layers(context, 'fing_lit_ctrl_R', [3, 6, 24, 25, 29])
        self.set_layers(context, 'fing_ring_ctrl_mstr_R', [28])
        self.set_layers(context, 'fing_ring_ctrl_R', [3, 6, 24, 25, 29])
        self.set_layers(context, 'fing_ind_ctrl_mstr_R', [28])
        self.set_layers(context, 'fing_ind_ctrl_R', [3, 6, 24, 25, 29])
        self.set_layers(context, 'fing_mid_ctrl_mstr_R', [28])
        self.set_layers(context, 'fing_mid_ctrl_R', [3, 6, 24, 25, 29])
        self.set_layers(context, 'fing_lit_1_R', [26])
        self.set_layers(context, 'fing_lit_ctrl_shp_at_R', [26])
        self.set_layers(context, 'fing_lit_2_R', [17, 24, 25])
        self.set_layers(context, 'fing_lit_3_R', [17, 24, 25])
        self.set_layers(context, 'fing_lit_4_R', [17, 24, 25])
        self.set_layers(context, 'fing_lit_shp_at_R', [26])
        self.set_layers(context, 'fing_lit_2_rot_R', [26])
        self.set_layers(context, 'fing_lit_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_up_1_R', [24, 27])
        self.set_layers(context, 'fing_lit_fix_low_1_R', [24, 27])
        self.set_layers(context, 'fing_lit_2_def_mstr_R', [26])
        self.set_layers(context, 'fing_lit_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_low_2_R', [24, 27])
        self.set_layers(context, 'fing_lit_fix_up_2_R', [24, 27])
        self.set_layers(context, 'fing_lit_3_def_mstr_R', [26])
        self.set_layers(context, 'fing_lit_3_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_low_3_R', [24, 27])
        self.set_layers(context, 'fing_lit_fix_up_3_R', [24, 27])
        self.set_layers(context, 'fing_lit_4_def_mstr_R', [26])
        self.set_layers(context, 'fing_lit_4_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_1_R', [26])
        self.set_layers(context, 'fing_ring_ctrl_shp_at_R', [26])
        self.set_layers(context, 'fing_ring_2_R', [17, 24, 25])
        self.set_layers(context, 'fing_ring_3_rot_R', [26])
        self.set_layers(context, 'fing_ring_4_rot_R', [26])
        self.set_layers(context, 'fing_ring_3_R', [17, 24, 25])
        self.set_layers(context, 'fing_ring_4_R', [17, 24, 25])
        self.set_layers(context, 'fing_ring_shp_at_R', [26])
        self.set_layers(context, 'fing_ring_2_rot_R', [26])
        self.set_layers(context, 'fing_ring_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_up_1_R', [24, 27])
        self.set_layers(context, 'fing_ring_fix_low_1_R', [24, 27])
        self.set_layers(context, 'fing_ring_2_def_mstr_R', [26])
        self.set_layers(context, 'fing_ring_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_low_2_R', [24, 27])
        self.set_layers(context, 'fing_ring_fix_up_2_R', [24, 27])
        self.set_layers(context, 'fing_ring_3_def_mstr_R', [26])
        self.set_layers(context, 'fing_ring_3_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_low_3_R', [24, 27])
        self.set_layers(context, 'fing_ring_fix_up_3_R', [24, 27])
        self.set_layers(context, 'fing_ring_4_def_mstr_R', [26])
        self.set_layers(context, 'fing_ring_4_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_1_R', [26])
        self.set_layers(context, 'fing_ind_ctrl_shp_at_R', [26])
        self.set_layers(context, 'fing_ind_2_R', [17, 24, 25])
        self.set_layers(context, 'fing_ind_3_R', [17, 24, 25])
        self.set_layers(context, 'fing_ind_4_R', [17, 24, 25])
        self.set_layers(context, 'fing_ind_shp_at_R', [26])
        self.set_layers(context, 'fing_ind_2_rot_R', [26])
        self.set_layers(context, 'fing_ind_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_1_R', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_1_R', [24, 27])
        self.set_layers(context, 'fing_ind_2_def_mstr_R', [26])
        self.set_layers(context, 'fing_ind_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_2_R', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_2_R', [24, 27])
        self.set_layers(context, 'fing_ind_3_def_mstr_R', [26])
        self.set_layers(context, 'fing_ind_3_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_3_R', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_3_R', [24, 27])
        self.set_layers(context, 'fing_ind_4_def_mstr_R', [26])
        self.set_layers(context, 'fing_ind_4_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_1_R', [26])
        self.set_layers(context, 'fing_mid_ctrl_shp_at_R', [26])
        self.set_layers(context, 'fing_mid_2_R', [17, 24, 25])
        self.set_layers(context, 'fing_mid_3_R', [17, 24, 25])
        self.set_layers(context, 'fing_mid_4_R', [17, 24, 25])
        self.set_layers(context, 'fing_mid_shp_at_R', [26])
        self.set_layers(context, 'fing_mid_2_rot_R', [26])
        self.set_layers(context, 'fing_mid_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_up_1_R', [24, 27])
        self.set_layers(context, 'fing_mid_fix_low_1_R', [24, 27])
        self.set_layers(context, 'fing_mid_2_def_mstr_R', [26])
        self.set_layers(context, 'fing_mid_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_low_2_R', [24, 27])
        self.set_layers(context, 'fing_mid_fix_up_2_R', [24, 27])
        self.set_layers(context, 'fing_mid_3_def_mstr_R', [26])
        self.set_layers(context, 'fing_mid_3_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_low_3_R', [24, 27])
        self.set_layers(context, 'fing_mid_fix_up_3_R', [24, 27])
        self.set_layers(context, 'fing_mid_4_def_mstr_R', [26])
        self.set_layers(context, 'fing_mid_4_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'hand_accessory_R', [11, 25])
        self.set_layers(context, 'forearm_rot_R', [26])
        self.set_layers(context, 'forearm_twist_R', [26])
        self.set_layers(context, 'forearm_twist_bezier_out_R', [26])
        self.set_layers(context, 'elbow_fix_R', [27])
        self.set_layers(context, 'arm_fix_R', [27])
        self.set_layers(context, 'elbow_line_R', [6, 25, 31])
        self.set_layers(context, 'arm_drv_R', [26])
        self.set_layers(context, 'back_fix_R', [27])
        self.set_layers(context, 'chest_fix_R', [27])
        self.set_layers(context, 'arm_twist_R', [26])
        self.set_layers(context, 'arm_def_bezier_in_R', [26])
        self.set_layers(context, 'arm_def_bbone_in_R', [26])
        self.set_layers(context, 'forearm_toon_R', [14, 25])
        self.set_layers(context, 'forearm_def_bezier_out_mstr_R', [26])
        self.set_layers(context, 'forearm_def_bezier_out_R', [26])
        self.set_layers(context, 'forearm_def_bezier_out_rot_R', [26])
        self.set_layers(context, 'forearm_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'forearm_twist_bezier_in_R', [26])
        self.set_layers(context, 'forearm_twist_bbone_in_R', [26])
        self.set_layers(context, 'forearm_twist_def_R', [27, 31])
        self.set_layers(context, 'hand_twist_drv_R', [26])
        self.set_layers(context, 'wrist_fix_low_R', [27])
        self.set_layers(context, 'wrist_fix_up_R', [27])
        self.set_layers(context, 'forearm_twist_bbone_out_R', [26])
        self.set_layers(context, 'forearm_no_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'forearm_no_twist_bezier_in_R', [26])
        self.set_layers(context, 'forearm_no_twist_bbone_in_R', [26])
        self.set_layers(context, 'forearm_no_twist_def_R', [27])
        self.set_layers(context, 'arm_toon_R', [14, 25])
        self.set_layers(context, 'arm_def_bezier_out_mstr_R', [26])
        self.set_layers(context, 'arm_def_bezier_out_R', [26])
        self.set_layers(context, 'arm_def_R', [27, 31])
        self.set_layers(context, 'arm_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'arm_twist_bezier_in_R', [26])
        self.set_layers(context, 'arm_def_bbone_out_R', [26])
        self.set_layers(context, 'arm_twist_bbone_in_R', [26])
        self.set_layers(context, 'arm_twist_def_R', [27, 31])
        self.set_layers(context, 'arm_twist_curved_R', [26])
        self.set_layers(context, 'arm_twist_bezier_out_mstr_R', [26])
        self.set_layers(context, 'arm_twist_bezier_out_R', [26])
        self.set_layers(context, 'arm_twist_bbone_out_rot_R', [26])
        self.set_layers(context, 'arm_twist_bbone_out_R', [26])
        self.set_layers(context, 'forearm_def_curved_R', [26])
        self.set_layers(context, 'forearm_def_bezier_in_mstr_R', [26])
        self.set_layers(context, 'forearm_def_bezier_in_R', [26])
        self.set_layers(context, 'forearm_def_bbone_in_R', [26])
        self.set_layers(context, 'forearm_def_R', [27, 31])
        self.set_layers(context, 'forearm_def_bbone_out_R', [26])
        self.set_layers(context, 'forearm_def_no_twist_bbone_in_R', [26])
        self.set_layers(context, 'forearm_def_no_twist_R', [27])
        self.set_layers(context, 'forearm_def_bbone_out_no_twist_R', [26])
        self.set_layers(context, 'forearm_ik_L', [16, 25, 30])
        self.set_layers(context, 'forearm_fk_L', [5, 25, 29, 30])
        self.set_layers(context, 'hand_toon_L', [14, 25])
        self.set_layers(context, 'forearm_twist_bezier_out_mstr_L', [26])
        self.set_layers(context, 'hand_fk_L', [5, 25, 30])
        self.set_layers(context, 'fing_ind_ik_mstr_L', [26])
        self.set_layers(context, 'fing_ind_hinge_L', [26])
        self.set_layers(context, 'fing_ind_ik_L', [19, 24, 25])
        self.set_layers(context, 'fing_thumb_ik_mstr_L', [26])
        self.set_layers(context, 'fing_thumb_hinge_L', [26])
        self.set_layers(context, 'fing_thumb_ik_L', [19, 24, 25])
        self.set_layers(context, 'fing_mid_ik_mstr_L', [26])
        self.set_layers(context, 'fing_mid_hinge_L', [26])
        self.set_layers(context, 'fing_mid_ik_L', [19, 24, 25])
        self.set_layers(context, 'fing_ring_ik_mstr_L', [26])
        self.set_layers(context, 'fing_ring_hinge_L', [26])
        self.set_layers(context, 'fing_ring_ik_L', [19, 24, 25])
        self.set_layers(context, 'fing_lit_ik_mstr_L', [26])
        self.set_layers(context, 'fing_lit_hinge_L', [26])
        self.set_layers(context, 'fing_lit_ik_L', [19, 24, 25])
        self.set_layers(context, 'hand_close_L', [5, 16, 24, 25])
        self.set_layers(context, 'palm_bend_fk_ctrl_L', [5, 25, 30])
        self.set_layers(context, 'palm_bend_fk_L', [30])
        self.set_layers(context, 'hand_mstr_L', [26])
        self.set_layers(context, 'arm_def_mstr_L', [26])
        self.set_layers(context, 'elbow_toon_L', [13, 25])
        self.set_layers(context, 'elbow_toon_loc_L', [26])
        self.set_layers(context, 'arm_toon_mstr_L', [26])
        self.set_layers(context, 'forearm_mstr_L', [26])
        self.set_layers(context, 'forearm_def_mstr_L', [26])
        self.set_layers(context, 'forearm_toon_2_mstr_L', [26])
        self.set_layers(context, 'forearm_toon_2_L', [26])
        self.set_layers(context, 'forearm_toon_mstr_L', [26])
        self.set_layers(context, 'hand_drv_L', [26])
        self.set_layers(context, 'forearm_toon_def_L', [26])
        self.set_layers(context, 'hand_def_mstr_L', [26])
        self.set_layers(context, 'hand_def_L', [27, 29, 31])
        self.set_layers(context, 'fing_lit_ctrl_bend_loc_L', [26])
        self.set_layers(context, 'fing_lit_ctrl_loc_L', [26])
        self.set_layers(context, 'fing_ring_ctrl_bend_loc_L', [26])
        self.set_layers(context, 'fing_ring_ctrl_loc_L', [26])
        self.set_layers(context, 'fing_ind_ctrl_bend_loc_L', [26])
        self.set_layers(context, 'fing_ind_ctrl_loc_L', [26])
        self.set_layers(context, 'fing_mid_ctrl_bend_loc_L', [26])
        self.set_layers(context, 'fing_mid_ctrl_loc_L', [26])
        self.set_layers(context, 'fing_thumb_1_L', [17, 24, 25])
        self.set_layers(context, 'fing_thumb_ctrl_mstr_L', [28])
        self.set_layers(context, 'fing_thumb_ctrl_L', [5, 16, 24, 25, 29])
        self.set_layers(context, 'fing_thumb_1_ik_L', [26])
        self.set_layers(context, 'fing_thumb_ctrl_shp_at_L', [26])
        self.set_layers(context, 'fing_thumb_2_L', [17, 24, 25])
        self.set_layers(context, 'fing_thumb_3_L', [17, 24, 25])
        self.set_layers(context, 'fing_thumb_shp_at_L', [26])
        self.set_layers(context, 'fing_thumb_2_rot_L', [26])
        self.set_layers(context, 'fing_thumb_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_thumb_fix_low_1_L', [24, 27])
        self.set_layers(context, 'fing_thumb_fix_up_1_L', [24, 27])
        self.set_layers(context, 'fing_thumb_2_def_mstr_L', [26])
        self.set_layers(context, 'fing_thumb_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_thumb_fix_low_2_L', [24, 27])
        self.set_layers(context, 'fing_thumb_fix_up_2_L', [24, 27])
        self.set_layers(context, 'fing_thumb_3_def_mstr_L', [26])
        self.set_layers(context, 'fing_thumb_3_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_spread_L', [5, 16, 24, 25])
        self.set_layers(context, 'fing_lit_ctrl_mstr_L', [28])
        self.set_layers(context, 'fing_lit_ctrl_L', [5, 16, 24, 25, 29])
        self.set_layers(context, 'fing_ring_ctrl_mstr_L', [28])
        self.set_layers(context, 'fing_ring_ctrl_L', [5, 16, 24, 25, 29])
        self.set_layers(context, 'fing_ind_ctrl_mstr_L', [28])
        self.set_layers(context, 'fing_ind_ctrl_L', [5, 16, 24, 25, 29])
        self.set_layers(context, 'fing_mid_ctrl_mstr_L', [28])
        self.set_layers(context, 'fing_mid_ctrl_L', [5, 16, 24, 25, 29])
        self.set_layers(context, 'fing_lit_1_L', [26])
        self.set_layers(context, 'fing_lit_ctrl_shp_at_L', [26])
        self.set_layers(context, 'fing_lit_2_L', [17, 24, 25])
        self.set_layers(context, 'fing_lit_3_L', [17, 24, 25])
        self.set_layers(context, 'fing_lit_4_L', [17, 24, 25])
        self.set_layers(context, 'fing_lit_shp_at_L', [26])
        self.set_layers(context, 'fing_lit_2_rot_L', [26])
        self.set_layers(context, 'fing_lit_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_low_1_L', [24, 27])
        self.set_layers(context, 'fing_lit_fix_up_1_L', [24, 27])
        self.set_layers(context, 'fing_lit_2_def_mstr_L', [26])
        self.set_layers(context, 'fing_lit_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_low_2_L', [24, 27])
        self.set_layers(context, 'fing_lit_fix_up_2_L', [24, 27])
        self.set_layers(context, 'fing_lit_3_def_mstr_L', [26])
        self.set_layers(context, 'fing_lit_3_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_lit_fix_low_3_L', [24, 27])
        self.set_layers(context, 'fing_lit_fix_up_3_L', [24, 27])
        self.set_layers(context, 'fing_lit_4_def_mstr_L', [26])
        self.set_layers(context, 'fing_lit_4_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_1_L', [26])
        self.set_layers(context, 'fing_ring_ctrl_shp_at_L', [26])
        self.set_layers(context, 'fing_ring_2_L', [17, 24, 25])
        self.set_layers(context, 'fing_ring_3_rot_L', [26])
        self.set_layers(context, 'fing_ring_4_rot_L', [26])
        self.set_layers(context, 'fing_ring_3_L', [17, 24, 25])
        self.set_layers(context, 'fing_ring_4_L', [17, 24, 25])
        self.set_layers(context, 'fing_ring_shp_at_L', [26])
        self.set_layers(context, 'fing_ring_2_rot_L', [26])
        self.set_layers(context, 'fing_ring_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_low_1_L', [24, 27])
        self.set_layers(context, 'fing_ring_fix_up_1_L', [24, 27])
        self.set_layers(context, 'fing_ring_2_def_mstr_L', [26])
        self.set_layers(context, 'fing_ring_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_low_2_L', [24, 27])
        self.set_layers(context, 'fing_ring_fix_up_2_L', [24, 27])
        self.set_layers(context, 'fing_ring_3_def_mstr_L', [26])
        self.set_layers(context, 'fing_ring_3_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ring_fix_low_3_L', [24, 27])
        self.set_layers(context, 'fing_ring_fix_up_3_L', [24, 27])
        self.set_layers(context, 'fing_ring_4_def_mstr_L', [26])
        self.set_layers(context, 'fing_ring_4_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_1_L', [26])
        self.set_layers(context, 'fing_ind_ctrl_shp_at_L', [26])
        self.set_layers(context, 'fing_ind_2_L', [17, 24, 25])
        self.set_layers(context, 'fing_ind_3_L', [17, 24, 25])
        self.set_layers(context, 'fing_ind_4_L', [17, 24, 25])
        self.set_layers(context, 'fing_ind_shp_at_L', [26])
        self.set_layers(context, 'fing_ind_2_rot_L', [26])
        self.set_layers(context, 'fing_ind_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_1_L', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_1_L', [24, 27])
        self.set_layers(context, 'fing_ind_2_def_mstr_L', [26])
        self.set_layers(context, 'fing_ind_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_2_L', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_2_L', [24, 27])
        self.set_layers(context, 'fing_ind_3_def_mstr_L', [26])
        self.set_layers(context, 'fing_ind_3_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_ind_fix_low_3_L', [24, 27])
        self.set_layers(context, 'fing_ind_fix_up_3_L', [24, 27])
        self.set_layers(context, 'fing_ind_4_def_mstr_L', [26])
        self.set_layers(context, 'fing_ind_4_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_1_L', [26])
        self.set_layers(context, 'fing_mid_ctrl_shp_at_L', [26])
        self.set_layers(context, 'fing_mid_2_L', [17, 24, 25])
        self.set_layers(context, 'fing_mid_3_L', [17, 24, 25])
        self.set_layers(context, 'fing_mid_4_L', [17, 24, 25])
        self.set_layers(context, 'fing_mid_shp_at_L', [26])
        self.set_layers(context, 'fing_mid_2_rot_L', [26])
        self.set_layers(context, 'fing_mid_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_low_1_L', [24, 27])
        self.set_layers(context, 'fing_mid_fix_up_1_L', [24, 27])
        self.set_layers(context, 'fing_mid_2_def_mstr_L', [26])
        self.set_layers(context, 'fing_mid_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_low_2_L', [24, 27])
        self.set_layers(context, 'fing_mid_fix_up_2_L', [24, 27])
        self.set_layers(context, 'fing_mid_3_def_mstr_L', [26])
        self.set_layers(context, 'fing_mid_3_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'fing_mid_fix_low_3_L', [24, 27])
        self.set_layers(context, 'fing_mid_fix_up_3_L', [24, 27])
        self.set_layers(context, 'fing_mid_4_def_mstr_L', [26])
        self.set_layers(context, 'fing_mid_4_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'hand_accessory_L', [11, 25])
        self.set_layers(context, 'forearm_rot_L', [26])
        self.set_layers(context, 'forearm_twist_L', [26])
        self.set_layers(context, 'forearm_twist_bezier_out_L', [26])
        self.set_layers(context, 'elbow_fix_L', [27])
        self.set_layers(context, 'arm_fix_L', [27])
        self.set_layers(context, 'elbow_line_L', [16, 25, 31])
        self.set_layers(context, 'arm_toon_L', [14, 25])
        self.set_layers(context, 'arm_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'arm_twist_bezier_in_L', [26])
        self.set_layers(context, 'arm_def_bezier_out_mstr_L', [26])
        self.set_layers(context, 'arm_def_bezier_out_L', [26])
        self.set_layers(context, 'forearm_def_curved_L', [26])
        self.set_layers(context, 'forearm_def_bezier_in_mstr_L', [26])
        self.set_layers(context, 'forearm_def_bezier_in_L', [26])
        self.set_layers(context, 'forearm_def_bbone_in_L', [26])
        self.set_layers(context, 'forearm_def_no_twist_bbone_in_L', [26])
        self.set_layers(context, 'forearm_toon_L', [14, 25])
        self.set_layers(context, 'forearm_def_bezier_out_mstr_L', [26])
        self.set_layers(context, 'forearm_def_bezier_out_L', [26])
        self.set_layers(context, 'forearm_def_bezier_out_rot_L', [26])
        self.set_layers(context, 'forearm_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'forearm_twist_bezier_in_L', [26])
        self.set_layers(context, 'forearm_twist_bbone_in_L', [26])
        self.set_layers(context, 'forearm_twist_def_L', [27, 31])
        self.set_layers(context, 'forearm_twist_bbone_out_L', [26])
        self.set_layers(context, 'hand_twist_drv_L', [26])
        self.set_layers(context, 'wrist_fix_low_L', [27])
        self.set_layers(context, 'wrist_fix_up_L', [27])
        self.set_layers(context, 'forearm_no_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'forearm_no_twist_bezier_in_L', [26])
        self.set_layers(context, 'forearm_no_twist_bbone_in_L', [26])
        self.set_layers(context, 'forearm_no_twist_def_L', [27])
        self.set_layers(context, 'forearm_def_L', [27, 31])
        self.set_layers(context, 'forearm_def_bbone_out_L', [26])
        self.set_layers(context, 'forearm_def_no_twist_L', [27])
        self.set_layers(context, 'forearm_def_bbone_out_no_twist_L', [26])
        self.set_layers(context, 'arm_drv_L', [26])
        self.set_layers(context, 'back_fix_L', [27])
        self.set_layers(context, 'chest_fix_L', [27])
        self.set_layers(context, 'arm_twist_L', [26])
        self.set_layers(context, 'arm_twist_bbone_in_L', [26])
        self.set_layers(context, 'arm_twist_def_L', [27, 31])
        self.set_layers(context, 'arm_twist_curved_L', [26])
        self.set_layers(context, 'arm_twist_bezier_out_mstr_L', [26])
        self.set_layers(context, 'arm_twist_bezier_out_L', [26])
        self.set_layers(context, 'arm_twist_bbone_out_rot_L', [26])
        self.set_layers(context, 'arm_twist_bbone_out_L', [26])
        self.set_layers(context, 'arm_def_bezier_in_L', [26])
        self.set_layers(context, 'arm_def_bbone_in_L', [26])
        self.set_layers(context, 'arm_def_L', [27, 31])
        self.set_layers(context, 'arm_def_bbone_out_L', [26])
        self.set_layers(context, 'shin_ik_R', [23, 25, 30])
        self.set_layers(context, 'shin_fk_R', [20, 25, 29, 30])
        self.set_layers(context, 'foot_toon_R', [14, 25])
        self.set_layers(context, 'shin_twist_bezier_out_mstr_R', [26])
        self.set_layers(context, 'foot_scale_R', [26])
        self.set_layers(context, 'foot_fk_R', [20, 25, 30])
        self.set_layers(context, 'toes_spread_R', [10, 24, 25])
        self.set_layers(context, 'toe_lit_str_1_R', [24])
        self.set_layers(context, 'toe_fourth_str_1_R', [24])
        self.set_layers(context, 'toe_mid_str_1_R', [24])
        self.set_layers(context, 'toe_big_str_1_R', [24])
        self.set_layers(context, 'toe_ind_str_1_R', [24])
        self.set_layers(context, 'snap_sole_ctrl_fk_R', [30])
        self.set_layers(context, 'toe_1_fk_R', [20, 23, 24, 25, 30])
        self.set_layers(context, 'snap_toes_ctrl_mid_fk_R', [30])
        self.set_layers(context, 'toe_2_fk_R', [20, 23, 24, 25, 30])
        self.set_layers(context, 'snap_toes_ctrl_fk_R', [30])
        self.set_layers(context, 'toes_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_1_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'toe_2_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'toe_fix_up_1_R', [24, 27])
        self.set_layers(context, 'toe_fix_low_1_R', [24, 27])
        self.set_layers(context, 'toe_big_ctrl_mstr_R', [26])
        self.set_layers(context, 'toe_big_ctrl_R', [10, 24, 25, 29])
        self.set_layers(context, 'toe_big_1_R', [26])
        self.set_layers(context, 'toe_big_2_R', [12, 24, 25])
        self.set_layers(context, 'toe_big_3_R', [12, 24, 25])
        self.set_layers(context, 'toe_big_shp_at_R', [26])
        self.set_layers(context, 'toe_big_2_rot_R', [26])
        self.set_layers(context, 'toe_big_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_big_1_def_R', [24, 29])
        self.set_layers(context, 'toe_big_fix_up_1_R', [24])
        self.set_layers(context, 'toe_big_fix_low_1_R', [24])
        self.set_layers(context, 'toe_big_2_def_mstr_R', [26])
        self.set_layers(context, 'toe_big_2_def_R', [24, 29])
        self.set_layers(context, 'toe_big_fix_up_2_R', [24])
        self.set_layers(context, 'toe_big_fix_low_2_R', [24])
        self.set_layers(context, 'toe_big_3_def_mstr_R', [26])
        self.set_layers(context, 'toe_big_3_def_R', [24, 29])
        self.set_layers(context, 'toe_lit_ctrl_mstr_R', [26])
        self.set_layers(context, 'toe_lit_ctrl_R', [10, 24, 25, 29])
        self.set_layers(context, 'toe_lit_1_R', [26])
        self.set_layers(context, 'toe_lit_2_R', [12, 24, 25])
        self.set_layers(context, 'toe_lit_3_R', [12, 24, 25])
        self.set_layers(context, 'toe_lit_shp_at_R', [26])
        self.set_layers(context, 'toe_lit_2_rot_R', [26])
        self.set_layers(context, 'toe_lit_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_lit_1_def_R', [24, 29])
        self.set_layers(context, 'toe_lit_fix_up_1_R', [24])
        self.set_layers(context, 'toe_lit_fix_low_1_R', [24])
        self.set_layers(context, 'toe_lit_2_def_mstr_R', [26])
        self.set_layers(context, 'toe_lit_2_def_R', [24, 29])
        self.set_layers(context, 'toe_lit_fix_up_2_R', [24])
        self.set_layers(context, 'toe_lit_fix_low_2_R', [24])
        self.set_layers(context, 'toe_lit_3_def_mstr_R', [26])
        self.set_layers(context, 'toe_lit_3_def_R', [24, 29])
        self.set_layers(context, 'toe_fourth_ctrl_mstr_R', [26])
        self.set_layers(context, 'toe_fourth_ctrl_R', [10, 24, 25, 29])
        self.set_layers(context, 'toe_fourth_1_R', [26])
        self.set_layers(context, 'toe_fourth_2_R', [12, 24, 25])
        self.set_layers(context, 'toe_fourth_3_R', [12, 24, 25])
        self.set_layers(context, 'toe_fourth_4_R', [12, 24, 25])
        self.set_layers(context, 'toe_fourth_shp_at_R', [26])
        self.set_layers(context, 'toe_fourth_2_rot_R', [26])
        self.set_layers(context, 'toe_fourth_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_fourth_1_def_R', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_1_R', [24])
        self.set_layers(context, 'toe_fourth_fix_low_1_R', [24])
        self.set_layers(context, 'toe_fourth_2_def_mstr_R', [26])
        self.set_layers(context, 'toe_fourth_2_def_R', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_2_R', [24])
        self.set_layers(context, 'toe_fourth_fix_low_2_R', [24])
        self.set_layers(context, 'toe_fourth_3_def_mstr_R', [26])
        self.set_layers(context, 'toe_fourth_3_def_R', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_3_R', [24])
        self.set_layers(context, 'toe_fourth_fix_low_3_R', [24])
        self.set_layers(context, 'toe_fourth_4_def_mstr_R', [26])
        self.set_layers(context, 'toe_fourth_4_def_R', [24, 29])
        self.set_layers(context, 'toe_mid_ctrl_mstr_R', [26])
        self.set_layers(context, 'toe_mid_ctrl_R', [10, 24, 25, 29])
        self.set_layers(context, 'toe_mid_1_R', [26])
        self.set_layers(context, 'toe_mid_2_R', [12, 24, 25])
        self.set_layers(context, 'toe_mid_3_R', [12, 24, 25])
        self.set_layers(context, 'toe_mid_4_R', [12, 24, 25])
        self.set_layers(context, 'toe_mid_shp_at_R', [26])
        self.set_layers(context, 'toe_mid_2_rot_R', [26])
        self.set_layers(context, 'toe_mid_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_mid_1_def_R', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_1_R', [24])
        self.set_layers(context, 'toe_mid_fix_low_1_R', [24])
        self.set_layers(context, 'toe_mid_2_def_mstr_R', [26])
        self.set_layers(context, 'toe_mid_2_def_R', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_2_R', [24])
        self.set_layers(context, 'toe_mid_fix_low_2_R', [24])
        self.set_layers(context, 'toe_mid_3_def_mstr_R', [26])
        self.set_layers(context, 'toe_mid_3_def_R', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_3_R', [24])
        self.set_layers(context, 'toe_mid_fix_low_3_R', [24])
        self.set_layers(context, 'toe_mid_4_def_mstr_R', [26])
        self.set_layers(context, 'toe_mid_4_def_R', [24, 29])
        self.set_layers(context, 'toe_ind_ctrl_mstr_R', [26])
        self.set_layers(context, 'toe_ind_ctrl_R', [10, 24, 25, 29])
        self.set_layers(context, 'toe_ind_1_R', [26])
        self.set_layers(context, 'toe_ind_2_R', [12, 24, 25])
        self.set_layers(context, 'toe_ind_3_R', [12, 24, 25])
        self.set_layers(context, 'toe_ind_4_R', [12, 24, 25])
        self.set_layers(context, 'toe_ind_shp_at_R', [26])
        self.set_layers(context, 'toe_ind_2_rot_R', [26])
        self.set_layers(context, 'toe_ind_ctrl_shp_at_R', [26])
        self.set_layers(context, 'toe_ind_1_def_R', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_1_R', [24])
        self.set_layers(context, 'toe_ind_fix_low_1_R', [24])
        self.set_layers(context, 'toe_ind_2_def_mstr_R', [26])
        self.set_layers(context, 'toe_ind_2_def_R', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_2_R', [24])
        self.set_layers(context, 'toe_ind_fix_low_2_R', [24])
        self.set_layers(context, 'toe_ind_3_def_mstr_R', [26])
        self.set_layers(context, 'toe_ind_3_def_R', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_3_R', [24])
        self.set_layers(context, 'toe_ind_fix_low_3_R', [24])
        self.set_layers(context, 'toe_ind_4_def_mstr_R', [26])
        self.set_layers(context, 'toe_ind_4_def_R', [24, 29])
        self.set_layers(context, 'shin_rot_R', [26])
        self.set_layers(context, 'foot_def_R', [24, 27, 29, 31])
        self.set_layers(context, 'foot_drv_R', [26])
        self.set_layers(context, 'thigh_def_mstr_R', [26])
        self.set_layers(context, 'knee_toon_R', [13, 25])
        self.set_layers(context, 'knee_toon_loc_R', [26])
        self.set_layers(context, 'thigh_toon_mstr_R', [26])
        self.set_layers(context, 'thigh_drv_R', [26])
        self.set_layers(context, 'hip_fix_R', [27])
        self.set_layers(context, 'thigh_fix_R', [27])
        self.set_layers(context, 'buttock_ctrl_R', [8, 18, 21, 26, 31])
        self.set_layers(context, 'buttock_def_R', [27, 31])
        self.set_layers(context, 'thigh_twist_R', [26])
        self.set_layers(context, 'thigh_def_bezier_in_R', [26])
        self.set_layers(context, 'thigh_def_bbone_in_R', [26])
        self.set_layers(context, 'knee_fix_R', [27])
        self.set_layers(context, 'shin_fix_R', [27])
        self.set_layers(context, 'thigh_toon_R', [14, 25])
        self.set_layers(context, 'thigh_def_bezier_out_mstr_R', [26])
        self.set_layers(context, 'thigh_def_bezier_out_R', [26])
        self.set_layers(context, 'thigh_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'thigh_twist_bezier_in_R', [26])
        self.set_layers(context, 'thigh_twist_bbone_in_R', [26])
        self.set_layers(context, 'thigh_twist_def_R', [27, 31])
        self.set_layers(context, 'thigh_def_R', [27, 31])
        self.set_layers(context, 'thigh_def_bbone_out_R', [26])
        self.set_layers(context, 'shin_mstr_R', [26])
        self.set_layers(context, 'shin_def_mstr_R', [26])
        self.set_layers(context, 'shin_toon_2_mstr_R', [26])
        self.set_layers(context, 'shin_toon_2_R', [26])
        self.set_layers(context, 'shin_toon_mstr_R', [26])
        self.set_layers(context, 'shin_toon_def_R', [26])
        self.set_layers(context, 'shin_twist_R', [26])
        self.set_layers(context, 'shin_twist_bezier_out_R', [26])
        self.set_layers(context, 'shin_def_curved_R', [26])
        self.set_layers(context, 'shin_def_bezier_in_mstr_R', [26])
        self.set_layers(context, 'shin_def_bezier_in_R', [26])
        self.set_layers(context, 'shin_def_bbone_in_R', [26])
        self.set_layers(context, 'shin_def_no_twist_bbone_in_R', [26])
        self.set_layers(context, 'shin_toon_R', [14, 25])
        self.set_layers(context, 'shin_def_bezier_out_mstr_R', [26])
        self.set_layers(context, 'shin_def_bezier_out_R', [26])
        self.set_layers(context, 'shin_def_bezier_out_rot_R', [26])
        self.set_layers(context, 'shin_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'shin_twist_bezier_in_R', [26])
        self.set_layers(context, 'shin_twist_bbone_in_R', [26])
        self.set_layers(context, 'shin_twist_def_R', [27, 31])
        self.set_layers(context, 'shin_twist_bbone_out_R', [26])
        self.set_layers(context, 'shin_no_twist_bezier_in_mstr_R', [26])
        self.set_layers(context, 'shin_no_twist_bezier_in_R', [26])
        self.set_layers(context, 'shin_no_twist_bbone_in_R', [26])
        self.set_layers(context, 'shin_no_twist_def_R', [27])
        self.set_layers(context, 'shin_def_R', [27, 31])
        self.set_layers(context, 'shin_def_bbone_out_R', [26])
        self.set_layers(context, 'shin_def_no_twist_R', [27])
        self.set_layers(context, 'shin_def_no_twist_bbone_out_R', [26])
        self.set_layers(context, 'thigh_twist_curved_R', [26])
        self.set_layers(context, 'thigh_twist_bezier_out_mstr_R', [26])
        self.set_layers(context, 'thigh_twist_bezier_out_R', [26])
        self.set_layers(context, 'thigh_twist_bbone_out_rot_R', [26])
        self.set_layers(context, 'thigh_twist_bbone_out_R', [26])
        self.set_layers(context, 'knee_line_R', [23, 25, 31])
        self.set_layers(context, 'shin_ik_L', [9, 25, 30])
        self.set_layers(context, 'shin_fk_L', [22, 25, 29, 30])
        self.set_layers(context, 'foot_toon_L', [14, 25])
        self.set_layers(context, 'shin_twist_bezier_out_mstr_L', [26])
        self.set_layers(context, 'foot_scale_L', [26])
        self.set_layers(context, 'foot_fk_L', [22, 25, 30])
        self.set_layers(context, 'toes_spread_L', [10, 24, 25])
        self.set_layers(context, 'toe_lit_str_1_L', [24])
        self.set_layers(context, 'toe_fourth_str_1_L', [24])
        self.set_layers(context, 'toe_mid_str_1_L', [24])
        self.set_layers(context, 'toe_big_str_1_L', [24])
        self.set_layers(context, 'toe_ind_str_1_L', [24])
        self.set_layers(context, 'toe_1_fk_L', [9, 22, 24, 25, 30])
        self.set_layers(context, 'snap_toes_ctrl_mid_fk_L', [30])
        self.set_layers(context, 'toe_2_fk_L', [9, 22, 24, 25, 30])
        self.set_layers(context, 'snap_toes_ctrl_fk_L', [30])
        self.set_layers(context, 'toes_ctrl_shp_at_L', [26])
        self.set_layers(context, 'toe_1_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'toe_2_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'toe_fix_up_1_L', [24, 27])
        self.set_layers(context, 'toe_fix_low_1_L', [24, 27])
        self.set_layers(context, 'snap_sole_ctrl_fk_L', [30])
        self.set_layers(context, 'toe_lit_ctrl_mstr_L', [26])
        self.set_layers(context, 'toe_lit_ctrl_L', [10, 24, 25, 29])
        self.set_layers(context, 'toe_lit_1_L', [26])
        self.set_layers(context, 'toe_lit_2_L', [12, 24, 25])
        self.set_layers(context, 'toe_lit_3_L', [12, 24, 25])
        self.set_layers(context, 'toe_lit_shp_at_L', [26])
        self.set_layers(context, 'toe_lit_2_rot_L', [26])
        self.set_layers(context, 'toe_lit_1_def_L', [24, 29])
        self.set_layers(context, 'toe_lit_fix_up_1_L', [24])
        self.set_layers(context, 'toe_lit_fix_low_1_L', [24])
        self.set_layers(context, 'toe_lit_2_def_mstr_L', [26])
        self.set_layers(context, 'toe_lit_2_def_L', [24, 29])
        self.set_layers(context, 'toe_lit_fix_up_2_L', [24])
        self.set_layers(context, 'toe_lit_fix_low_2_L', [24])
        self.set_layers(context, 'toe_lit_3_def_mstr_L', [26])
        self.set_layers(context, 'toe_lit_3_def_L', [24, 29])
        self.set_layers(context, 'toe_lit_ctrl_shp_at_L', [26])
        self.set_layers(context, 'toe_big_ctrl_mstr_L', [26])
        self.set_layers(context, 'toe_big_ctrl_L', [10, 24, 25, 29])
        self.set_layers(context, 'toe_big_1_L', [26])
        self.set_layers(context, 'toe_big_2_L', [12, 24, 25])
        self.set_layers(context, 'toe_big_3_L', [12, 24, 25])
        self.set_layers(context, 'toe_big_shp_at_L', [26])
        self.set_layers(context, 'toe_big_ctrl_shp_at_L', [26])
        self.set_layers(context, 'toe_big_2_rot_L', [26])
        self.set_layers(context, 'toe_big_1_def_L', [24, 29])
        self.set_layers(context, 'toe_big_fix_up_1_L', [24])
        self.set_layers(context, 'toe_big_fix_low_1_L', [24])
        self.set_layers(context, 'toe_big_2_def_mstr_L', [26])
        self.set_layers(context, 'toe_big_2_def_L', [24, 29])
        self.set_layers(context, 'toe_big_fix_up_2_L', [24])
        self.set_layers(context, 'toe_big_fix_low_2_L', [24])
        self.set_layers(context, 'toe_big_3_def_mstr_L', [26])
        self.set_layers(context, 'toe_big_3_def_L', [24, 29])
        self.set_layers(context, 'toe_fourth_ctrl_mstr_L', [26])
        self.set_layers(context, 'toe_fourth_ctrl_L', [10, 24, 25, 29])
        self.set_layers(context, 'toe_fourth_1_L', [26])
        self.set_layers(context, 'toe_fourth_2_L', [12, 24, 25])
        self.set_layers(context, 'toe_fourth_3_L', [12, 24, 25])
        self.set_layers(context, 'toe_fourth_4_L', [12, 24, 25])
        self.set_layers(context, 'toe_fourth_shp_at_L', [26])
        self.set_layers(context, 'toe_fourth_2_rot_L', [26])
        self.set_layers(context, 'toe_fourth_1_def_L', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_1_L', [24])
        self.set_layers(context, 'toe_fourth_fix_low_1_L', [24])
        self.set_layers(context, 'toe_fourth_2_def_mstr_L', [26])
        self.set_layers(context, 'toe_fourth_2_def_L', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_2_L', [24])
        self.set_layers(context, 'toe_fourth_fix_low_2_L', [24])
        self.set_layers(context, 'toe_fourth_3_def_mstr_L', [26])
        self.set_layers(context, 'toe_fourth_3_def_L', [24, 29])
        self.set_layers(context, 'toe_fourth_fix_up_3_L', [24])
        self.set_layers(context, 'toe_fourth_fix_low_3_L', [24])
        self.set_layers(context, 'toe_fourth_4_def_mstr_L', [26])
        self.set_layers(context, 'toe_fourth_4_def_L', [24, 29])
        self.set_layers(context, 'toe_fourth_ctrl_shp_at_L', [26])
        self.set_layers(context, 'toe_mid_ctrl_mstr_L', [26])
        self.set_layers(context, 'toe_mid_ctrl_L', [10, 24, 25, 29])
        self.set_layers(context, 'toe_mid_1_L', [26])
        self.set_layers(context, 'toe_mid_2_L', [12, 24, 25])
        self.set_layers(context, 'toe_mid_3_L', [12, 24, 25])
        self.set_layers(context, 'toe_mid_4_L', [12, 24, 25])
        self.set_layers(context, 'toe_mid_shp_at_L', [26])
        self.set_layers(context, 'toe_mid_2_rot_L', [26])
        self.set_layers(context, 'toe_mid_ctrl_shp_at_L', [26])
        self.set_layers(context, 'toe_mid_1_def_L', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_1_L', [24])
        self.set_layers(context, 'toe_mid_fix_low_1_L', [24])
        self.set_layers(context, 'toe_mid_2_def_mstr_L', [26])
        self.set_layers(context, 'toe_mid_2_def_L', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_2_L', [24])
        self.set_layers(context, 'toe_mid_fix_low_2_L', [24])
        self.set_layers(context, 'toe_mid_3_def_mstr_L', [26])
        self.set_layers(context, 'toe_mid_3_def_L', [24, 29])
        self.set_layers(context, 'toe_mid_fix_up_3_L', [24])
        self.set_layers(context, 'toe_mid_fix_low_3_L', [24])
        self.set_layers(context, 'toe_mid_4_def_mstr_L', [26])
        self.set_layers(context, 'toe_mid_4_def_L', [24, 29])
        self.set_layers(context, 'toe_ind_ctrl_mstr_L', [26])
        self.set_layers(context, 'toe_ind_ctrl_L', [10, 24, 25, 29])
        self.set_layers(context, 'toe_ind_1_L', [26])
        self.set_layers(context, 'toe_ind_2_L', [12, 24, 25])
        self.set_layers(context, 'toe_ind_3_L', [12, 24, 25])
        self.set_layers(context, 'toe_ind_4_L', [12, 24, 25])
        self.set_layers(context, 'toe_ind_shp_at_L', [26])
        self.set_layers(context, 'toe_ind_2_rot_L', [26])
        self.set_layers(context, 'toe_ind_1_def_L', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_1_L', [24])
        self.set_layers(context, 'toe_ind_fix_low_1_L', [24])
        self.set_layers(context, 'toe_ind_2_def_mstr_L', [26])
        self.set_layers(context, 'toe_ind_2_def_L', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_2_L', [24])
        self.set_layers(context, 'toe_ind_fix_low_2_L', [24])
        self.set_layers(context, 'toe_ind_3_def_mstr_L', [26])
        self.set_layers(context, 'toe_ind_3_def_L', [24, 29])
        self.set_layers(context, 'toe_ind_fix_up_3_L', [24])
        self.set_layers(context, 'toe_ind_fix_low_3_L', [24])
        self.set_layers(context, 'toe_ind_4_def_mstr_L', [26])
        self.set_layers(context, 'toe_ind_4_def_L', [24, 29])
        self.set_layers(context, 'toe_ind_ctrl_shp_at_L', [26])
        self.set_layers(context, 'shin_rot_L', [26])
        self.set_layers(context, 'foot_def_L', [24, 27, 29, 31])
        self.set_layers(context, 'foot_drv_L', [26])
        self.set_layers(context, 'thigh_def_mstr_L', [26])
        self.set_layers(context, 'knee_toon_L', [13, 25])
        self.set_layers(context, 'knee_toon_loc_L', [26])
        self.set_layers(context, 'thigh_toon_mstr_L', [26])
        self.set_layers(context, 'thigh_drv_L', [26])
        self.set_layers(context, 'buttock_ctrl_L', [8, 18, 21, 26, 31])
        self.set_layers(context, 'buttock_def_L', [27, 31])
        self.set_layers(context, 'hip_fix_L', [27])
        self.set_layers(context, 'thigh_fix_L', [27])
        self.set_layers(context, 'thigh_twist_L', [26])
        self.set_layers(context, 'thigh_def_bezier_in_L', [26])
        self.set_layers(context, 'thigh_def_bbone_in_L', [26])
        self.set_layers(context, 'knee_fix_L', [27])
        self.set_layers(context, 'shin_fix_L', [27])
        self.set_layers(context, 'thigh_toon_L', [14, 25])
        self.set_layers(context, 'thigh_def_bezier_out_mstr_L', [26])
        self.set_layers(context, 'thigh_def_bezier_out_L', [26])
        self.set_layers(context, 'thigh_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'thigh_twist_bezier_in_L', [26])
        self.set_layers(context, 'thigh_twist_bbone_in_L', [26])
        self.set_layers(context, 'thigh_twist_def_L', [27, 31])
        self.set_layers(context, 'thigh_def_L', [27, 31])
        self.set_layers(context, 'thigh_def_bbone_out_L', [26])
        self.set_layers(context, 'shin_mstr_L', [26])
        self.set_layers(context, 'shin_def_mstr_L', [26])
        self.set_layers(context, 'shin_toon_2_mstr_L', [26])
        self.set_layers(context, 'shin_toon_2_L', [26])
        self.set_layers(context, 'shin_toon_mstr_L', [26])
        self.set_layers(context, 'shin_toon_def_L', [26])
        self.set_layers(context, 'shin_twist_L', [26])
        self.set_layers(context, 'shin_twist_bezier_out_L', [26])
        self.set_layers(context, 'shin_def_curved_L', [26])
        self.set_layers(context, 'shin_def_bezier_in_mstr_L', [26])
        self.set_layers(context, 'shin_def_bezier_in_L', [26])
        self.set_layers(context, 'shin_def_bbone_in_L', [26])
        self.set_layers(context, 'shin_def_no_twist_bbone_in_L', [26])
        self.set_layers(context, 'shin_toon_L', [14, 25])
        self.set_layers(context, 'shin_def_bezier_out_mstr_L', [26])
        self.set_layers(context, 'shin_def_bezier_out_L', [26])
        self.set_layers(context, 'shin_def_bezier_out_rot_L', [26])
        self.set_layers(context, 'shin_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'shin_twist_bezier_in_L', [26])
        self.set_layers(context, 'shin_twist_bbone_in_L', [26])
        self.set_layers(context, 'shin_twist_def_L', [27, 31])
        self.set_layers(context, 'shin_twist_bbone_out_L', [26])
        self.set_layers(context, 'shin_no_twist_bezier_in_mstr_L', [26])
        self.set_layers(context, 'shin_no_twist_bezier_in_L', [26])
        self.set_layers(context, 'shin_no_twist_bbone_in_L', [26])
        self.set_layers(context, 'shin_no_twist_def_L', [27])
        self.set_layers(context, 'shin_def_L', [27, 31])
        self.set_layers(context, 'shin_def_bbone_out_L', [26])
        self.set_layers(context, 'shin_def_no_twist_L', [27])
        self.set_layers(context, 'shin_def_no_twist_bbone_out_L', [26])
        self.set_layers(context, 'thigh_twist_curved_L', [26])
        self.set_layers(context, 'thigh_twist_bezier_out_mstr_L', [26])
        self.set_layers(context, 'thigh_twist_bezier_out_L', [26])
        self.set_layers(context, 'thigh_twist_bbone_out_rot_L', [26])
        self.set_layers(context, 'thigh_twist_bbone_out_L', [26])
        self.set_layers(context, 'knee_line_L', [9, 25, 31])
        self.set_layers(context, 'look_mstr', [26])
        self.set_layers(context, 'look', [0, 8, 18, 21, 25])
        self.set_layers(context, 'look_L', [0, 8, 18, 21, 25, 31])
        self.set_layers(context, 'eye_ctrl_L', [0, 8, 18, 21, 25])
        self.set_layers(context, 'pupil_ctrl_L', [0, 8, 18, 21, 25])
        self.set_layers(context, 'iris_ctrl_L', [0, 8, 18, 21, 25])
        self.set_layers(context, 'eye_mstr_str_L', [29, 31])
        self.set_layers(context, 'toon_eye_up_L', [14, 25])
        self.set_layers(context, 'toon_eye_out_L', [14, 25])
        self.set_layers(context, 'toon_eye_in_L', [14, 25])
        self.set_layers(context, 'toon_eye_low_L', [14, 25])
        self.set_layers(context, 'eye_mstr_vert_roll_L', [29])
        self.set_layers(context, 'eye_def_L', [27])
        self.set_layers(context, 'lattice_eye_L', [7, 25, 26])
        self.set_layers(context, 'eyelid_up_ctrl_L', [0, 25, 28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_2_L', [1, 25])
        self.set_layers(context, 'eyelid_up_vert_def_2_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_up_vert_def_2_L', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_2_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_1_L', [1, 25])
        self.set_layers(context, 'eyelid_up_vert_def_1_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_up_3_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_3_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_up_3_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_up_1_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_1_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_up_vert_def_1_L', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_1_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_2_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_up_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_3_L', [1, 25])
        self.set_layers(context, 'eyelid_up_vert_def_3_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_up_2_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_up_4_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_4_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_up_4_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_up_vert_def_3_L', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_3_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_3_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_up_3_def_L', [27, 31])
        self.set_layers(context, 'eyelid_low_ctrl_L', [0, 25, 28, 31])
        self.set_layers(context, 'eyelid_ctrl_out_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_ctrl_out_L', [1, 25])
        self.set_layers(context, 'eyelid_out_vert_def_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_out_vert_def_L', [27, 31])
        self.set_layers(context, 'eyelid_out_vert_def_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_3_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_3_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_up_4_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_up_3_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_4_def_L', [27, 31])
        self.set_layers(context, 'eyelid_up_4_L_bbone_out_L', [26, 31])
        self.set_layers(context, 'eyelid_low_ctrl_2_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_2_L', [1, 25])
        self.set_layers(context, 'eyelid_low_vert_def_2_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_low_vert_def_2_L', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_2_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_ctrl_1_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_1_L', [1, 25])
        self.set_layers(context, 'eyelid_low_vert_def_1_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_low_1_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_1_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_low_3_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_low_2_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_low_vert_def_1_L', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_1_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_ctrl_3_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_3_L', [1, 25])
        self.set_layers(context, 'eyelid_low_vert_def_3_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_low_4_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_4_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_low_4_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_low_vert_def_3_L', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_3_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_3_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_low_3_def_L', [27, 31])
        self.set_layers(context, 'eyelid_low_3_bezier_out_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_out_L', [26])
        self.set_layers(context, 'eyelid_low_4_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_low_3_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_4_def_L', [27, 31])
        self.set_layers(context, 'eyelid_low_4_L_bbone_out_L', [26, 31])
        self.set_layers(context, 'eyelid_ctrl_in_mstr_L', [28, 31])
        self.set_layers(context, 'eyelid_ctrl_in_L', [1, 25])
        self.set_layers(context, 'eyelid_in_vert_def_bezier_out_L', [26, 31])
        self.set_layers(context, 'eyelid_in_vert_def_L', [27, 31])
        self.set_layers(context, 'eyelid_in_vert_def_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_low_2_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_low_2_def_L', [27, 31])
        self.set_layers(context, 'eyelid_low_2_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_low_1_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_low_1_L_bezier_in_mstr_L', [26, 31])
        self.set_layers(context, 'eyelid_low_1_L_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_low_1_L_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_low_1_def_L', [27, 31])
        self.set_layers(context, 'eyelid_low_1_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_in_mstr_L', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_up_2_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_up_2_def_L', [27, 31])
        self.set_layers(context, 'eyelid_up_2_bbone_out_L', [26])
        self.set_layers(context, 'eyelid_up_1_rot_L', [26, 29])
        self.set_layers(context, 'eyelid_up_1_L_bezier_in_mstr_L', [26, 31])
        self.set_layers(context, 'eyelid_up_1_L_bezier_in_L', [26])
        self.set_layers(context, 'eyelid_up_1_L_bbone_in_L', [26])
        self.set_layers(context, 'eyelid_up_1_def_L', [27, 31])
        self.set_layers(context, 'eyelid_up_1_bbone_out_L', [26])
        self.set_layers(context, 'look_R', [0, 8, 18, 21, 25, 31])
        self.set_layers(context, 'eye_ctrl_R', [0, 8, 18, 21, 25])
        self.set_layers(context, 'pupil_ctrl_R', [0, 8, 18, 21, 25])
        self.set_layers(context, 'iris_ctrl_R', [0, 8, 18, 21, 25])
        self.set_layers(context, 'eye_mstr_str_R', [29, 31])
        self.set_layers(context, 'toon_eye_up_R', [14, 25])
        self.set_layers(context, 'toon_eye_out_R', [14, 25])
        self.set_layers(context, 'toon_eye_in_R', [14, 25])
        self.set_layers(context, 'toon_eye_low_R', [14, 25])
        self.set_layers(context, 'eye_mstr_vert_roll_R', [29])
        self.set_layers(context, 'eye_def_R', [27])
        self.set_layers(context, 'lattice_eye_R', [7, 25, 26])
        self.set_layers(context, 'eyelid_low_ctrl_R', [0, 25, 28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_R', [0, 25, 28, 31])
        self.set_layers(context, 'eyelid_up_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_2_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_2_R', [1, 25])
        self.set_layers(context, 'eyelid_low_vert_def_2_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_low_vert_def_2_R', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_2_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_ctrl_2_R', [1, 25])
        self.set_layers(context, 'eyelid_up_vert_def_2_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_up_vert_def_2_R', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_2_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_1_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_1_R', [1, 25])
        self.set_layers(context, 'eyelid_low_vert_def_1_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_low_1_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_1_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_low_3_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_low_vert_def_1_R', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_1_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_2_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_up_ctrl_1_R', [1, 25])
        self.set_layers(context, 'eyelid_up_vert_def_1_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_up_3_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_3_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_up_3_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_up_1_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_1_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_up_vert_def_1_R', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_1_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_2_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_up_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_3_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_low_ctrl_3_R', [1, 25])
        self.set_layers(context, 'eyelid_low_vert_def_3_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_low_4_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_4_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_low_4_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_low_vert_def_3_R', [27, 31])
        self.set_layers(context, 'eyelid_low_vert_def_3_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_3_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_low_3_def_R', [27, 31])
        self.set_layers(context, 'eyelid_up_ctrl_3_R', [1, 25])
        self.set_layers(context, 'eyelid_up_vert_def_3_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_up_2_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_up_4_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_4_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_up_4_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_up_vert_def_3_R', [27, 31])
        self.set_layers(context, 'eyelid_up_vert_def_3_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_3_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_up_3_def_R', [27, 31])
        self.set_layers(context, 'eyelid_ctrl_in_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_ctrl_in_R', [1, 25])
        self.set_layers(context, 'eyelid_in_vert_def_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_in_vert_def_R', [27, 31])
        self.set_layers(context, 'eyelid_in_vert_def_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_2_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_low_2_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_low_2_def_R', [27, 31])
        self.set_layers(context, 'eyelid_low_2_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_1_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_low_1_L_bezier_in_mstr_R', [26, 31])
        self.set_layers(context, 'eyelid_low_1_L_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_low_1_L_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_low_1_def_R', [27, 31])
        self.set_layers(context, 'eyelid_low_1_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_in_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_2_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_up_2_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_up_2_def_R', [27, 31])
        self.set_layers(context, 'eyelid_up_2_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_1_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_up_1_L_bezier_in_mstr_R', [26, 31])
        self.set_layers(context, 'eyelid_up_1_L_bezier_in_R', [26])
        self.set_layers(context, 'eyelid_up_1_L_bbone_in_R', [26])
        self.set_layers(context, 'eyelid_up_1_def_R', [27, 31])
        self.set_layers(context, 'eyelid_up_1_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_ctrl_out_mstr_R', [28, 31])
        self.set_layers(context, 'eyelid_ctrl_out_R', [1, 25])
        self.set_layers(context, 'eyelid_out_vert_def_bezier_out_R', [26, 31])
        self.set_layers(context, 'eyelid_out_vert_def_R', [27, 31])
        self.set_layers(context, 'eyelid_out_vert_def_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_low_3_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_low_4_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_low_3_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_low_4_def_R', [27, 31])
        self.set_layers(context, 'eyelid_low_4_L_bbone_out_R', [26, 31])
        self.set_layers(context, 'eyelid_up_3_bezier_out_mstr_R', [26])
        self.set_layers(context, 'eyelid_up_3_bezier_out_R', [26])
        self.set_layers(context, 'eyelid_up_4_rot_R', [26, 29])
        self.set_layers(context, 'eyelid_up_3_bbone_out_R', [26])
        self.set_layers(context, 'eyelid_up_4_def_R', [27, 31])
        self.set_layers(context, 'eyelid_up_4_L_bbone_out_R', [26, 31])
        self.set_layers(context, 'hat_free', [11, 25, 31])
        self.set_layers(context, 'hat_head', [26])
        self.set_layers(context, 'hat_mstr', [11, 25])
        self.set_layers(context, 'eyeglasses_free', [11, 25, 31])
        self.set_layers(context, 'eyeglasses_head', [26])
        self.set_layers(context, 'eyeglasses_mstr', [11, 25])
        self.set_layers(context, 'accessory_mstr', [26])
        self.set_layers(context, 'accessory', [11, 25])

    def set_data_props(self, context):
        scene = bpy.context.view_layer      
        bpy.context.active_object.data['bone_auto_hide'] = 0.0
        bpy.context.active_object.data['custom_layers'] = 1.0
        bpy.context.active_object.data['layers_count'] = 24
        bpy.context.active_object.data['layer_list'] = 'FACIAL 1, FACIAL 2, FACIAL 3, ARM_R FK, NECK FK, ARM_L FK, ARM_R IK, NECK IK, ARM_L IK, FINGERS, TORSO FK, FINGERS IK, LEG_R FK, TORSO IK, LEG_L FK, LEG_R IK, TORSO INV, LEG_L IK, TOES, EXTRAS, TOES FK, TOON 1, TOON 2, SCALE, OPTIONALS, PROTECTED, MECH, DEFORMATION, ACTIONS, BONE-ROLLS, SNAPPING, REPROPORTION'
        scene.update()         

    def update_scene(self, context):
        current = bpy.context.scene.frame_current
        bpy.context.scene.frame_set(2, subframe=0)
        bpy.context.scene.frame_set(current, subframe=0)

    def unhide_bones(self, context):
        pbones = bpy.context.active_object.pose.bones
        for b in pbones:
            b.bone.hide = False

    def execute(self, context):
        self.expanded_layers(context)   
        self.set_data_props(context) 
        self.update_scene(context) 
        self.unhide_bones(context)

        return {'FINISHED'}    